/*
*****************************************************************************
**
** \file        sys_services_core.c
** 
** \brief       System service interface.
** 
** This file contains the implementation of the system service API 
** used by an operating environment to provide the services needed by
** the FAPI core drivers.   
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
** (C) Copyright 2008-2010 by Fujitsu Semiconductor Limited.
** 
*****************************************************************************
*/

#include "fapi/sys_services.h"

/*!
*****************************************************************************
** \brief System Service Interface implementation.
** 
** This global datastucture represents the current implementation of the system 
** services. Per \b default services for an OS-less environment are assigned
** as follows:
** - FAPI_SYS_ServicesT::lockSchedulerFunc    <== NULL
** - FAPI_SYS_ServicesT::unlockSchedulerFunc  <== NULL
** - FAPI_SYS_ServicesT::sleepFunc            <== #RTOS_Sleep()
** - FAPI_SYS_ServicesT::disableIrqFunc       <== #RTOS_EnterCritical()
** - FAPI_SYS_ServicesT::enableIrqFunc        <== #RTOS_LeaveCritical()
** - FAPI_SYS_ServicesT::createSemaphoreFunc  <== #RTOS_CreateSemaphore()
** - FAPI_SYS_ServicesT::destroySemaphoreFunc <== #RTOS_DestroySemaphore()
** - FAPI_SYS_ServicesT::getSemaphoreFunc     <== #RTOS_GetSemaphore()
** - FAPI_SYS_ServicesT::setSemaphoreFunc     <== #RTOS_SetSemaphore()
** - FAPI_SYS_ServicesT::createMailqueueFunc  <== #RTOS_CreateMailqueue()
** - FAPI_SYS_ServicesT::destroyMailqueueFunc <== #RTOS_DestroyMailqueue()
** - FAPI_SYS_ServicesT::getMailqueueFunc     <== #RTOS_GetMailqueue()
** - FAPI_SYS_ServicesT::setMailqueueFunc     <== #RTOS_SetMailqueue()
** - FAPI_SYS_ServicesT::getPhysAddressFunc   <== #FAPI_MMU_GetPhysAddr()
** - FAPI_SYS_ServicesT::getVirtAddressFunc   <== #FAPI_MMU_GetVirtAddr()
** - FAPI_SYS_ServicesT::mallocFunc           <== #FAPI_MMU_Malloc()
** - FAPI_SYS_ServicesT::mallocUncachedFunc   <== #FAPI_MMU_Malloc()
** - FAPI_SYS_ServicesT::mallocCachedFunc     <== #FAPI_MMU_Malloc()
** - FAPI_SYS_ServicesT::mallocSegmentFunc    <== #FAPI_MMU_Malloc()
** - FAPI_SYS_ServicesT::freeFunc             <== #FAPI_MMU_Free()
** - FAPI_SYS_ServicesT::printfFunc           <== standard libc \b printf()
** - FAPI_SYS_ServicesT::initDriversFunc
**
** \attention A particular RTOS may overwrite these services with its own 
**            appropriate service implementation.
**
** \note This datastructure is used by all \b service \b access \b macros.
**
** \sa FAPI_SYS_ServicesT
** \sa FAPI_SYS_SetServices
** \sa FAPI_SYS_SetMasterCpu
** \sa FAPI_SYS_LOCK_SCHEDULER
** \sa FAPI_SYS_UNLOCK_SCHEDULER
** \sa FAPI_SYS_SLEEP
** \sa FAPI_SYS_DISABLE_IRQ
** \sa FAPI_SYS_ENABLE_IRQ
** \sa FAPI_SYS_CREATE_SEMAPHORE
** \sa FAPI_SYS_DESTROY_SEMAPHORE
** \sa FAPI_SYS_GET_SEMAPHORE
** \sa FAPI_SYS_SET_SEMAPHORE
** \sa FAPI_SYS_CREATE_MAILQUEUE
** \sa FAPI_SYS_DESTROY_MAILQUEUE
** \sa FAPI_SYS_GET_MAILQUEUE
** \sa FAPI_SYS_SET_MAILQUEUE
** \sa FAPI_SYS_GET_PHYSICAL_ADDRESS
** \sa FAPI_SYS_GET_VIRTUAL_ADDRESS
** \sa FAPI_SYS_MALLOC
** \sa FAPI_SYS_MALLOC_UNCACHED
** \sa FAPI_SYS_MALLOC_CACHED
** \sa FAPI_SYS_MALLOC_SEGMENT
** \sa FAPI_SYS_FREE
** \sa FAPI_SYS_PRINT_MSG
** \sa FAPI_SYS_PRINT_DEBUG
** \sa FAPI_SYS_INIT_DRIVERS
****************************************************************************
*/      
FAPI_SYS_ServicesT FAPI_SYS_Services = {
    /* lockSchedulerFunc    */ NULL,
    /* unlockSchedulerFunc  */ NULL,
    /* sleepFunc            */ NULL,
    /* disableIrqFunc       */ NULL,
    /* enableIrqFunc        */ NULL,
    /* createSemaphoreFunc  */ NULL,
    /* destroySemaphoreFunc */ NULL,
    /* getSemaphoreFunc     */ NULL,
    /* setSemaphoreFunc     */ NULL,
    /* createMailqueueFunc  */ NULL,
    /* destroyMailqueueFunc */ NULL,
    /* getMailqueueFunc     */ NULL,
    /* setMailqueueFunc     */ NULL,
    /* getPhysAddressFunc   */ NULL,
    /* getVirtAddressFunc   */ NULL,
    /* mallocFunc           */ NULL,
    /* mallocUncachedFunc   */ NULL,
    /* mallocCachedFunc     */ NULL,
    /* mallocSegmentFunc    */ NULL,
    /* freeFunc             */ NULL,
    /* printfFunc           */ NULL,
    /* initDriversFunc      */ NULL
};

/*!
*****************************************************************************
** \brief Set the system services.
** 
** This function sets the system services needed by the FAPI core 
** drivers. It has to be called by the operating environment to 
** register its own service implementation before using the FAPI drivers.
**
** \attention The function overwrites the global settings of 
**            \ref FAPI_SYS_Services. Afterwards the service access macros
**            will use these new services.
**  
** \param sysServicesPtr Pointer to the system service data 
**        structure.
*****************************************************************************
*/    
void FAPI_SYS_SetServices( FAPI_SYS_ServicesT* sysServicesPtr )
{
    if ( sysServicesPtr != NULL ) {
        memcpy( &FAPI_SYS_Services, 
                sysServicesPtr, 
                sizeof( FAPI_SYS_ServicesT ));
    }
}
char_t* FAPI_SYS_GetErrorString(int32_t error)
{
    switch(error)
    {
        case 0: return("FAPI_OK");
        case -1 : return("FAPI_ERR_BAD_PARAMETER");
        case -2 : return("FAPI_ERR_OUT_OF_MEMORY");
        case -3 : return("FAPI_ERR_ALREADY_INITIALIZED");
        case -4 : return("FAPI_ERR_NOT_INITIALIZED");
        case -5 : return("FAPI_ERR_FEATURE_NOT_SUPPORTED");
        case -6 : return("FAPI_ERR_TIMEOUT");
        case -7 : return("FAPI_ERR_DEVICE_BUSY");
        case -8 : return("FAPI_ERR_INVALID_HANDLE");
        case -9 : return("FAPI_ERR_SEMAPHORE_CREATE");
        case -10 : return("FAPI_ERR_UNSUPPORTED_VERSION");
        case -11 : return("FAPI_ERR_MAILQUEUE_CREATE");
        case -12 : return("FAPI_ERR_NOT_OPEN");
        case -13 : return("FAPI_ERR_ALREADY_OPEN");
        case -1000 : return("FAPI_ATA_MODULE_BASE");
        case -1001 : return("FAPI_ATA_ERR_BAD_PARAMETER");
        case -1002 : return("FAPI_ATA_ERR_OUT_OF_MEMORY");
        case -1003 : return("FAPI_ATA_ERR_ALREADY_INITIALIZED");
        case -1004 : return("FAPI_ATA_ERR_NOT_INITIALIZED");
        case -1005 : return("FAPI_ATA_ERR_FEATURE_NOT_SUPPORTED");
        case -1006 : return("FAPI_ATA_ERR_TIMEOUT");
        case -1007 : return("FAPI_ATA_ERR_DEVICE_BUSY");
        case -1008 : return("FAPI_ATA_ERR_INVALID_HANDLE");
        case -1009 : return("FAPI_ATA_ERR_SEMAPHORE_CREATE");
        case -1010 : return("FAPI_ATA_ERR_UNSUPPORTED_VERSION");
        case -1101 : return("FAPI_ATA_ERR_DEVICE_NOT_SUPPORTED");
        case -1102 : return("FAPI_ATA_ERR_UPI_CONFIGURATION");
        case -1103 : return("FAPI_ATA_ERR_SET_COMMAND");
        case -1104 : return("FAPI_ATA_ERR_IDENTIFY_DEVICE");
        case -1105 : return("FAPI_ATA_ERR_NOT_ENABLED");
        case -1106 : return("FAPI_ATA_ERR_SECTOR_COUNT");
        case -1107 : return("FAPI_ATA_ERR_LBA_NUMBER");
        case -1108 : return("FAPI_ATA_ERR_COMMAND_FAILED");
        case -1109 : return("FAPI_ATA_ERR_DMA_REQUEST_FAILED");
        case -1110 : return("FAPI_ATA_ERR_DMA_TRANSFER_FAILED");
        case -1111 : return("FAPI_ATA_ERR_PIOMODE_NOT_SUPPORTED");
        case -1112 : return("FAPI_ATA_ERR_DMAMODE_NOT_SUPPORTED");
        case -1113 : return("FAPI_ATA_ERR_DEVICE_IN_STANDBY");
        case -2000 : return("FAPI_AUDEC_MODULE_BASE");
        case -2001 : return("FAPI_AUDEC_ERR_BAD_PARAMETER");
        case -2002 : return("FAPI_AUDEC_ERR_OUT_OF_MEMORY");
        case -2003 : return("FAPI_AUDEC_ERR_ALREADY_INITIALIZED");
        case -2004 : return("FAPI_AUDEC_ERR_NOT_INITIALIZED");
        case -2005 : return("FAPI_AUDEC_ERR_FEATURE_NOT_SUPPORTED");
        case -2006 : return("FAPI_AUDEC_ERR_TIMEOUT");
        case -2007 : return("FAPI_AUDEC_ERR_DEVICE_BUSY");
        case -2008 : return("FAPI_AUDEC_ERR_INVALID_HANDLE");
        case -2009 : return("FAPI_AUDEC_ERR_SEMAPHORE_CREATE");
        case -2010 : return("FAPI_AUDEC_ERR_UNSUPPORTED_VERSION");
        case -2100 : return("FAPI_AUDEC_ERR_AUDIO_PROCESSOR");
        case -2101 : return("FAPI_AUDEC_ERR_HEADER_NOT_FOUND");
        case -2102 : return("FAPI_AUDEC_ERR_ALREADY_RUNNING");
        case -2103 : return("FAPI_AUDEC_ERR_FIRMWARE_NOT_SUPPORTED");
        case -2104 : return("FAPI_AUDEC_ERR_CODEC_NOT_SUPPORTED");
        case -2105 : return("FAPI_AUDEC_ERR_AUDIO_SUSPENDED");
        case -3000 : return("FAPI_AUOUT_MODULE_BASE");
        case -3001 : return("FAPI_AUOUT_ERR_BAD_PARAMETER");
        case -3002 : return("FAPI_AUOUT_ERR_OUT_OF_MEMORY");
        case -3003 : return("FAPI_AUOUT_ERR_ALREADY_INITIALIZED");
        case -3004 : return("FAPI_AUOUT_ERR_NOT_INITIALIZED");
        case -3005 : return("FAPI_AUOUT_ERR_FEATURE_NOT_SUPPORTED");
        case -3006 : return("FAPI_AUOUT_ERR_TIMEOUT");
        case -3007 : return("FAPI_AUOUT_ERR_DEVICE_BUSY");
        case -3008 : return("FAPI_AUOUT_ERR_INVALID_HANDLE");
        case -3009 : return("FAPI_AUOUT_ERR_SEMAPHORE_CREATE");
        case -3010 : return("FAPI_AUOUT_ERR_UNSUPPORTED_VERSION");
        case -3100 : return("FAPI_AUOUT_ERR_HDMI_RUNNING");
        case -3101 : return("FAPI_AUOUT_ERR_MULTIPLE_ASSIGNMENT");
        case -3102 : return("FAPI_AUOUT_ERR_MAX_HANDLES_REACHED");
        case -3103 : return("FAPI_AUOUT_ERR_MAX_FDCLK_REACHED");
        case -3104 : return("FAPI_AUOUT_ERR_INVALID_SAMPLE_FREQ");
        case -3105 : return("FAPI_AUOUT_WAR_NEED_RESTART");
        case -4000 : return("FAPI_BOOT_MODULE_BASE");
        case -4001 : return("FAPI_BOOT_ERR_BAD_PARAMETER");
        case -4002 : return("FAPI_BOOT_ERR_OUT_OF_MEMORY");
        case -4003 : return("FAPI_BOOT_ERR_ALREADY_INITIALIZED");
        case -4004 : return("FAPI_BOOT_ERR_NOT_INITIALIZED");
        case -4005 : return("FAPI_BOOT_ERR_FEATURE_NOT_SUPPORTED");
        case -4008 : return("FAPI_BOOT_ERR_INVALID_HANDLE");
        case -4009 : return("FAPI_BOOT_ERR_SEMAPHORE_CREATE");
        case -4010 : return("FAPI_BOOT_ERR_UNSUPPORTED_VERSION");
        case -4101 : return("FAPI_BOOT_ERR_DEVICE_UNKNOWN");
        case -4102 : return("FAPI_BOOT_ERR_MODE_UNKNOWN");
        case -5000 : return("FAPI_BM_MODULE_BASE");
        case -5001 : return("FAPI_BM_ERR_BAD_PARAMETER");
        case -5002 : return("FAPI_BM_ERR_OUT_OF_MEMORY");
        case -5003 : return("FAPI_BM_ERR_ALREADY_INITIALIZED");
        case -5004 : return("FAPI_BM_ERR_NOT_INITIALIZED");
        case -5005 : return("FAPI_BM_ERR_FEATURE_NOT_SUPPORTED");
        case -5006 : return("FAPI_BM_ERR_TIMEOUT");
        case -5007 : return("FAPI_BM_ERR_DEVICE_BUSY");
        case -5008 : return("FAPI_BM_ERR_INVALID_HANDLE");
        case -5009 : return("FAPI_BM_ERR_SEMAPHORE_CREATE");
        case -5010 : return("FAPI_BM_ERR_UNSUPPORTED_VERSION");
        case -5011 : return("FAPI_BM_ERR_MAILQUEUE_CREATE");
        case -5100 : return("FAPI_BM_ERR_PTR_OUTSIDE_BUFFER_BOUNDS");
        case -5101 : return("FAPI_BM_ERR_NO_MORE_HWCB_AVAILABLE");
        case -5102 : return("FAPI_BM_ERR_HWCB_USED_BY_OTHER_BM");
        case -5103 : return("FAPI_BM_ERR_NO_HWCB");
        case -5104 : return("FAPI_BM_ERR_AUD_NOT_ASSIGNED");
        case -5105 : return("FAPI_BM_ERR_POINTER_OUT_OF_BOUNDS");
        case -5106 : return("FAPI_BM_ERR_PURPOSE_ALREADY_ASSIGNED");
        case -5107 : return("FAPI_BM_ERR_INVALID_PURPOSE");
        case -5108 : return("FAPI_BM_ERR_PURPOSE_NOT_ASSIGNED");
        case -5109 : return("FAPI_BM_ERR_HIGH_LEVEL_IRQ_NOT_ASSIGNED");
        case -5110 : return("FAPI_BM_ERR_LOW_LEVEL_IRQ_NOT_ASSIGNED");
        case -5111 : return("FAPI_BM_ERR_AUD_DETECTOR_COUNT_EXCEEDED");
        case -6000 : return("FAPI_CLKPWR_MODULE_BASE");
        case -6001 : return("FAPI_CLKPWR_ERR_BAD_PARAMETER");
        case -6002 : return("FAPI_CLKPWR_ERR_OUT_OF_MEMORY");
        case -6003 : return("FAPI_CLKPWR_ERR_ALREADY_INITIALIZED");
        case -6004 : return("FAPI_CLKPWR_ERR_NOT_INITIALIZED");
        case -6005 : return("FAPI_CLKPWR_ERR_FEATURE_NOT_SUPPORTED");
        case -6006 : return("FAPI_CLKPWR_ERR_TIMEOUT");
        case -6007 : return("FAPI_CLKPWR_ERR_DEVICE_BUSY");
        case -6008 : return("FAPI_CLKPWR_ERR_INVALID_HANDLE");
        case -6009 : return("FAPI_CLKPWR_ERR_SEMAPHORE_CREATE");
        case -6010 : return("FAPI_CLKPWR_ERR_UNSUPPORTED_VERSION");
        case -6011 : return("FAPI_CLKPWR_ERR_MAILQUEUE_CREATE");
        case -6100 : return("FAPI_CLKPWR_ERR_INVALID_START_VALUE");
        case -7000 : return("FAPI_DESC_MODULE_BASE");
        case -7001 : return("FAPI_DESC_ERR_BAD_PARAMETER");
        case -7002 : return("FAPI_DESC_ERR_OUT_OF_MEMORY");
        case -7003 : return("FAPI_DESC_ERR_ALREADY_INITIALIZED");
        case -7004 : return("FAPI_DESC_ERR_NOT_INITIALIZED");
        case -7005 : return("FAPI_DESC_ERR_FEATURE_NOT_SUPPORTED");
        case -7006 : return("FAPI_DESC_ERR_TIMEOUT");
        case -7007 : return("FAPI_DESC_ERR_DEVICE_BUSY");
        case -7008 : return("FAPI_DESC_ERR_INVALID_HANDLE");
        case -7009 : return("FAPI_DESC_ERR_SEMAPHORE_CREATE");
        case -7010 : return("FAPI_DESC_ERR_UNSUPPORTED_VERSION");
        case -7011 : return("FAPI_DESC_ERR_MAILQUEUE_CREATE");
        case -7100 : return("FAPI_DESC_ERR_INVALID_STATUS");
        case -7101 : return("FAPI_DESC_ERR_INVALID_KEY_LENGTH");
        case -8000 : return("FAPI_DMA_MODULE_BASE");
        case -8001 : return("FAPI_DMA_ERR_BAD_PARAMETER");
        case -8002 : return("FAPI_DMA_ERR_OUT_OF_MEMORY");
        case -8003 : return("FAPI_DMA_ERR_ALREADY_INITIALIZED");
        case -8004 : return("FAPI_DMA_ERR_NOT_INITIALIZED");
        case -8005 : return("FAPI_DMA_ERR_FEATURE_NOT_SUPPORTED");
        case -8006 : return("FAPI_DMA_ERR_TIMEOUT");
        case -8007 : return("FAPI_DMA_ERR_DEVICE_BUSY");
        case -8008 : return("FAPI_DMA_ERR_INVALID_HANDLE");
        case -8009 : return("FAPI_DMA_ERR_SEMAPHORE_CREATE");
        case -8010 : return("FAPI_DMA_ERR_UNSUPPORTED_VERSION");
        case -8101 : return("FAPI_DMA_ERR_INIT_FAILED");
        case -8102 : return("FAPI_DMA_ERR_TOO_MANY_OPEN");
        case -8103 : return("FAPI_DMA_ERR_CONFIG_OVERFLOW");
        case -8104 : return("FAPI_DMA_ERR_FIFO_OVERFLOW");
        case -8105 : return("FAPI_DMA_ERR_BAD_FEATURE");
        case -8106 : return("FAPI_DMA_ERR_BAD_REQUEST");
        case -8107 : return("FAPI_DMA_ERR_STATUS");
        case -8108 : return("FAPI_DMA_ERR_CCIR_OVERFLOW");
        case -8109 : return("FAPI_DMA_ERR_AUDIO_OVERFLOW");
        case -8110 : return("FAPI_DMA_ERR_CANCELED");
        case -8111 : return("FAPI_DMA_ERR_CCIR_BUFFER_TO_SMALL");
        case -8112 : return("FAPI_DMA_ERR_AUDIO_BUFFER_TO_SMALL");
        case -9000 : return("FAPI_ETH_MODULE_BASE");
        case -9001 : return("FAPI_ETH_ERR_BAD_PARAMETER");
        case -9002 : return("FAPI_ETH_ERR_OUT_OF_MEMORY");
        case -9003 : return("FAPI_ETH_ERR_ALREADY_INITIALIZED");
        case -9004 : return("FAPI_ETH_ERR_NOT_INITIALIZED");
        case -9005 : return("FAPI_ETH_ERR_FEATURE_NOT_SUPPORTED");
        case -9006 : return("FAPI_ETH_ERR_TIMEOUT");
        case -9007 : return("FAPI_ETH_ERR_DEVICE_BUSY");
        case -9008 : return("FAPI_ETH_ERR_INVALID_HANDLE");
        case -9009 : return("FAPI_ETH_ERR_SEMAPHORE_CREATE");
        case -9010 : return("FAPI_ETH_ERR_UNSUPPORTED_VERSION");
        case -9011 : return("FAPI_ETH_ERR_MAILQUEUE_CREATE");
        case -9100 : return("FAPI_ETH_ERR_NODEV");
        case -9101 : return("FAPI_ETH_ERR_DATACCESS");
        case -9102 : return("FAPI_ETH_ERR_RXSTOP");
        case -9103 : return("FAPI_ETH_ERR_TXSTOP");
        case -9104 : return("FAPI_ETH_ERR_NULL_PTR");
        case -9105 : return("FAPI_ETH_ERR_FATALSTOP");
        case -9106 : return("FAPI_ETH_ERR_TOO_BIG_FRAME");
        case -9107 : return("FAPI_ETH_ERR_TOO_SHORT_FRAME");
        case -9108 : return("FAPI_ETH_ERR_AGAIN");
        case -9109 : return("FAPI_ETH_ERR_LOCK_FAIL");
        case -9110 : return("FAPI_ETH_ERR_INV_ADDR");
        case -9111 : return("FAPI_ETH_ERR_PHY_BUSY");
        case -9112 : return("FAPI_ETH_ERR_UNKNOWN");
        case -9113 : return("FAPI_ETH_ERR_LINKDOWN");
        case -10000 : return("FAPI_FLASH_MODULE_BASE");
        case -10001 : return("FAPI_FLASH_ERR_BAD_PARAMETER");
        case -10002 : return("FAPI_FLASH_ERR_OUT_OF_MEMORY");
        case -10003 : return("FAPI_FLASH_ERR_ALREADY_INITIALIZED");
        case -10004 : return("FAPI_FLASH_ERR_NOT_INITIALIZED");
        case -10005 : return("FAPI_FLASH_ERR_FEATURE_NOT_SUPPORTED");
        case -10006 : return("FAPI_FLASH_ERR_TIMEOUT");
        case -10007 : return("FAPI_FLASH_ERR_DEVICE_BUSY");
        case -10008 : return("FAPI_FLASH_ERR_INVALID_HANDLE");
        case -10009 : return("FAPI_FLASH_ERR_SEMAPHORE_CREATE");
        case -10010 : return("FAPI_FLASH_ERR_UNSUPPORTED_VERSION");
        case -10011 : return("FAPI_FLASH_ERR_MAILQUEUE_CREATE");
        case -10101 : return("FAPI_FLASH_ERR_DEVICE_NOT_SUPPORTED");
        case -10102 : return("FAPI_FLASH_ERR_SECTOR_PROTECTED");
        case -10103 : return("FAPI_FLASH_ERR_NO_MAGIC_ID_FOUND");
        case -10104 : return("FAPI_FLASH_ERR_PROCESS_FAILED");
        case -10105 : return("FAPI_FLASH_ERR_PROTECTED");
        case -10106 : return("FAPI_FLASH_ERR_INVALID_FUNCTION");
        case -10107 : return("FAPI_FLASH_ERR_INVALID_DEVICE");
        case -10108 : return("FAPI_FLASH_ERR_EXT_ADDRESS_FAILED");
        case -10109 : return("FAPI_FLASH_ERR_OVERLAP_DETECTED");
        case -10110 : return("FAPI_FLASH_ERR_BADBLOCK_DETECTION_FAILED");
        case -10111 : return("FAPI_FLASH_ERR_ECC_CHECK_FAILED");
        case -11000 : return("FAPI_FPC_MODULE_BASE");
        case -11001 : return("FAPI_FPC_ERR_BAD_PARAMETER");
        case -11002 : return("FAPI_FPC_ERR_OUT_OF_MEMORY");
        case -11003 : return("FAPI_FPC_ERR_ALREADY_INITIALIZED");
        case -11004 : return("FAPI_FPC_ERR_NOT_INITIALIZED");
        case -11005 : return("FAPI_FPC_ERR_FEATURE_NOT_SUPPORTED");
        case -11006 : return("FAPI_FPC_ERR_TIMEOUT");
        case -11007 : return("FAPI_FPC_ERR_DEVICE_BUSY");
        case -11008 : return("FAPI_FPC_ERR_INVALID_HANDLE");
        case -11009 : return("FAPI_FPC_ERR_SEMAPHORE_CREATE");
        case -11010 : return("FAPI_FPC_ERR_UNSUPPORTED_VERSION");
        case -11100 : return("FAPI_FPC_ERR_MODE_NOT_SUPPORTED");
        case -11101 : return("FAPI_FPC_DIGIT_CARRY");
        case -11102 : return("FAPI_FPC_BOUNDS_REACHED");
        case -11103 : return("FAPI_FPC_MAX_HANDLE_COUNT");
        case -12000 : return("FAPI_GPIO_MODULE_BASE");
        case -12001 : return("FAPI_GPIO_ERR_BAD_PARAMETER");
        case -12002 : return("FAPI_GPIO_ERR_OUT_OF_MEMORY");
        case -12003 : return("FAPI_GPIO_ERR_ALREADY_INITIALIZED");
        case -12004 : return("FAPI_GPIO_ERR_NOT_INITIALIZED");
        case -12005 : return("FAPI_GPIO_ERR_FEATURE_NOT_SUPPORTED");
        case -12006 : return("FAPI_GPIO_ERR_TIMEOUT");
        case -12007 : return("FAPI_GPIO_ERR_DEVICE_BUSY");
        case -12008 : return("FAPI_GPIO_ERR_INVALID_HANDLE");
        case -12009 : return("FAPI_GPIO_ERR_SEMAPHORE_CREATE");
        case -12010 : return("FAPI_GPIO_ERR_UNSUPPORTED_VERSION");
        case -12011 : return("FAPI_GPIO_ERR_MAILQUEUE_CREATE");
        case -12101 : return("FAPI_GPIO_ERR_PIN_NOT_FOUND");
        case -13000 : return("FAPI_GPREG_MODULE_BASE");
        case -13001 : return("FAPI_GPREG_ERR_BAD_PARAMETER");
        case -13002 : return("FAPI_GPREG_ERR_OUT_OF_MEMORY");
        case -13003 : return("FAPI_GPREG_ERR_ALREADY_INITIALIZED");
        case -13004 : return("FAPI_GPREG_ERR_NOT_INITIALIZED");
        case -13005 : return("FAPI_GPREG_ERR_FEATURE_NOT_SUPPORTED");
        case -13006 : return("FAPI_GPREG_ERR_TIMEOUT");
        case -13007 : return("FAPI_GPREG_ERR_DEVICE_BUSY");
        case -13008 : return("FAPI_GPREG_ERR_INVALID_HANDLE");
        case -13009 : return("FAPI_GPREG_ERR_SEMAPHORE_CREATE");
        case -13010 : return("FAPI_GPREG_ERR_UNSUPPORTED_VERSION");
        case -14000 : return("FAPI_HDMI_MODULE_BASE");
        case -14001 : return("FAPI_HDMI_ERR_BAD_PARAMETER");
        case -14002 : return("FAPI_HDMI_ERR_OUT_OF_MEMORY");
        case -14003 : return("FAPI_HDMI_ERR_ALREADY_INITIALIZED");
        case -14004 : return("FAPI_HDMI_ERR_NOT_INITIALIZED");
        case -14005 : return("FAPI_HDMI_ERR_FEATURE_NOT_SUPPORTED");
        case -14006 : return("FAPI_HDMI_ERR_TIMEOUT");
        case -14007 : return("FAPI_HDMI_ERR_DEVICE_BUSY");
        case -14008 : return("FAPI_HDMI_ERR_INVALID_HANDLE");
        case -14009 : return("FAPI_HDMI_ERR_SEMAPHORE_CREATE");
        case -14010 : return("FAPI_HDMI_ERR_UNSUPPORTED_VERSION");
        case -14100 : return("FAPI_HDMI_ERR_OPEN_PARAM");
        case -14101 : return("FAPI_HDMI_ERR_PARAMETER");
        case -14110 : return("FAPI_HDMI_ERR_I2C_OPEN");
        case -14120 : return("FAPI_HDMI_ERR_EDID");
        case -14121 : return("FAPI_HDMI_ERR_DDC_TIMEOUT");
        case -14130 : return("FAPI_HDMI_ERR_HDCP_TIMEOUT");
        case -14131 : return("FAPI_HDMI_ERR_HDCP_MAX_DEVS_EXCEED");
        case -14132 : return("FAPI_HDMI_ERR_HDCP_MAX_CASCADE_EXCEEDED");
        case -14133 : return("FAPI_HDMI_ERR_HDCP_SHA_1_MISMATCH");
        case -14134 : return("FAPI_HDMI_ERR_HDCP_AN_DDC");
        case -14135 : return("FAPI_HDMI_ERR_HDCP_AKSV_DDC");
        case -14136 : return("FAPI_HDMI_ERR_HDCP_BKSV");
        case -14137 : return("FAPI_HDMI_ERR_HDCP_RI_FAILURE");
        case -14138 : return("FAPI_HDMI_ERR_HDCP_RI_NOT_ABAILABLE");
        case -14140 : return("FAPI_HDMI_ERR_HPD_NOT_DETECT");
        case -14141 : return("FAPI_HDMI_ERR_EXIST_IN_REVOCATION_LISTS");
        case -14142 : return("FAPI_HDMI_ERR_HDCP_NOT_SUPPORT");
        case -14143 : return("FAPI_HDMI_ERR_HDCP_AKSV");
        case -14151 : return("FAPI_HDMI_ERR_CEC_BUSY");
        case -14152 : return("FAPI_HDMI_ERR_CEC_SENDING");
        case -14153 : return("FAPI_HDMI_ERR_CEC_GPIO");
        case -14154 : return("FAPI_HDMI_ERR_CEC_MESSAGE_BUFFER_FULL");
        case -14155 : return("FAPI_HDMI_ERR_CEC_MESSAGE_BUFFER_EMPTY");
        case -14156 : return("FAPI_HDMI_ERR_CEC_RECEIVE_PARAMETER");
        case -14157 : return("FAPI_HDMI_ERR_CEC_SEND_PARAMETER");
        case -14158 : return("FAPI_HDMI_ERR_CEC_TIMER");
        case -14159 : return("FAPI_HDMI_ERR_CEC_OPEN");
        case -15000 : return("FAPI_I2C_MODULE_BASE");
        case -15001 : return("FAPI_I2C_ERR_BAD_PARAMETER");
        case -15002 : return("FAPI_I2C_ERR_OUT_OF_MEMORY");
        case -15003 : return("FAPI_I2C_ERR_ALREADY_INITIALIZED");
        case -15004 : return("FAPI_I2C_ERR_NOT_INITIALIZED");
        case -15005 : return("FAPI_I2C_ERR_FEATURE_NOT_SUPPORTED");
        case -15006 : return("FAPI_I2C_ERR_TIMEOUT");
        case -15007 : return("FAPI_I2C_ERR_DEVICE_BUSY");
        case -15008 : return("FAPI_I2C_ERR_INVALID_HANDLE");
        case -15009 : return("FAPI_I2C_ERR_SEMAPHORE_CREATE");
        case -15010 : return("FAPI_I2C_ERR_UNSUPPORTED_VERSION");
        case -15011 : return("FAPI_I2C_ERR_MAILQUEUE_CREATE");
        case -15100 : return("FAPI_I2C_ERR_BUSY");
        case -15101 : return("FAPI_I2C_ERR_BUS_OCCUP");
        case -15102 : return("FAPI_I2C_ERR_SL_STOP_REC");
        case -15103 : return("FAPI_I2C_ERR_RC_NACK");
        case -15104 : return("FAPI_I2C_ERR_BE_STOP");
        case -15105 : return("FAPI_I2C_ERR_BE_START");
        case -15106 : return("FAPI_I2C_ERR_AL_ADRSTA");
        case -15107 : return("FAPI_I2C_ERR_AL_DATA");
        case -15108 : return("FAPI_I2C_ERR_AL_STOP");
        case -15109 : return("FAPI_I2C_ERR_SL_NACK");
        case -15110 : return("FAPI_I2C_ERR_MS_STOP_REC");
        case -15111 : return("FAPI_I2C_ERR_AMDMA");
        case -15112 : return("FAPI_I2C_ERR_UNSUPPORTED");
        case -15113 : return("FAPI_I2C_ERR_GPIO_FAILED");
        case -15114 : return("FAPI_I2C_ERR_TIMER_FAILED");
        case -16000 : return("FAPI_ICC_MODULE_BASE");
        case -16001 : return("FAPI_ICC_ERR_BAD_PARAMETER");
        case -16002 : return("FAPI_ICC_ERR_OUT_OF_MEMORY");
        case -16003 : return("FAPI_ICC_ERR_ALREADY_INITIALIZED");
        case -16004 : return("FAPI_ICC_ERR_NOT_INITIALIZED");
        case -16005 : return("FAPI_ICC_ERR_FEATURE_NOT_SUPPORTED");
        case -16006 : return("FAPI_ICC_ERR_TIMEOUT");
        case -16007 : return("FAPI_ICC_ERR_DEVICE_BUSY");
        case -16008 : return("FAPI_ICC_ERR_INVALID_HANDLE");
        case -16009 : return("FAPI_ICC_ERR_SEMAPHORE_CREATE");
        case -16010 : return("FAPI_ICC_ERR_UNSUPPORTED_VERSION");
        case -16100 : return("FAPI_ICC_ERR_SLOT_EMPTY");
        case -16101 : return("FAPI_ICC_ERR_RUNNING");
        case -16102 : return("FAPI_ICC_ERR_OVERFLOW");
        case -16103 : return("FAPI_ICC_ERR_NO_ANSWER");
        case -16104 : return("FAPI_ICC_ERR_NO_BAUDRATE");
        case -16105 : return("FAPI_ICC_ERR_NO_CARDCLOCK");
        case -16106 : return("FAPI_ICC_ERR_NO_PROTOCOL");
        case -16107 : return("FAPI_ICC_ERR_WRITE_FAIL");
        case -16108 : return("FAPI_ICC_ERR_READ_FAIL");
        case -16109 : return("FAPI_ICC_ERR_BUFFER_OVERFLOW");
        case -16110 : return("FAPI_ICC_ERR_PROTOCOL_FAIL");
        case -16111 : return("FAPI_ICC_ERR_UNKNOWN");
        case -17000 : return("FAPI_INTR_MODULE_BASE");
        case -17001 : return("FAPI_INTR_ERR_BAD_PARAMETER");
        case -17002 : return("FAPI_INTR_ERR_OUT_OF_MEMORY");
        case -17003 : return("FAPI_INTR_ERR_ALREADY_INITIALIZED");
        case -17004 : return("FAPI_INTR_ERR_NOT_INITIALIZED");
        case -17005 : return("FAPI_INTR_ERR_FEATURE_NOT_SUPPORTED");
        case -17006 : return("FAPI_INTR_ERR_TIMEOUT");
        case -17007 : return("FAPI_INTR_ERR_DEVICE_BUSY");
        case -17008 : return("FAPI_INTR_ERR_INVALID_HANDLE");
        case -17009 : return("FAPI_INTR_ERR_SEMAPHORE_CREATE");
        case -17010 : return("FAPI_INTR_ERR_UNSUPPORTED_VERSION");
        case -18000 : return("FAPI_IR_MODULE_BASE");
        case -18001 : return("FAPI_IR_ERR_BAD_PARAMETER");
        case -18002 : return("FAPI_IR_ERR_OUT_OF_MEMORY");
        case -18003 : return("FAPI_IR_ERR_ALREADY_INITIALIZED");
        case -18004 : return("FAPI_IR_ERR_NOT_INITIALIZED");
        case -18005 : return("FAPI_IR_ERR_FEATURE_NOT_SUPPORTED");
        case -18006 : return("FAPI_IR_ERR_TIMEOUT");
        case -18007 : return("FAPI_IR_ERR_DEVICE_BUSY");
        case -18008 : return("FAPI_IR_ERR_INVALID_HANDLE");
        case -18009 : return("FAPI_IR_ERR_SEMAPHORE_CREATE");
        case -18010 : return("FAPI_IR_ERR_UNSUPPORTED_VERSION");
        case -18100 : return("FAPI_IR_ERR_NO_SEQUENCE");
        case -18101 : return("FAPI_IR_ERR_INVAL_SEQ");
        case -18102 : return("FAPI_IR_ERR_UNSUPPORTED_MODE");
        case -18103 : return("FAPI_IR_ERR_HARDWARE_MODE");
        case -18104 : return("FAPI_IR_ERR_PARAM_OUTOFRANGE");
        case -19000 : return("FAPI_MMU_MODULE_BASE");
        case -19001 : return("FAPI_MMU_ERR_BAD_PARAMETER");
        case -19002 : return("FAPI_MMU_ERR_OUT_OF_MEMORY");
        case -19003 : return("FAPI_MMU_ERR_ALREADY_INITIALIZED");
        case -19004 : return("FAPI_MMU_ERR_NOT_INITIALIZED");
        case -19005 : return("FAPI_MMU_ERR_FEATURE_NOT_SUPPORTED");
        case -19008 : return("FAPI_MMU_ERR_INVALID_HANDLE");
        case -19009 : return("FAPI_MMU_ERR_SEMAPHORE_CREATE");
        case -19010 : return("FAPI_MMU_ERR_UNSUPPORTED_VERSION");
        case -19101 : return("FAPI_MMU_ERR_SEGMENT_CREATE");
        case -20000 : return("FAPI_SSP_MODULE_BASE");
        case -20001 : return("FAPI_SSP_ERR_BAD_PARAMETER");
        case -20002 : return("FAPI_SSP_ERR_OUT_OF_MEMORY");
        case -20003 : return("FAPI_SSP_ERR_ALREADY_INITIALIZED");
        case -20004 : return("FAPI_SSP_ERR_NOT_INITIALIZED");
        case -20005 : return("FAPI_SSP_ERR_FEATURE_NOT_SUPPORTED");
        case -20006 : return("FAPI_SSP_ERR_TIMEOUT");
        case -20007 : return("FAPI_SSP_ERR_DEVICE_BUSY");
        case -20008 : return("FAPI_SSP_ERR_INVALID_HANDLE");
        case -20009 : return("FAPI_SSP_ERR_SEMAPHORE_CREATE");
        case -20010 : return("FAPI_SSP_ERR_UNSUPPORTED_VERSION");
        case -20100 : return("FAPI_SSP_ERR_BUSY");
        case -20101 : return("FAPI_SSP_ERR_NO_DATA");
        case -21000 : return("FAPI_SYNC_MODULE_BASE");
        case -21001 : return("FAPI_SYNC_ERR_BAD_PARAMETER");
        case -21002 : return("FAPI_SYNC_ERR_OUT_OF_MEMORY");
        case -21003 : return("FAPI_SYNC_ERR_ALREADY_INITIALIZED");
        case -21004 : return("FAPI_SYNC_ERR_NOT_INITIALIZED");
        case -21005 : return("FAPI_SYNC_ERR_FEATURE_NOT_SUPPORTED");
        case -21006 : return("FAPI_SYNC_ERR_TIMEOUT");
        case -21007 : return("FAPI_SYNC_ERR_DEVICE_BUSY");
        case -21008 : return("FAPI_SYNC_ERR_INVALID_HANDLE");
        case -21009 : return("FAPI_SYNC_ERR_SEMAPHORE_CREATE");
        case -21010 : return("FAPI_SYNC_ERR_UNSUPPORTED_VERSION");
        case -21011 : return("FAPI_SYNC_ERR_MAILQUEUE_CREATE");
        case -21100 : return("FAPI_SYNC_ERR_PCR_NOT_AVAILABLE");
        case -21101 : return("FAPI_SYNC_ERR_STC_NOT_AVAILABLE");
        case -21102 : return("FAPI_SYNC_ERR_PCR_PID_FAILED");
        case -21103 : return("FAPI_SYNC_ERR_INVALID_STATE");
        case -21104 : return("FAPI_SYNC_ERR_INVALID_DECODER_OFFSET");
        case -22000 : return("FAPI_TIMER_MODULE_BASE");
        case -22001 : return("FAPI_TIMER_ERR_BAD_PARAMETER");
        case -22002 : return("FAPI_TIMER_ERR_OUT_OF_MEMORY");
        case -22003 : return("FAPI_TIMER_ERR_ALREADY_INITIALIZED");
        case -22004 : return("FAPI_TIMER_ERR_NOT_INITIALIZED");
        case -22005 : return("FAPI_TIMER_ERR_FEATURE_NOT_SUPPORTED");
        case -22006 : return("FAPI_TIMER_ERR_TIMEOUT");
        case -22007 : return("FAPI_TIMER_ERR_DEVICE_BUSY");
        case -22008 : return("FAPI_TIMER_ERR_INVALID_HANDLE");
        case -22009 : return("FAPI_TIMER_ERR_SEMAPHORE_CREATE");
        case -22010 : return("FAPI_TIMER_ERR_UNSUPPORTED_VERSION");
        case -22011 : return("FAPI_TIMER_ERR_MAILQUEUE_CREATE");
        case -23000 : return("FAPI_TSD_MODULE_BASE");
        case -23001 : return("FAPI_TSD_ERR_BAD_PARAMETER");
        case -23002 : return("FAPI_TSD_ERR_OUT_OF_MEMORY");
        case -23003 : return("FAPI_TSD_ERR_ALREADY_INITIALIZED");
        case -23004 : return("FAPI_TSD_ERR_NOT_INITIALIZED");
        case -23005 : return("FAPI_TSD_ERR_FEATURE_NOT_SUPPORTED");
        case -23006 : return("FAPI_TSD_ERR_TIMEOUT");
        case -23007 : return("FAPI_TSD_ERR_DEVICE_BUSY");
        case -23008 : return("FAPI_TSD_ERR_INVALID_HANDLE");
        case -23009 : return("FAPI_TSD_ERR_SEMAPHORE_CREATE");
        case -23010 : return("FAPI_TSD_ERR_UNSUPPORTED_VERSION");
        case -23011 : return("FAPI_TSD_ERR_MAILQUEUE_CREATE");
        case -23100 : return("FAPI_TSD_ERR_INVALID_PID");
        case -23101 : return("FAPI_TSD_ERR_NO_PID_ASSIGNED");
        case -23102 : return("FAPI_TSD_ERR_PES_PARSER_COUNT_EXEEDED");
        case -23103 : return("FAPI_TSD_ERR_NO_RECONFIGURE");
        case -23104 : return("FAPI_TSD_ERR_INVALID_FILTER_SETTINGS");
        case -23105 : return("FAPI_TSD_ERR_PID_ALREADY_ASSIGNED");
        case -23106 : return("FAPI_TSD_ERR_NO_MAPPED_BUFFER");
        case -23107 : return("FAPI_TSD_ERR_NOTIFIER_ALREADY_ASSIGNED");
        case -23108 : return("FAPI_TSD_ERR_GPIO_FAILED");
        case -23109 : return("FAPI_TSD_ERR_NO_FILTER_ASSIGNED");
        case -23110 : return("FAPI_TSD_ERR_NO_PES_PARSER_ASSIGNED");
        case -23111 : return("FAPI_TSD_ERR_SYNC_DISABLED");
        case -23112 : return("FAPI_TSD_ERR_STC_SELECT");
        case -23113 : return("FAPI_TSD_ERR_PID_ACTIVE");
        case -23114 : return("FAPI_TSD_ERR_EVALUATOR_ALREADY_ASSIGNED");
        case -24000 : return("FAPI_TTX_MODULE_BASE");
        case -24001 : return("FAPI_TTX_ERR_BAD_PARAMETER");
        case -24002 : return("FAPI_TTX_ERR_OUT_OF_MEMORY");
        case -24003 : return("FAPI_TTX_ERR_ALREADY_INITIALIZED");
        case -24004 : return("FAPI_TTX_ERR_NOT_INITIALIZED");
        case -24005 : return("FAPI_TTX_ERR_FEATURE_NOT_SUPPORTED");
        case -24006 : return("FAPI_TTX_ERR_TIMEOUT");
        case -24007 : return("FAPI_TTX_ERR_DEVICE_BUSY");
        case -24008 : return("FAPI_TTX_ERR_INVALID_HANDLE");
        case -24009 : return("FAPI_TTX_ERR_SEMAPHORE_CREATE");
        case -24010 : return("FAPI_TTX_ERR_UNSUPPORTED_VERSION");
        case -24011 : return("FAPI_TTX_ERR_MAILQUEUE_CREATE");
        case -24100 : return("FAPI_TTX_ERR_NO_PID_ASSIGNED");
        case -24101 : return("FAPI_TTX_ERR_NOT_SYNCHRONISED");
        case -24102 : return("FAPI_TTX_ERR_BM_ERROR");
        case -25000 : return("FAPI_UART_MODULE_BASE");
        case -25001 : return("FAPI_UART_ERR_BAD_PARAMETER");
        case -25002 : return("FAPI_UART_ERR_OUT_OF_MEMORY");
        case -25003 : return("FAPI_UART_ERR_ALREADY_INITIALIZED");
        case -25004 : return("FAPI_UART_ERR_NOT_INITIALIZED");
        case -25005 : return("FAPI_UART_ERR_FEATURE_NOT_SUPPORTED");
        case -25006 : return("FAPI_UART_ERR_TIMEOUT");
        case -25007 : return("FAPI_UART_ERR_DEVICE_BUSY");
        case -25008 : return("FAPI_UART_ERR_INVALID_HANDLE");
        case -25009 : return("FAPI_UART_ERR_SEMAPHORE_CREATE");
        case -25010 : return("FAPI_UART_ERR_UNSUPPORTED_VERSION");
        case -25011 : return("FAPI_UART_ERR_MAILQUEUE_CREATE");
        case -25101 : return("FAPI_UART_ERR_FIFO_OVERFLOW");
        case -26000 : return("FAPI_UPI_MODULE_BASE");
        case -26001 : return("FAPI_UPI_ERR_BAD_PARAMETER");
        case -26002 : return("FAPI_UPI_ERR_OUT_OF_MEMORY");
        case -26003 : return("FAPI_UPI_ERR_ALREADY_INITIALIZED");
        case -26004 : return("FAPI_UPI_ERR_NOT_INITIALIZED");
        case -26005 : return("FAPI_UPI_ERR_FEATURE_NOT_SUPPORTED");
        case -26006 : return("FAPI_UPI_ERR_TIMEOUT");
        case -26007 : return("FAPI_UPI_ERR_DEVICE_BUSY");
        case -26008 : return("FAPI_UPI_ERR_INVALID_HANDLE");
        case -26009 : return("FAPI_UPI_ERR_SEMAPHORE_CREATE");
        case -26010 : return("FAPI_UPI_ERR_UNSUPPORTED_VERSION");
        case -27000 : return("FAPI_USB_MODULE_BASE");
        case -27001 : return("FAPI_USB_ERR_BAD_PARAMETER");
        case -27002 : return("FAPI_USB_ERR_OUT_OF_MEMORY");
        case -27003 : return("FAPI_USB_ERR_ALREADY_INITIALIZED");
        case -27004 : return("FAPI_USB_ERR_NOT_INITIALIZED");
        case -27005 : return("FAPI_USB_ERR_FEATURE_NOT_SUPPORTED");
        case -27006 : return("FAPI_USB_ERR_TIMEOUT");
        case -27007 : return("FAPI_USB_ERR_DEVICE_BUSY");
        case -27008 : return("FAPI_USB_ERR_INVALID_HANDLE");
        case -27009 : return("FAPI_USB_ERR_SEMAPHORE_CREATE");
        case -27010 : return("FAPI_USB_ERR_UNSUPPORTED_VERSION");
        case -27100 : return("FAPI_USB_ERR_WRONG_ID");
        case -27101 : return("FAPI_USB_ERR_ALREADY_OPEN");
        case -27102 : return("FAPI_USB_ERR_TOO_MANY_DRIVERS");
        case -27103 : return("FAPI_USB_ERR_ALREADY_RUNNING");
        case -27104 : return("FAPI_USB_ERR_NOT_RUNNING");
        case -27105 : return("FAPI_USB_ERR_MUSB_START");
        case -27106 : return("FAPI_USB_ERR_WRONG_MODE");
        case -28000 : return("FAPI_VIDEC_MODULE_BASE");
        case -28001 : return("FAPI_VIDEC_ERR_BAD_PARAMETER");
        case -28002 : return("FAPI_VIDEC_ERR_OUT_OF_MEMORY");
        case -28003 : return("FAPI_VIDEC_ERR_ALREADY_INITIALIZED");
        case -28004 : return("FAPI_VIDEC_ERR_NOT_INITIALIZED");
        case -28005 : return("FAPI_VIDEC_ERR_FEATURE_NOT_SUPPORTED");
        case -28006 : return("FAPI_VIDEC_ERR_TIMEOUT");
        case -28007 : return("FAPI_VIDEC_ERR_DEVICE_BUSY");
        case -28008 : return("FAPI_VIDEC_ERR_INVALID_HANDLE");
        case -28009 : return("FAPI_VIDEC_ERR_SEMAPHORE_CREATE");
        case -28010 : return("FAPI_VIDEC_ERR_UNSUPPORTED_VERSION");
        case -28101 : return("FAPI_VIDEC_ERR_DRV_VERSION_CHECK_FAILED");
        case -28102 : return("FAPI_VIDEC_ERR_INVALID_OPEN_PARAMS");
        case -28103 : return("FAPI_VIDEC_ERR_INVALID_COMBINATION_OF_OPEN_PARAMS");
        case -28104 : return("FAPI_VIDEC_ERR_EXCEEDED_MAX_HANDLES");
        case -28105 : return("FAPI_VIDEC_ERR_ISR_STATE_MACHINE_ALLOCATION_FAILURE");
        case -28106 : return("FAPI_VIDEC_ERR_UNSUPPORTED_FUNCTIONALITY");
        case -28107 : return("FAPI_VIDEC_ERR_HANDLE_CHECK_FAILED");
        case -28108 : return("FAPI_VIDEC_ERR_INVALID_ARGUEMENT_LIST");
        case -28109 : return("FAPI_VIDEC_ERR_FIRMWARE_FLASH_FAILURE");
        case -28110 : return("FAPI_VIDEC_ERR_VIDEC_FIRMWARE_VERSION_MISMATCH");
        case -28111 : return("FAPI_VIDEC_ERR_VIDEC_FIRMWARE_NOT_FOUND");
        case -28112 : return("FAPI_VIDEC_ERR_BOOT_TIMEOUT");
        case -28113 : return("FAPI_VIDEC_ERR_START_TIMEOUT");
        case -28114 : return("FAPI_VIDEC_ERR_STOP_TIMEOUT");
        case -28115 : return("FAPI_VIDEC_ERR_RESET_TIMEOUT");
        case -28116 : return("FAPI_VIDEC_ERR_ENABLE_DISABLE_TIMEOUT");
        case -28117 : return("FAPI_VIDEC_ERR_UNSUPPORTED_STREAM");
        case -28118 : return("FAPI_VIDEC_ERR_CALLBACK_ALREADY_ASSIGNED");
        case -28119 : return("FAPI_VIDEC_ERR_VIDEO_DECODER_NOT_RUNNING");
        case -28120 : return("FAPI_VIDEC_ERR_WRONG_HANDLE");
        case -28121 : return("FAPI_VIDEC_ERR_INVALID_INITIALIZATION_PARAMS");
        case -28122 : return("FAPI_VIDEC_ERR_DIRECT_IFRAME_EXCEEDS_ES_BUFF");
        case -28123 : return("FAPI_VIDEC_ERR_DIRECT_IFRAME_WITH_WRONG_HEADER");
        case -28124 : return("FAPI_VIDEC_ERR_DIRECT_IFRAME_NO_DECODED_FRAME_AVAILABLE");
        case -28125 : return("FAPI_VIDEC_ERR_ISR_ASSOCIATION_FAILURE");
        case -28126 : return("FAPI_VIDEC_ERR_VIDEO_DECODER_NOT_BOOTED");
        case -28127 : return("FAPI_VIDEC_ERR_VES_BUFFER_OVERFLOW");
        case -28128 : return("FAPI_VIDEC_ERR_INCORRECT_PLAYBACK_ENVIRONMENT");
        case -28129 : return("FAPI_VIDEC_ERR_VES_PARAMS_NOT_AVAILABLE");
        case -28130 : return("FAPI_VIDEC_ERR_VIDEO_MEMORY_SEGMENT_ALLOCATION_FAILURE");
        case -28131 : return("FAPI_VIDEC_ERR_VIDEO_MEMORY_CHUNK_ALLOCATION_FAILURE");
        case -28132 : return("FAPI_VIDEC_ERR_VIDEO_MEMORY_CHUNK_VIRTUAL_ADDRESS_UNAVAILABLE");
        case -28133 : return("FAPI_VIDEC_ERR_VIDEO_MEMORY_CHUNK_PHYSICAL_ADDRESS_UNAVAILABLE");
        case -28134 : return("FAPI_VIDEC_ERR_VIDEO_MEMORY_SEGMENT_STATIC_ADDRESS_INVALID");
        case -28135 : return("FAPI_VIDEC_ERR_VIDEO_DRIVER_PREINITIALIZATION_FAILURE");
        case -28136 : return("FAPI_VIDEC_ERR_VIDEO_WRONG_MODE");
        case -28137 : return("FAPI_VIDEC_ERR_VIDEO_BOARD_MEMORY_NOT_REGISTERED");
        case -28138 : return("FAPI_VIDEC_ERR_VIDEO_BOARD_MEMORY_REGISTRATION_FAILURE");
        case -28139 : return("FAPI_VIDEC_ERR_INCORRECT_PLAYBACK_MODE");
        case -28140 : return("FAPI_VIDEC_ERR_BSR_STATE_MACHINE_ALLOCATION_FAILURE");
        case -28141 : return("FAPI_VIDEC_ERR_AUD_HEADER_CODE");
        case -28142 : return("FAPI_VIDEC_ERR_INCORRECT_SYNC_BASE");
        case -28143 : return("FAPI_VIDEC_ERR_OUT_OF_BOUNDS");
        case -28144 : return("FAPI_VIDEC_ERR_WRONG_RAM_INTERFACE");
        case -28145 : return("FAPI_VIDEC_ERR_WRONG_ALLIGNMENT_OF_START_ADDRESS");
        case -28146 : return("FAPI_VIDEC_ERR_WRONG_FIRMWARE_FOR_VIDEO_DECODER");
        case -28147 : return("FAPI_VIDEC_ERR_NOT_SUFFICIENT_FRAME_BANK_MEMORY");
        case -29000 : return("FAPI_VIENC_MODULE_BASE");
        case -29001 : return("FAPI_VIENC_ERR_BAD_PARAMETER");
        case -29002 : return("FAPI_VIENC_ERR_OUT_OF_MEMORY");
        case -29003 : return("FAPI_VIENC_ERR_ALREADY_INITIALIZED");
        case -29004 : return("FAPI_VIENC_ERR_NOT_INITIALIZED");
        case -29005 : return("FAPI_VIENC_ERR_FEATURE_NOT_SUPPORTED");
        case -29006 : return("FAPI_VIENC_ERR_TIMEOUT");
        case -29007 : return("FAPI_VIENC_ERR_DEVICE_BUSY");
        case -29008 : return("FAPI_VIENC_ERR_INVALID_HANDLE");
        case -29009 : return("FAPI_VIENC_ERR_SEMAPHORE_CREATE");
        case -29010 : return("FAPI_VIENC_ERR_UNSUPPORTED_VERSION");
        case -29101 : return("FAPI_VIENC_ERR_DRV_VERSION_CHECK_FAILED");
        case -29102 : return("FAPI_VIENC_ERR_INVALID_OPEN_PARAMS");
        case -29103 : return("FAPI_VIENC_ERR_INVALID_COMBINATION_OF_OPEN_PARAMS");
        case -29104 : return("FAPI_VIENC_ERR_EXCEEDED_MAX_HANDLES");
        case -29105 : return("FAPI_VIENC_ERR_UNSUPPORTED_FUNCTIONALITY");
        case -29106 : return("FAPI_VIENC_ERR_HANDLE_CHECK_FAILED");
        case -29107 : return("FAPI_VIENC_ERR_HANDLE_OPERATION_BLOCKED");
        case -29108 : return("FAPI_VIENC_ERR_UNKNOWN_DEVICE");
        case -29109 : return("FAPI_VIENC_ERR_INVALID_ARGUEMENT_LIST");
        case -29110 : return("FAPI_VIENC_ERR_RGB_GPIOS_ALLOCATION_FAILURE");
        case -29111 : return("FAPI_VIENC_ERR_HSYNC_GPIOS_ALLOCATION_FAILURE");
        case -29112 : return("FAPI_VIENC_ERR_VSYNC_GPIOS_ALLOCATION_FAILURE");
        case -29113 : return("FAPI_VIENC_ERR_CLOCK_GPIOS_ALLOCATION_FAILURE");
        case -29114 : return("FAPI_VIENC_ERR_UNDEFINED_PLATFORM");
        case -29115 : return("FAPI_VIENC_ERR_INVALID_INITIALIZATION_PARAMS");
        case -30000 : return("FAPI_VISCALE_MODULE_BASE");
        case -30001 : return("FAPI_VISCALE_ERR_BAD_PARAMETER");
        case -30002 : return("FAPI_VISCALE_ERR_OUT_OF_MEMORY");
        case -30003 : return("FAPI_VISCALE_ERR_ALREADY_INITIALIZED");
        case -30004 : return("FAPI_VISCALE_ERR_NOT_INITIALIZED");
        case -30005 : return("FAPI_VISCALE_ERR_FEATURE_NOT_SUPPORTED");
        case -30006 : return("FAPI_VISCALE_ERR_TIMEOUT");
        case -30007 : return("FAPI_VISCALE_ERR_DEVICE_BUSY");
        case -30008 : return("FAPI_VISCALE_ERR_INVALID_HANDLE");
        case -30009 : return("FAPI_VISCALE_ERR_SEMAPHORE_CREATE");
        case -30010 : return("FAPI_VISCALE_ERR_UNSUPPORTED_VERSION");
        case -30101 : return("FAPI_VISCALE_ERR_DRV_VERSION_CHECK_FAILED");
        case -30102 : return("FAPI_VISCALE_ERR_INVALID_INITIALIZATION_PARAMS");
        case -30103 : return("FAPI_VISCALE_ERR_INVALID_OPEN_PARAMS");
        case -30104 : return("FAPI_VISCALE_ERR_INVALID_SCALER_PARAMS");
        case -30105 : return("FAPI_VISCALE_ERR_INVALID_COMBINATION_OF_OPEN_PARAMS");
        case -30106 : return("FAPI_VISCALE_ERR_EXCEEDED_MAX_HANDLES");
        case -30107 : return("FAPI_VISCALE_ERR_UNSUPPORTED_FUNCTIONALITY");
        case -30108 : return("FAPI_VISCALE_ERR_HANDLE_CHECK_FAILED");
        case -30109 : return("FAPI_VISCALE_ERR_HANDLE_OPERATION_BLOCKED");
        case -30110 : return("FAPI_VISCALE_ERR_WRONG_HANDLE");
        case -30111 : return("FAPI_VISCALE_ERR_OPERATION_TIMEOUT");
        case -30112 : return("FAPI_VISCALE_ERR_EXCEEDED_MAX_VALUE");
        case -30113 : return("FAPI_VISCALE_ERR_ISR_STATE_MACHINE_ALLOCATION_FAILURE");
        case -30114 : return("FAPI_VISCALE_ERR_INVALID_SCALING_METHOD");
        case -30115 : return("FAPI_VISCALE_ERR_PARAMETER_RANGE_EXCEEDED");
        case -30116 : return("FAPI_VISCALE_ERR_UNDEFINED_PLATFORM");
        case -30117 : return("FAPI_VISCALE_ERR_INVALID_ARGUEMENT_LIST");
        case -30118 : return("FAPI_VISCALE_ERR_OTHER_PLANE_ALREADY_OVERLAID");
        case -30119 : return("FAPI_VISCALE_ERR_WRONG_ALLIGNMENT_OF_START_ADDRESS");
        case -30120 : return("FAPI_VISCALE_ERR_WRONG_RAM_INTERFACE");
        case -30121 : return("FAPI_VIDEC_ERR_VISCALE_DRIVER_PREINITIALIZATION_FAILURE");
        case -31000 : return("FAPI_BOARD_MODULE_BASE");
        case -31001 : return("FAPI_BOARD_ERR_BAD_PARAMETER");
        case -31002 : return("FAPI_BOARD_ERR_OUT_OF_MEMORY");
        case -31003 : return("FAPI_BOARD_ERR_ALREADY_INITIALIZED");
        case -31004 : return("FAPI_BOARD_ERR_NOT_INITIALIZED");
        case -31005 : return("FAPI_BOARD_ERR_FEATURE_NOT_SUPPORTED");
        case -31006 : return("FAPI_BOARD_ERR_TIMEOUT");
        case -31007 : return("FAPI_BOARD_ERR_DEVICE_BUSY");
        case -31008 : return("FAPI_BOARD_ERR_INVALID_HANDLE");
        case -31009 : return("FAPI_BOARD_ERR_SEMAPHORE_CREATE");
        case -31010 : return("FAPI_BOARD_ERR_UNSUPPORTED_VERSION");
        case -32000 : return("FAPI_BOARDS_MODULE_BASE");
        case -33000 : return("FAPI_CEC_MODULE_BASE");
        case -33001 : return("FAPI_CEC_ERR_BAD_PARAMETER");
        case -33002 : return("FAPI_CEC_ERR_OUT_OF_MEMORY");
        case -33003 : return("FAPI_CEC_ERR_ALREADY_INITIALIZED");
        case -33004 : return("FAPI_CEC_ERR_NOT_INITIALIZED");
        case -33005 : return("FAPI_CEC_ERR_FEATURE_NOT_SUPPORTED");
        case -33006 : return("FAPI_CEC_ERR_TIMEOUT");
        case -33007 : return("FAPI_CEC_ERR_DEVICE_BUSY");
        case -33008 : return("FAPI_CEC_ERR_INVALID_HANDLE");
        case -33009 : return("FAPI_CEC_ERR_SEMAPHORE_CREATE");
        case -33010 : return("FAPI_CEC_ERR_UNSUPPORTED_VERSION");
        case -33100 : return("FAPI_CEC_ERR_MESSAGE_BUFFER_FULL");
        case -33101 : return("FAPI_CEC_ERR_MESSAGE_BUFFER_EMPTY");
        case -33102 : return("FAPI_CEC_ERR_MESSAGE_BUFFER_OVERFLOW");
        case -33103 : return("FAPI_CEC_ERR_MESSAGE_ERROR");
        case -33104 : return("FAPI_CEC_ERR_RECEIVE_AWAITING");
        case -33105 : return("FAPI_CEC_ERR_NONE_OPCODE_PARAM");
        case -33106 : return("FAPI_CEC_ERR_BAD_OPCODE_PARAM");
        case -34000 : return("FAPI_SATA_MODULE_BASE");
        case -34001 : return("FAPI_SATA_ERR_BAD_PARAMETER");
        case -34002 : return("FAPI_SATA_ERR_OUT_OF_MEMORY");
        case -34003 : return("FAPI_SATA_ERR_ALREADY_INITIALIZED");
        case -34004 : return("FAPI_SATA_ERR_NOT_INITIALIZED");
        case -34005 : return("FAPI_SATA_ERR_FEATURE_NOT_SUPPORTED");
        case -34006 : return("FAPI_SATA_ERR_TIMEOUT");
        case -34007 : return("FAPI_SATA_ERR_DEVICE_BUSY");
        case -34008 : return("FAPI_SATA_ERR_INVALID_HANDLE");
        case -34009 : return("FAPI_SATA_ERR_SEMAPHORE_CREATE");
        case -34010 : return("FAPI_SATA_ERR_UNSUPPORTED_VERSION");
        case -34100 : return("FAPI_SATA_ERR_XXX");
        case -35000 : return("FAPI_SDIO_MODULE_BASE");
        case -35001 : return("FAPI_SDIO_ERR_BAD_PARAMETER");
        case -35002 : return("FAPI_SDIO_ERR_OUT_OF_MEMORY");
        case -35003 : return("FAPI_SDIO_ERR_ALREADY_INITIALIZED");
        case -35004 : return("FAPI_SDIO_ERR_NOT_INITIALIZED");
        case -35005 : return("FAPI_SDIO_ERR_FEATURE_NOT_SUPPORTED");
        case -35006 : return("FAPI_SDIO_ERR_TIMEOUT");
        case -35007 : return("FAPI_SDIO_ERR_DEVICE_BUSY");
        case -35008 : return("FAPI_SDIO_ERR_INVALID_HANDLE");
        case -35009 : return("FAPI_SDIO_ERR_SEMAPHORE_CREATE");
        case -35010 : return("FAPI_SDIO_ERR_UNSUPPORTED_VERSION");
        case -35100 : return("FAPI_SDIO_ERR_HDMAC_OPEN");
        case -35101 : return("FAPI_SDIO_ERR_CRC");
        case -35102 : return("FAPI_SDIO_ERR_CMD");
        case -35103 : return("FAPI_SDIO_ERR_WRITE_PROTECTED");
        case -35104 : return("FAPI_SDIO_ERR_MODE_UNSOPORTED");
        case -35105 : return("FAPI_SDIO_ERR_AT_POWER_UP");
        case -35106 : return("FAPI_SDIO_ERR_CARD_REMOVED");
        case -35107 : return("FAPI_SDIO_ERR_CARD_INIT_FAILED");
        case -35108 : return("FAPI_SDIO_ERR_READ");
        case -35109 : return("FAPI_SDIO_ERR_WRITE");
        case -35110 : return("FAPI_SDIO_ERR_CARD_NOT_READY");
        case -35111 : return("FAPI_SDIO_ERR_LBA_NUMBER");
        case -36000 : return("FAPI_BLTE_MODULE_BASE");
        case -36001 : return("FAPI_BLTE_ERR_BAD_PARAMETER");
        case -36002 : return("FAPI_BLTE_ERR_OUT_OF_MEMORY");
        case -36003 : return("FAPI_BLTE_ERR_ALREADY_INITIALIZED");
        case -36004 : return("FAPI_BLTE_ERR_NOT_INITIALIZED");
        case -36005 : return("FAPI_BLTE_ERR_FEATURE_NOT_SUPPORTED");
        case -36006 : return("FAPI_BLTE_ERR_TIMEOUT");
        case -36007 : return("FAPI_BLTE_ERR_DEVICE_BUSY");
        case -36008 : return("FAPI_BLTE_ERR_INVALID_HANDLE");
        case -36009 : return("FAPI_BLTE_ERR_SEMAPHORE_CREATE");
        case -36010 : return("FAPI_BLTE_ERR_UNSUPPORTED_VERSION");
        case -36100 : return("FAPI_BLTE_ERR_XXX");
        case -37000 : return("FAPI_HDMAC_MODULE_BASE");
        case -37001 : return("FAPI_HDMAC_ERR_BAD_PARAMETER");
        case -37002 : return("FAPI_HDMAC_ERR_OUT_OF_MEMORY");
        case -37003 : return("FAPI_HDMAC_ERR_ALREADY_INITIALIZED");
        case -37004 : return("FAPI_HDMAC_ERR_NOT_INITIALIZED");
        case -37005 : return("FAPI_HDMAC_ERR_FEATURE_NOT_SUPPORTED");
        case -37006 : return("FAPI_HDMAC_ERR_TIMEOUT");
        case -37007 : return("FAPI_HDMAC_ERR_DEVICE_BUSY");
        case -37008 : return("FAPI_HDMAC_ERR_INVALID_HANDLE");
        case -37009 : return("FAPI_HDMAC_ERR_SEMAPHORE_CREATE");
        case -37010 : return("FAPI_HDMAC_ERR_UNSUPPORTED_VERSION");
        case -37100 : return("FAPI_HDMAC_ERR_TRANSFER");
        case -37101 : return("FAPI_HDMAC_ERR_BAD_BLOCKCOUNT");
        case -37102 : return("FAPI_HDMAC_ERR_BAD_TRANSFERCOUNT");
        case -37103 : return("FAPI_HDMAC_ERR_BAD_MEMCOPY_SYIZE");
        case -38000 : return("FAPI_PV_MODULE_BASE");
        case -38001 : return("FAPI_PV_ERR_BAD_PARAMETER");
        case -38002 : return("FAPI_PV_ERR_OUT_OF_MEMORY");
        case -38003 : return("FAPI_PV_ERR_ALREADY_INITIALIZED");
        case -38004 : return("FAPI_PV_ERR_NOT_INITIALIZED");
        case -38005 : return("FAPI_PV_ERR_FEATURE_NOT_SUPPORTED");
        case -38006 : return("FAPI_PV_ERR_TIMEOUT");
        case -38007 : return("FAPI_PV_ERR_DEVICE_BUSY");
        case -38008 : return("FAPI_PV_ERR_INVALID_HANDLE");
        case -38009 : return("FAPI_PV_ERR_SEMAPHORE_CREATE");
        case -38010 : return("FAPI_PV_ERR_UNSUPPORTED_VERSION");
        case -38101 : return("FAPI_PV_ERR_OUT_OF_RANGE");
        case -38102 : return("FAPI_PV_ERR_HIDDEN_ADDRESS");
        case -38103 : return("FAPI_PV_ERR_BURN_DATA");
        case -39000 : return("FAPI_R2R_MODULE_BASE");
        case -39001 : return("FAPI_R2R_ERR_BAD_PARAMETER");
        case -39002 : return("FAPI_R2R_ERR_OUT_OF_MEMORY");
        case -39003 : return("FAPI_R2R_ERR_ALREADY_INITIALIZED");
        case -39004 : return("FAPI_R2R_ERR_NOT_INITIALIZED");
        case -39005 : return("FAPI_R2R_ERR_FEATURE_NOT_SUPPORTED");
        case -39006 : return("FAPI_R2R_ERR_TIMEOUT");
        case -39007 : return("FAPI_R2R_ERR_DEVICE_BUSY");
        case -39008 : return("FAPI_R2R_ERR_INVALID_HANDLE");
        case -39009 : return("FAPI_R2R_ERR_SEMAPHORE_CREATE");
        case -39010 : return("FAPI_R2R_ERR_UNSUPPORTED_VERSION");
        case -39101 : return("FAPI_R2R_ERR_ALREADY_STARTED");
        case -39102 : return("FAPI_R2R_ERR_ALREADY_STOPPED");
        case -39103 : return("FAPI_R2R_ERR_NOT_256_BYTE_ALIGNED");
        case -1000000 : return("RTOS_SHARED_MODULE_BASE");
        case -1000101 : return("RTOS_ERR_NOT_INITIALISED");
        case -1000102 : return("RTOS_ERR_ALREADY_INITIALIZED");
        case -1000103 : return("RTOS_ERR_TABLE_OVERFLOW");
        case -1000104 : return("RTOS_ERR_INVALID_HANDLE");
        case -1000105 : return("RTOS_ERR_INVALID_PARAMETER");
        case -1000106 : return("RTOS_ERR_TIMEOUT");
        case -1000107 : return("RTOS_ERR_OUT_OF_MEMORY");
        case -1000108 : return("RTOS_ERR_NOT_ALLOWED");
        case -1000109 : return("RTOS_ERR_HANDLE_DESTROYED");
        case -1100000 : return("RTOS_FAMOS_MODULE_BASE");
        case -1100101 : return("FAMOS_ERR_NOT_INITIALISED");
        case -1100102 : return("FAMOS_ERR_ALREADY_INITIALIZED");
        case -1100103 : return("FAMOS_ERR_TABLE_OVERFLOW");
        case -1100104 : return("FAMOS_ERR_INVALID_HANDLE");
        case -1100105 : return("FAMOS_ERR_INVALID_PARAMETER");
        case -1100106 : return("FAMOS_ERR_TIMEOUT");
        case -1100107 : return("FAMOS_ERR_OUT_OF_MEMORY");
        case -1100108 : return("FAMOS_ERR_NOT_ALLOWED");
        case -1100109 : return("FAMOS_ERR_HANDLE_DESTROYED");
        case -1200000 : return("RTOS_STUB_MODULE_BASE");
        case -1200001 : return("RTOS_STUB_ERR_BAD_PARAMETER");
        case -1200002 : return("RTOS_STUB_ERR_OUT_OF_MEMORY");
        case -1200003 : return("RTOS_STUB_ERR_ALREADY_INITIALIZED");
        case -1200004 : return("RTOS_STUB_ERR_NOT_INITIALIZED");
        case -1200005 : return("RTOS_STUB_ERR_FEATURE_NOT_SUPPORTED");
        case -1200006 : return("RTOS_STUB_ERR_TIMEOUT");
        case -1200007 : return("RTOS_STUB_ERR_DEVICE_BUSY");
        case -1200008 : return("RTOS_STUB_ERR_INVALID_HANDLE");
        case -1200009 : return("RTOS_STUB_ERR_SEMAPHORE_CREATE");
        case -1200010 : return("RTOS_STUB_ERR_UNSUPPORTED_VERSION");
        case -2000000 : return("SUBSYS_FE_MODULE_BASE");
        case -2100000 : return("SUBSYS_FE_CE6353_MODULE_BASE");
        case -2200000 : return("SUBSYS_FE_DSF89000_MODULE_BASE");
        case -2300000 : return("SUBSYS_FE_MB86668_MODULE_BASE");
        case -2400000 : return("SUBSYS_PAINT_MODULE_BASE");
        case -2500000 : return("SUBSYS_PAINT_DRAW_MODULE_BASE");
        case -2500001 : return("DRAW_ERR_BAD_PARAMETER");
        case -2500002 : return("DRAW_ERR_OUT_OF_MEMORY");
        case -2500003 : return("DRAW_ERR_ALREADY_INITIALIZED");
        case -2500004 : return("DRAW_ERR_NOT_INITIALIZED");
        case -2500005 : return("DRAW_ERR_FEATURE_NOT_SUPPORTED");
        case -2500006 : return("DRAW_ERR_TIMEOUT");
        case -2500007 : return("DRAW_ERR_DEVICE_BUSY");
        case -2500008 : return("DRAW_ERR_INVALID_HANDLE");
        case -2500009 : return("DRAW_ERR_SEMAPHORE_CREATE");
        case -2500010 : return("DRAW_ERR_UNSUPPORTED_VERSION");
        case -2500100 : return("DRAW_ERR_OUT_OF_OSD_SIZE");
        case -2500101 : return("DRAW_ERR_WRONG_COLOUR_MODE");
        case -2500102 : return("DRAW_ERR_WRONG_LAYOUT_MODE");
        case -2500103 : return("DRAW_ERR_BITMAP_NOT_SUPPORTED");
        case -2500104 : return("DRAW_ERR_UNKNOWN_FILE_TYPE");
        case -2500105 : return("DRAW_ERR_PNG_NOT_SUPPORTED");
        case -2500106 : return("DRAW_ERR_PNG_INIT");
        case -2500107 : return("DRAW_ERR_PNG_DECODE");
        case -2600000 : return("SUBSYS_PAINT_FONT_MODULE_BASE");
        case -2601000 : return("SUBSYS_PAINT_FONTSERVER_MODULE_BASE");
        case -2601001 : return("FONTSERVER_ERR_BAD_PARAMETER");
        case -2601002 : return("FONTSERVER_ERR_OUT_OF_MEMORY");
        case -2601003 : return("FONTSERVER_ERR_ALREADY_INITIALIZED");
        case -2601004 : return("FONTSERVER_ERR_NOT_INITIALIZED");
        case -2601005 : return("FONTSERVER_ERR_FEATURE_NOT_SUPPORTED");
        case -2601006 : return("FONTSERVER_ERR_TIMEOUT");
        case -2601007 : return("FONTSERVER_ERR_DEVICE_BUSY");
        case -2601008 : return("FONTSERVER_ERR_INVALID_HANDLE");
        case -2601009 : return("FONTSERVER_ERR_SEMAPHORE_CREATE");
        case -2601010 : return("FONTSERVER_ERR_UNSUPPORTED_VERSION");
        case -2601100 : return("FONTSERVER_ERR_BAD_FORMAT");
        case -2601101 : return("FONTSERVER_ERR_OUT_OF_OSD_SIZE");
        case -2601102 : return("FONTSERVER_ERR_NEGATIVE_CHAR_ADVANCE");
        case -2601103 : return("FONTSERVER_ERR_MAX_HANDLE_COUNT");
        case -2601104 : return("FONTSERVER_ERR_BAD_COLOUR");
        case -2601105 : return("FONTSERVER_ERR_MAX_FONT_COUNT");
        case -2601106 : return("FONTSERVER_ERR_CHAR_NOT_SUPPORTED");
        case -2601200 : return("FONTSERVER_ERR_FBF_FONT_INIT");
        case -2601201 : return("FONTSERVER_ERR_FBF_BAD_FBF_FILE");
        case -2601202 : return("FONTSERVER_ERR_FBF_MEMORY_FAULT");
        case -2601203 : return("FONTSERVER_ERR_FBF_KERNING_NOT_SUPPORTED");
        case -2601300 : return("FONTSERVER_ERR_FT_INIT");
        case -2601301 : return("FONTSERVER_ERR_FT_INIT_CACHE_SUBSYSTEM");
        case -2601302 : return("FONTSERVER_ERR_FT_BAD_FORMAT");
        case -2601303 : return("FONTSERVER_ERR_FT_CANNOT_SET_FONT_SIZE");
        case -2601304 : return("FONTSERVER_ERR_FT_LOOKUP_SCALER");
        case -2601305 : return("FONTSERVER_ERR_FT_BAD_FONT_SIZE");
        case -2601306 : return("FONTSERVER_ERR_FT_STYLE_NOT_AVAILABLE");
        case -2601307 : return("FONTSERVER_ERR_FT_FREETYPE_NOT_REGISTERED");
        case -2601308 : return("FONTSERVER_ERR_FT_KERNING_NOT_SUPPORTED");
        case -2700000 : return("SUBSYS_PAINT_OSDHANDLER_MODULE_BASE");
        case -2700001 : return("OSDHANDLER_ERR_BAD_PARAMETER");
        case -2700002 : return("OSDHANDLER_ERR_OUT_OF_MEMORY");
        case -2700003 : return("OSDHANDLER_ERR_ALREADY_INITIALIZED");
        case -2700004 : return("OSDHANDLER_ERR_NOT_INITIALIZED");
        case -2700005 : return("OSDHANDLER_ERR_FEATURE_NOT_SUPPORTED");
        case -2700006 : return("OSDHANDLER_ERR_TIMEOUT");
        case -2700007 : return("OSDHANDLER_ERR_DEVICE_BUSY");
        case -2700008 : return("OSDHANDLER_ERR_INVALID_HANDLE");
        case -2700009 : return("OSDHANDLER_ERR_SEMAPHORE_CREATE");
        case -2700010 : return("OSDHANDLER_ERR_UNSUPPORTED_VERSION");
        case -2700100 : return("OSDHANDLER_ERR_BAD_LAYER_ID");
        case -2700101 : return("OSDHANDLER_ERR_NO_OSD_HANDLE");
        case -2700102 : return("OSDHANDLER_ERR_BAD_COLOUR_MODE");
        case -2700103 : return("OSDHANDLER_ERR_OUT_OF_SIZE");
        case -2700104 : return("OSDHANDLER_ERR_OUT_OF_BOUNDS");
        case -2700105 : return("OSDHANDLER_ERR_LAYER_NOT_STARTED");
        case -2700106 : return("OSDHANDLER_ERR_CP_4KB_EXCEEDED");
        case -2700200 : return("OSDHANDLER_ERR_VL_ALREADY_EXISTS");
        case -2700201 : return("OSDHANDLER_ERR_VL_ALREADY_HIDDEN");
        case -2700202 : return("OSDHANDLER_ERR_VL_NO_FREE_LAYER");
        case -2700203 : return("OSDHANDLER_ERR_VL_LAYER_IS_LOCKED");
        case -2700204 : return("OSDHANDLER_ERR_VL_LAYER_IS_VIRTUAL");
        case -2700205 : return("OSDHANDLER_ERR_VL_BUSY");
        case -2700206 : return("OSDHANDLER_ERR_VL_ALREADY_UPDATED");
        case -2700207 : return("OSDHANDLER_ERR_VL_FIFO_OVERFLOW");
        case -2700300 : return("OSDHANDLER_FB_ERR_BAD_HANDLE");
        case -2700301 : return("OSDHANDLER_FB_ERR_MEMORY_ALIGN");
        case -2700302 : return("OSDHANDLER_FB_ERR_BAD_COLOUR_MODE");
        case -2800000 : return("SUBSYS_STORAGE_MODULE_BASE");
        case -2900000 : return("SUBSYS_STORAGE_MMCSD_MODULE_BASE");
        case -2900001 : return("SUBSYS_STORAGE_MMCSD_ERR_BAD_PARAMETER");
        case -2900002 : return("SUBSYS_STORAGE_MMCSD_ERR_OUT_OF_MEMORY");
        case -2900003 : return("SUBSYS_STORAGE_MMCSD_ERR_ALREADY_INITIALIZED");
        case -2900004 : return("SUBSYS_STORAGE_MMCSD_ERR_NOT_INITIALIZED");
        case -2900005 : return("SUBSYS_STORAGE_MMCSD_ERR_FEATURE_NOT_SUPPORTED");
        case -2900006 : return("SUBSYS_STORAGE_MMCSD_ERR_TIMEOUT");
        case -2900007 : return("SUBSYS_STORAGE_MMCSD_ERR_DEVICE_BUSY");
        case -2900008 : return("SUBSYS_STORAGE_MMCSD_ERR_INVALID_HANDLE");
        case -2900009 : return("SUBSYS_STORAGE_MMCSD_ERR_SEMAPHORE_CREATE");
        case -2900010 : return("SUBSYS_STORAGE_MMCSD_ERR_UNSUPPORTED_VERSION");
        case -2900100 : return("SUBSYS_STORAGE_MMCSD_ERR_UNABLE_CREATE_SSP_HANDLE");
        case -2900101 : return("SUBSYS_STORAGE_MMCSD_ERR_NO_DATA");
        case -2900102 : return("SUBSYS_STORAGE_MMCSD_ERR_NO_RESPONSE");
        case -2900103 : return("SUBSYS_STORAGE_MMCSD_ERR_NO_CARD");
        case -2900104 : return("SUBSYS_STORAGE_MMCSD_ERR_BAD_ADDRESS");
        case -2900105 : return("SUBSYS_STORAGE_MMCSD_ERR_NOT_READY");
        case -3000000 : return("SUBSYS_AUDIO_MIXER_MODULE_BASE");
        case -3100000 : return("SUBSYS_FS_FLASHFS_MODULE_BASE");
        case -3100001 : return("FLASHFS_ERR_BAD_PARAMETER");
        case -3100002 : return("FLASHFS_ERR_OUT_OF_MEMORY");
        case -3100003 : return("FLASHFS_ERR_ALREADY_INITIALIZED");
        case -3100004 : return("FLASHFS_ERR_NOT_INITIALIZED");
        case -3100005 : return("FLASHFS_ERR_FEATURE_NOT_SUPPORTED");
        case -3100006 : return("FLASHFS_ERR_TIMEOUT");
        case -3100007 : return("FLASHFS_ERR_DEVICE_BUSY");
        case -3100008 : return("FLASHFS_ERR_INVALID_HANDLE");
        case -3100009 : return("FLASHFS_ERR_SEMAPHORE_CREATE");
        case -3100010 : return("FLASHFS_ERR_UNSUPPORTED_VERSION");
        case -3100011 : return("FLASHFS_ERR_MAILQUEUE_CREATE");
        case -3100012 : return("FLASHFS_ERR_NOT_OPEN");
        case -3100013 : return("FLASHFS_ERR_ALREADY_OPEN");
        case -3100100 : return("FLASHFS_ERR_INVALID_DEVICE");
        case -3100101 : return("FLASHFS_ERR_INVALID_ALLOCATION");
        case -3100102 : return("FLASHFS_ERR_DUPLICATED_ID");
        case -3100103 : return("FLASHFS_ERR_FILE_NOT_EXIST");
        case -3100104 : return("FLASHFS_ERR_CHECKSUM");
        case -3100105 : return("FLASHFS_ERR_END_OF_FILE");
        case -3100106 : return("FLASHFS_ERR_PROHIBITED");
        case -3100107 : return("FLASHFS_ERR_INVALID_FILE");
        case -3100108 : return("FLASHFS_ERR_WRITE_OUT_OF_BOUNDS");
        case -3100109 : return("FLASHFS_ERR_THREAD_STATUS");
        case -3200000 : return("SUBSYS_FOTA_MODULE_BASE");
        case -3200001 : return("SUBSYS_FOTA_ERR_BAD_PARAMETER");
        case -3200002 : return("SUBSYS_FOTA_ERR_OUT_OF_MEMORY");
        case -3200003 : return("SUBSYS_FOTA_ERR_ALREADY_INITIALIZED");
        case -3200004 : return("SUBSYS_FOTA_ERR_NOT_INITIALIZED");
        case -3200005 : return("SUBSYS_FOTA_ERR_FEATURE_NOT_SUPPORTED");
        case -3200006 : return("SUBSYS_FOTA_ERR_TIMEOUT");
        case -3200007 : return("SUBSYS_FOTA_ERR_DEVICE_BUSY");
        case -3200008 : return("SUBSYS_FOTA_ERR_INVALID_HANDLE");
        case -3200009 : return("SUBSYS_FOTA_ERR_SEMAPHORE_CREATE");
        case -3200010 : return("SUBSYS_FOTA_ERR_UNSUPPORTED_VERSION");
        case -3200100 : return("SUBSYS_FOTA_ERR_UPDATE_ABORT");
        case -3300000 : return("SUBSYS_GPROF_MODULE_BASE");
        case -3300001 : return("SUBSYS_GPROF_ERR_BAD_PARAMETER");
        case -3300002 : return("SUBSYS_GPROF_ERR_OUT_OF_MEMORY");
        case -3300003 : return("SUBSYS_GPROF_ERR_ALREADY_INITIALIZED");
        case -3300004 : return("SUBSYS_GPROF_ERR_NOT_INITIALIZED");
        case -3300005 : return("SUBSYS_GPROF_ERR_FEATURE_NOT_SUPPORTED");
        case -3300006 : return("SUBSYS_GPROF_ERR_TIMEOUT");
        case -3300007 : return("SUBSYS_GPROF_ERR_DEVICE_BUSY");
        case -3300008 : return("SUBSYS_GPROF_ERR_INVALID_HANDLE");
        case -3300009 : return("SUBSYS_GPROF_ERR_SEMAPHORE_CREATE");
        case -3300010 : return("SUBSYS_GPROF_ERR_UNSUPPORTED_VERSION");
        case -3300101 : return("SUBSYS_GPROF_ERR_HDD_INIT");
        case -3300102 : return("SUBSYS_GPROF_ERR_RAM_INIT");
        case -3300103 : return("SUBSYS_GPROF_ERR_CREATE_THREAD");
        case -3400000 : return("SUBSYS_VOC_MODULE_BASE");
        case -3400001 : return("VOC_ERR_BAD_PARAMETER");
        case -3400002 : return("VOC_ERR_OUT_OF_MEMORY");
        case -3400003 : return("VOC_ERR_ALREADY_INITIALIZED");
        case -3400004 : return("VOC_ERR_NOT_INITIALIZED");
        case -3400005 : return("VOC_ERR_FEATURE_NOT_SUPPORTED");
        case -3400006 : return("VOC_ERR_TIMEOUT");
        case -3400007 : return("VOC_ERR_DEVICE_BUSY");
        case -3400008 : return("VOC_ERR_INVALID_HANDLE");
        case -3400009 : return("VOC_ERR_SEMAPHORE_CREATE");
        case -3400010 : return("VOC_ERR_UNSUPPORTED_VERSION");
        case -3400012 : return("VOC_ERR_NOT_OPEN");
        case -3400013 : return("VOC_ERR_ALREADY_OPEN");
        case -3400100 : return("VOC_ERR_OUT_OF_OSD_SIZE");
        case -3400101 : return("VOC_ERR_WRONG_COLOUR_MODE");
        case -3400102 : return("VOC_ERR_UNKNOWN_FRAMEBANK");
        case -3500000 : return("SUBSYS_VIDEO_VAL_MODULE_BASE");
        case -3500001 : return("SUBSYS_VIDEO_VAL_ERR_BAD_PARAMETER");
        case -3500002 : return("SUBSYS_VIDEO_VAL_ERR_OUT_OF_MEMORY");
        case -3500003 : return("SUBSYS_VIDEO_VAL_ERR_ALREADY_INITIALIZED");
        case -3500004 : return("SUBSYS_VIDEO_VAL_ERR_NOT_INITIALIZED");
        case -3500005 : return("SUBSYS_VIDEO_VAL_ERR_FEATURE_NOT_SUPPORTED");
        case -3500006 : return("SUBSYS_VIDEO_VAL_ERR_TIMEOUT");
        case -3500007 : return("SUBSYS_VIDEO_VAL_ERR_DEVICE_BUSY");
        case -3500008 : return("SUBSYS_VIDEO_VAL_ERR_INVALID_HANDLE");
        case -3500009 : return("SUBSYS_VIDEO_VAL_ERR_SEMAPHORE_CREATE");
        case -3500010 : return("SUBSYS_VIDEO_VAL_ERR_UNSUPPORTED_VERSION");
        case -3500012 : return("SUBSYS_VIDEO_VAL_ERR_NOT_OPEN");
        case -3500013 : return("SUBSYS_VIDEO_VAL_ERR_ALREADY_OPEN");
        case -3500100 : return("SUBSYS_VIDEO_VAL_ERR_DATA_NOT_AVAILABLE");
        case -3500101 : return("SUBSYS_VIDEO_VAL_ERR_STATE");
        case -3500102 : return("SUBSYS_VIDEO_VAL_ERR_ALREADY_STARTED");
        case -3500103 : return("SUBSYS_VIDEO_VAL_ERR_BAD_MODE");
        case -3500104 : return("SUBSYS_VIDEO_VAL_ERR_INTERNAL");
        case -3500105 : return("SUBSYS_VIDEO_VAL_ERR_THREAD_CREATE");
        case -3600000 : return("SUBSYS_SFAL_MODULE_BASE");
        case -3600001 : return("SUBSYS_SFAL_ERR_BAD_PARAMETER");
        case -3600002 : return("SUBSYS_SFAL_ERR_OUT_OF_MEMORY");
        case -3600003 : return("SUBSYS_SFAL_ERR_ALREADY_INITIALIZED");
        case -3600004 : return("SUBSYS_SFAL_ERR_NOT_INITIALIZED");
        case -3600005 : return("SUBSYS_SFAL_ERR_FEATURE_NOT_SUPPORTED");
        case -3600006 : return("SUBSYS_SFAL_ERR_TIMEOUT");
        case -3600007 : return("SUBSYS_SFAL_ERR_DEVICE_BUSY");
        case -3600008 : return("SUBSYS_SFAL_ERR_INVALID_HANDLE");
        case -3600009 : return("SUBSYS_SFAL_ERR_SEMAPHORE_CREATE");
        case -3600010 : return("SUBSYS_SFAL_ERR_UNSUPPORTED_VERSION");
        case -3600011 : return("SUBSYS_SFAL_ERR_MAILQUEUE_CREATE");
        case -3600100 : return("SUBSYS_SFAL_ERR_INVALID_PID");
        case -3600101 : return("SUBSYS_SFAL_ERR_NO_PID_ASSIGNED");
        case -3600102 : return("SUBSYS_SFAL_ERR_THREAD_CREATE");
        case -3600103 : return("SUBSYS_SFAL_ERR_NO_FREE_CHANNEL");
        case -3600104 : return("SUBSYS_SFAL_ERR_NO_FREE_FILTER");
        case -3600105 : return("SUBSYS_SFAL_ERR_SECTION_TIMEOUT");
        case -3600106 : return("SUBSYS_SFAL_ERR_SECTION_CRC");
        case -3600107 : return("SUBSYS_SFAL_ERR_ALREADY_ACTIVE");
        case -3600108 : return("SUBSYS_SFAL_ERR_FILTER_DEPTH");
        case -3600109 : return("SUBSYS_SFAL_ERR_BUFFER_SIZE_TO_SMALL");
        case -3700000 : return("SUBSYS_AUDIO_AAL_MODULE_BASE");
        case -3700001 : return("SUBSYS_AUDIO_AAL_ERR_BAD_PARAMETER");
        case -3700002 : return("SUBSYS_AUDIO_AAL_ERR_OUT_OF_MEMORY");
        case -3700003 : return("SUBSYS_AUDIO_AAL_ERR_ALREADY_INITIALIZED");
        case -3700004 : return("SUBSYS_AUDIO_AAL_ERR_NOT_INITIALIZED");
        case -3700005 : return("SUBSYS_AUDIO_AAL_ERR_FEATURE_NOT_SUPPORTED");
        case -3700006 : return("SUBSYS_AUDIO_AAL_ERR_TIMEOUT");
        case -3700007 : return("SUBSYS_AUDIO_AAL_ERR_DEVICE_BUSY");
        case -3700008 : return("SUBSYS_AUDIO_AAL_ERR_INVALID_HANDLE");
        case -3700009 : return("SUBSYS_AUDIO_AAL_ERR_SEMAPHORE_CREATE");
        case -3700010 : return("SUBSYS_AUDIO_AAL_ERR_UNSUPPORTED_VERSION");
        case -3700012 : return("SUBSYS_AUDIO_AAL_ERR_NOT_OPEN");
        case -3700013 : return("SUBSYS_AUDIO_AAL_ERR_ALREADY_OPEN");
        case -3700100 : return("SUBSYS_AUDIO_AAL_ERR_DATA_NOT_AVAILABLE");
        case -3700101 : return("SUBSYS_AUDIO_AAL_ERR_ALREADY_STARTED");
        case -3800000 : return("SUBSYS_CI_MODULE_BASE");
        case -3800001 : return("SUBSYS_CI_ERR_BAD_PARAMETER");
        case -3800005 : return("SUBSYS_CI_ERR_FEATURE_NOT_SUPPORTED");
        case -3800020 : return("SUBSYS_CI_ERR_NO_HW_DRIVER");
        case -3800021 : return("SUBSYS_CI_ERR_BAD_INIT_PARAMETER");
        case -3800022 : return("SUBSYS_CI_ERR_INIT_PHY_LAYER");
        case -3800023 : return("SUBSYS_CI_ERR_INIT_TRA_LAYER");
        case -3800024 : return("SUBSYS_CI_ERR_INIT_SES_LAYER");
        case -3900000 : return("SUBSYS_AVCAP_MODULE_BASE");
        case -3900001 : return("SUBSYS_AVCAP_ERR_BAD_PARAMETER");
        case -3900002 : return("SUBSYS_AVCAP_ERR_OUT_OF_MEMORY");
        case -3900003 : return("SUBSYS_AVCAP_ERR_ALREADY_INITIALIZED");
        case -3900004 : return("SUBSYS_AVCAP_ERR_NOT_INITIALIZED");
        case -3900005 : return("SUBSYS_AVCAP_ERR_FEATURE_NOT_SUPPORTED");
        case -3900006 : return("SUBSYS_AVCAP_ERR_TIMEOUT");
        case -3900007 : return("SUBSYS_AVCAP_ERR_DEVICE_BUSY");
        case -3900008 : return("SUBSYS_AVCAP_ERR_INVALID_HANDLE");
        case -3900009 : return("SUBSYS_AVCAP_ERR_SEMAPHORE_CREATE");
        case -3900010 : return("SUBSYS_AVCAP_ERR_UNSUPPORTED_VERSION");
        case -3900011 : return("SUBSYS_AVCAP_ERR_MAILQUEUE_CREATE");
        case -3900100 : return("SUBSYS_AVCAP_ERR_DMA_FAILED");
        case -3900101 : return("SUBSYS_AVCAP_ERR_ALLREADY_OPEN");
        case -4000000 : return("SUBSYS_PVR_MODULE_BASE");
        case -4000001 : return("SUBSYS_PVR_ERR_BAD_PARAMETER");
        case -4000002 : return("SUBSYS_PVR_ERR_OUT_OF_MEMORY");
        case -4000003 : return("SUBSYS_PVR_ERR_ALREADY_INITIALIZED");
        case -4000004 : return("SUBSYS_PVR_ERR_NOT_INITIALIZED");
        case -4000005 : return("SUBSYS_PVR_ERR_FEATURE_NOT_SUPPORTED");
        case -4000006 : return("SUBSYS_PVR_ERR_TIMEOUT");
        case -4000007 : return("SUBSYS_PVR_ERR_DEVICE_BUSY");
        case -4000008 : return("SUBSYS_PVR_ERR_INVALID_HANDLE");
        case -4000009 : return("SUBSYS_PVR_ERR_SEMAPHORE_CREATE");
        case -4000010 : return("SUBSYS_PVR_ERR_UNSUPPORTED_VERSION");
        case -4000100 : return("SUBSYS_PVR_ERR_FILE_HANDLING");
        case -4000101 : return("SUBSYS_PVR_ERR_BAD_FILE_FORMAT");
        case -4000102 : return("SUBSYS_PVR_ERR_FILE_READ");
        case -4000103 : return("SUBSYS_PVR_ERR_BAD_PACKET_SIZE");
        case -4000104 : return("SUBSYS_PVR_ERR_FILE_EXISTS");
        case -4000105 : return("SUBSYS_PVR_ERR_THREAD_ERROR");
        case -4000106 : return("SUBSYS_PVR_ERR_BAD_MODE");
        case -4000107 : return("SUBSYS_PVR_ERR_DISK_WRITE");
        case -4000108 : return("SUBSYS_PVR_ERR_NO_VIDEO");
        case -4000109 : return("SUBSYS_PVR_ERR_NO_TRACK_ASSIGNED");
        case -4000110 : return("SUBSYS_PVR_ERR_WRONG_MARKER_TYPE");
        case -4000111 : return("SUBSYS_PVR_ERR_END_OF_DATA");
        case -4000112 : return("SUBSYS_PVR_ERR_BEGIN_OF_DATA");
        case -4000113 : return("SUBSYS_PVR_ERR_FILE_DOES_NOT_EXIST");
        case -4000114 : return("SUBSYS_PVR_ERR_TSD");
        case -4000115 : return("SUBSYS_PVR_ERR_BAD_FILE_VERSION");
        case -4000116 : return("SUBSYS_PVR_ERR_NO_BUFFER_SPACE");
        case -4000117 : return("SUBSYS_PVR_ERR_CRYPTO_OP_FAILED");
        case -5000000 : return("MIDDLEWARE_MHEG_MODULE_BASE");
        case -10000000 : return("APPLICATION_SMARTGO_MODULE_BASE");
        case -10000001 : return("APPL_SMARTGO_ERR_BAD_PARAMETER");
        case -10000002 : return("APPL_SMARTGO_ERR_OUT_OF_MEMORY");
        case -10000003 : return("APPL_SMARTGO_ERR_ALREADY_INITIALIZED");
        case -10000004 : return("APPL_SMARTGO_ERR_NOT_INITIALIZED");
        case -10000005 : return("APPL_SMARTGO_ERR_FEATURE_NOT_SUPPORTED");
        case -10000006 : return("APPL_SMARTGO_ERR_TIMEOUT");
        case -10000007 : return("APPL_SMARTGO_ERR_DEVICE_BUSY");
        case -10000008 : return("APPL_SMARTGO_ERR_INVALID_HANDLE");
        case -10000009 : return("APPL_SMARTGO_ERR_SEMAPHORE_CREATE");
        case -10000010 : return("APPL_SMARTGO_ERR_UNSUPPORTED_VERSION");
        case -10000012 : return("APPL_SMARTGO_ERR_NOT_OPEN");
        case -10000013 : return("APPL_SMARTGO_ERR_ALREADY_OPEN");
        case -10001001 : return("SMARTGO_SYS_ERR_PROHIBITED");
        case -10001002 : return("SMARTGO_SYS_ERR_INTERNAL");
        case -10001003 : return("SMARTGO_SYS_ERR_ABORTED");
        case -10001004 : return("SMARTGO_SYS_ERR_CONFIG_ERROR");
        case -10001005 : return("SMARTGO_SYS_ERR_CAPACITY");
        case -10001006 : return("SMARTGO_SYS_ERR_DATA_NOT_AVL");
        case -10001007 : return("SMARTGO_SYS_ERR_ALREADY_IN_LIST");
        case -10001008 : return("SMARTGO_SYS_ERR_REPLACED_IN_LIST");
        case -10001009 : return("SMARTGO_SYS_ERR_INVALID_DATA");
        case -10001010 : return("SMARTGO_SYS_ERR_RESOURCES");
        case -10001011 : return("SMARTGO_SYS_ERR_ITEM_EMPTY");
        case -10001013 : return("SMARTGO_SYS_ERR_RTOS_ERROR");
        case -10001014 : return("SMARTGO_SYS_ERR_DATA_NOT_PROVIDED");
        case -10001015 : return("SMARTGO_SYS_ERR_RSRC");
        case -10001016 : return("SMARTGO_SYS_ERR_RSRC_LIVE");
        case -10001017 : return("SMARTGO_SYS_ERR_RSRC_PLAY");
        case -10001018 : return("SMARTGO_SYS_ERR_RSRC_RECORD");
        case -10001019 : return("SMARTGO_SYS_ERR_RSRC_SCAN");
        case -10001020 : return("SMARTGO_SYS_ERR_RSRC_CI");
        case -10001021 : return("SMARTGO_SYS_ERR_RSRC_NETWORK");
        case -10001022 : return("SMARTGO_SYS_ERR_RSRC_PIP");
        case -10001023 : return("SMARTGO_SYS_ERR_RSRC_TUNEMODE");
        case -10001024 : return("SMARTGO_SYS_ERR_RSRC_UPDATE");
        case -10001025 : return("SMARTGO_SYS_ERR_RSRC_NOT_ACTIVE");
        case -10001026 : return("SMARTGO_SYS_ERR_UPDATE_CRC");
        case -10001027 : return("SMARTGO_SYS_ERR_UPDATE_CAPACITY");
        case -10001028 : return("SMARTGO_SYS_ERR_FILEREAD");
        case -10001029 : return("SMARTGO_SYS_ERR_FILEWRITE");
        case -10002001 : return("SMARTGO_FGS_ERR_BAD_PARAMETER");
        case -10002002 : return("SMARTGO_FGS_ERR_OUT_OF_MEMORY");
        case -10002003 : return("SMARTGO_FGS_ERR_ALREADY_INITIALIZED");
        case -10002004 : return("SMARTGO_FGS_ERR_NOT_INITIALIZED");
        case -10002005 : return("SMARTGO_FGS_ERR_FEATURE_NOT_SUPPORTED");
        case -10002006 : return("SMARTGO_FGS_ERR_TIMEOUT");
        case -10002007 : return("SMARTGO_FGS_ERR_DEVICE_BUSY");
        case -10002008 : return("SMARTGO_FGS_ERR_INVALID_HANDLE");
        case -10002009 : return("SMARTGO_FGS_ERR_SEMAPHORE_CREATE");
        case -10002010 : return("SMARTGO_FGS_ERR_UNSUPPORTED_VERSION");
        case -10002101 : return("SMARTGO_FGS_ERR_PROHIBITED");
        case -10002102 : return("SMARTGO_FGS_ERR_RESOURCES");
        case -10003001 : return("SMARTGO_APPL_ERR_PROHIBITED");
        case -10003002 : return("SMARTGO_APPL_ERR_INTERNAL");
        case -10003003 : return("SMARTGO_APPL_ERR_CAPACITY");
        case -10003004 : return("SMARTGO_APPL_ERR_ALREADY_OPEN");
        case -10003005 : return("SMARTGO_APPL_ERR_DATA_NOT_AVL");

        default: return("UNKNOWN_ERROR");
    }
}