/*
*****************************************************************************
**
** \file        sys_services_ext.c
** 
** \brief       System service interface.
** 
** This file contains the implementation of the system service API 
** used by an operating environment to provide the services needed by
** the FAPI core drivers.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
** (C) Copyright 2008-2010 by Fujitsu Semiconductor Limited
** 
*****************************************************************************
*/

#include "fapi/sys_services.h"
#include "fapi/reg_ddr.h"

#ifdef __LINUX__

#include <linux/slab.h>
#include <linux/vmalloc.h>
#include "fapex_cfdlsaf.h"
#include "fapex/sys_config.h"

//***************************************************************************
//***************************************************************************
//** Local Defines
//***************************************************************************
//***************************************************************************

#if defined(__arm__)
#define FAPI_SYS_CPU_STATUS_INTR_MASK 0xC0
//      FAPI_SYS_CPU_STATUS_INTR_FIQ  0x40
#define FAPI_SYS_CPU_STATUS_INTR_IRQ  0x80
#define FAPI_SYS_CPU_STATUS_MODE_MASK 0x1F
#define FAPI_SYS_CPU_STATUS_MODE_USR  0x10
//      FAPI_SYS_CPU_STATUS_MODE_FIQ  0x11
//      FAPI_SYS_CPU_STATUS_MODE_IRQ  0x12
//      FAPI_SYS_CPU_STATUS_MODE_SVC  0x13
//      FAPI_SYS_CPU_STATUS_MODE_ABT  0x17
//      FAPI_SYS_CPU_STATUS_MODE_UND  0x1B
#define FAPI_SYS_CPU_STATUS_MODE_SYS  0x1F
#endif

#if defined(_ARC)
#define FAPI_SYS_CPU_STATUS_INTR_MASK 0x0C000000
//      FAPI_SYS_CPU_STATUS_INTR_L2   0x04000000
#define FAPI_SYS_CPU_STATUS_INTR_L1   0x08000000
#define FAPI_SYS_CPU_STATUS_MODE_MASK 0x00000018
#define FAPI_SYS_CPU_STATUS_MODE_USR  0x00000000
//      FAPI_SYS_CPU_STATUS_MODE_IL1  0x00000008
//      FAPI_SYS_CPU_STATUS_MODE_IL2  0x00000010
#endif

#define FAPI_SYS_CPU_ID_ARM           0
#if defined(_ARC)
#define FAPI_SYS_CPU_ID_ARC           1
#endif

/* min value for schedule_timeout() */
#define FAPIDRVSYS_SLEEP_TIMOUT_MIN   ((HZ >= 1000) ? 1 : (1000 / HZ))

/* kmalloc allocation max size */
#define FAPIDRVSYS_KMALLOC_SIZE_MAX   KMALLOC_MAX_SIZE

/*!
*************************************************************************
** \brief uncashed buffer management control parameter
*************************************************************************/
#define FDSYS_UNCASHED_BUFF_AREA_MAX        128
#define FDSYS_UNCASHED_BUFF_ALIGNED_SIZE    64


//******************************************************************************
//******************************************************************************
//** Local Data
//******************************************************************************
//******************************************************************************

static uint32_t sysMasterCpuId = FAPI_SYS_CPU_ID_ARM;

/*!< management paramater pointer for uncashed area */
static FAPISAF_BuffManageT  *saf_ua_manage = NULL;


//***************************************************************************
//***************************************************************************
//** Local Functions
//***************************************************************************
//***************************************************************************

/*!
*******************************************************************************
**
** \brief  initialization for static common area allocation
**
** \param  baseAddress  base physical address of static allocation area
** \param  totalSize    buffer size of static allocation area
** \param  alignSize    aligned size for static allocation area address
**                      (it should be 2^n)
** \param  maxNum       max number of static allocation area
**
** \return Valid pointer to FAPISAF_BuffManageT data or NULL if no memory available.
**
******************************************************************************/
FAPISAF_BuffManageT *FAPISAF_InitSBuffArea(uint32_t baseAddress, uint32_t totalSize,
                                           uint32_t alignSize, uint32_t maxNum)
{
    FAPISAF_BuffManageT *buff_manage;
    FAPISAF_BuffCtrlT   *buff_ctrl, *prev_ctrl;
    uint32_t  buff_top_addr;
    uint32_t  buff_align_rest;
    int32_t   size, i;

    FAPI_SYS_PRINT_DEBUG(1, "[SYS/FAPISAF_InitSBuffArea]\n");

    buff_manage = (FAPISAF_BuffManageT *)kmalloc(sizeof(FAPISAF_BuffManageT),
                                                 GFP_KERNEL);
    if(buff_manage == NULL) {
        return NULL;
    }

    /* allocate static buffer information */
    size = maxNum * sizeof(FAPISAF_BuffCtrlT);
    buff_ctrl = (FAPISAF_BuffCtrlT *)vmalloc(size);
    if(buff_ctrl == NULL) {
        kfree(buff_manage);
        return NULL;
    }
    memset(buff_ctrl, 0, size);

    /* setup static area information */
    buff_manage->base_info  = buff_manage->free_area = buff_ctrl;
    buff_manage->alloc_area = prev_ctrl              = NULL;
    for(i=0; i<maxNum; i++) {
        buff_ctrl->buff_top  = (uint32_t)-1;
        buff_ctrl->prev_info = prev_ctrl;
        if(prev_ctrl != NULL) {
            prev_ctrl->next_info = buff_ctrl;
        }
        prev_ctrl = buff_ctrl;
        buff_ctrl ++;
    }
    buff_align_rest = alignSize - 1;
    buff_top_addr  = baseAddress + buff_align_rest;
    buff_top_addr &= ~buff_align_rest;
    size = totalSize - (buff_top_addr - baseAddress);

    /* set first of stream buffer free list */
    if(size > 0) {
        buff_manage->free_area->buff_top  = buff_top_addr;
        buff_manage->free_area->buff_size = size;
    }
    else {
        buff_manage->free_area->buff_top  = 0;
        buff_manage->free_area->buff_size = 0;
    }

    /* set end of stream buffer free list */
    buff_manage->free_area->prev_info = prev_ctrl;

    sema_init(&buff_manage->sem, 1);
    buff_manage->align_mask = ~buff_align_rest;

    return buff_manage;
}

/*!
*******************************************************************************
**
** \brief  exit for static common area allocation
**
** \param  pManage      pointer to FAPISAF_BuffManageT
**
******************************************************************************/
int32_t FAPISAF_ExitSBuffArea(FAPISAF_BuffManageT *pManage)
{
    if(pManage != NULL) {
        if(pManage->base_info != NULL) {
            vfree(pManage->base_info);
            pManage->base_info = NULL;
        }
        kfree(pManage);
    }
    return 0;
}

/*!
*******************************************************************************
**
** \brief  allocate buffer from static common area
**
** \param  pSem   pointer to semaphore
** \param  size   allocate buffer size
**
** \return Valid address to the allocated memory or 0 if no static
**         memory available. (physical address)
**
******************************************************************************/
uint32_t FAPISAF_AllocSBuffArea(FAPISAF_BuffManageT *pManage, uint32_t size)
{
    FAPISAF_BuffCtrlT  *buff_ctrl, *best_ctrl, *tmp_ctrl, *end_ctrl;
    uint32_t  buff_align_rest;
    uint32_t  allocate_size;
    int32_t   fit_rest, min_rest;
    uint32_t  ret_address;
    int32_t   ret;

    buff_align_rest = ~(pManage->align_mask);
    allocate_size   = (size + buff_align_rest) & pManage->align_mask;

    /*---------- LOCK ----------*/
    ret = down_interruptible(&pManage->sem);
    if(ret < 0) {
        return 0;
    }

    buff_ctrl = pManage->free_area;
    if(buff_ctrl == NULL) {
        /*--------- UNLOCK ---------*/
        up(&pManage->sem);
        return 0;
    }
    end_ctrl = buff_ctrl->prev_info;
    buff_ctrl->prev_info = NULL;

    /* search best fit free area */
    best_ctrl = NULL;
    min_rest  = 0x7FFFFFFF;
    while((buff_ctrl != NULL) && (buff_ctrl->buff_size > 0)) {
        fit_rest = buff_ctrl->buff_size - allocate_size;
        if((fit_rest >= 0) && (min_rest > fit_rest)) {
            min_rest  = fit_rest;
            best_ctrl = buff_ctrl;
            if(fit_rest == 0) {
                break;
            }
        }
        buff_ctrl = buff_ctrl->next_info;
    }
    if(best_ctrl == NULL) {
        ret_address = 0;
    }
    else {
        /* update free area information */
        if((min_rest == 0) || (best_ctrl == end_ctrl)) {
            buff_ctrl = best_ctrl;
            if(buff_ctrl->next_info != NULL) {
                buff_ctrl->next_info->prev_info = buff_ctrl->prev_info;
            }
            if(buff_ctrl->prev_info != NULL) {
                buff_ctrl->prev_info->next_info = buff_ctrl->next_info;
            }
            if(best_ctrl == end_ctrl) {
                pManage->free_area = NULL;
            }
            else if(pManage->free_area == buff_ctrl){
                pManage->free_area = buff_ctrl->next_info;
            }
        }
        else {
            buff_ctrl = end_ctrl;
            end_ctrl  = end_ctrl->prev_info;
            end_ctrl->next_info = NULL;

            buff_ctrl->buff_top  = best_ctrl->buff_top;
            buff_ctrl->buff_size = allocate_size;

            best_ctrl->buff_top  += allocate_size;
            best_ctrl->buff_size -= allocate_size;
        }

        /* update allocate area information */
        if(pManage->alloc_area != NULL) {
            tmp_ctrl = pManage->alloc_area->prev_info;
            tmp_ctrl->next_info  = buff_ctrl;
            buff_ctrl->prev_info = tmp_ctrl;
        }
        else {
            pManage->alloc_area        = buff_ctrl;
            buff_ctrl->prev_info = NULL;
        }
        buff_ctrl->next_info = NULL;

        /* set end of list */
        pManage->alloc_area->prev_info = buff_ctrl;
        ret_address = buff_ctrl->buff_top;
    }
    if(pManage->free_area != NULL) {
        /* set end of list */
        pManage->free_area->prev_info = end_ctrl;
    }

    /*--------- UNLOCK ---------*/
    up(&pManage->sem);

    return ret_address;
}

/*!
*******************************************************************************
**
** \brief  free buffer from static common area
**
** \param  pSem     pointer to semaphore
** \param  address  free buffer physical address
**
******************************************************************************/
void FAPISAF_FreeSBuffArea(FAPISAF_BuffManageT *pManage, uint32_t address)
{
    FAPISAF_BuffCtrlT  *buff_ctrl, *tmp0_ctrl, *tmp1_ctrl, *end_ctrl;
    int32_t  ret;

    /*---------- LOCK ----------*/
    ret = down_interruptible(&pManage->sem);
    if(ret < 0) {
        return;
    }

    buff_ctrl = pManage->alloc_area;
    if(buff_ctrl == NULL) {
        /*--------- UNLOCK ---------*/
        up(&pManage->sem);
        return;
    }
    end_ctrl = buff_ctrl->prev_info;
    buff_ctrl->prev_info = NULL;

    /* search allocate information */
    while(buff_ctrl->buff_top != address) {
        buff_ctrl = buff_ctrl->next_info;
        if(buff_ctrl == NULL) {
            pManage->alloc_area->prev_info = end_ctrl;

            /*--------- UNLOCK ---------*/
            up(&pManage->sem);
            return;
        }
    }

    /* update allocate area information */
    if(buff_ctrl->prev_info != NULL) {
        buff_ctrl->prev_info->next_info = buff_ctrl->next_info;
    }
    else {
        pManage->alloc_area = buff_ctrl->next_info;
    }
    if(buff_ctrl->next_info != NULL) {
        buff_ctrl->next_info->prev_info = buff_ctrl->prev_info;
    }
    /* set end of list */
    if(buff_ctrl == end_ctrl) {
        end_ctrl = buff_ctrl->prev_info;
    }
    if(end_ctrl != NULL) {
        pManage->alloc_area->prev_info = end_ctrl;
    }

    /* update free area information */
    if(pManage->free_area != NULL) {
        end_ctrl = pManage->free_area->prev_info;
        tmp0_ctrl = NULL;
        tmp1_ctrl = pManage->free_area;
        tmp1_ctrl->prev_info = NULL;

        /* update free area information with sorting by address */
        while(address > tmp1_ctrl->buff_top) {
            tmp0_ctrl = tmp1_ctrl;
            tmp1_ctrl = tmp1_ctrl->next_info;
            if(tmp1_ctrl == NULL) {
                break;
            }
        }
        /* insert to free list */
        buff_ctrl->next_info = tmp1_ctrl;
        buff_ctrl->prev_info = tmp0_ctrl;
        if(tmp0_ctrl != NULL) {
            tmp0_ctrl->next_info = buff_ctrl;
        }
        else {
            pManage->free_area = buff_ctrl;
        }
        if(tmp1_ctrl != NULL) {
            tmp1_ctrl->prev_info = buff_ctrl;
        }
        else {
            end_ctrl = buff_ctrl;
        }
        /* check continuity area */
        if(tmp0_ctrl != NULL) {
            if((tmp0_ctrl->buff_top + tmp0_ctrl->buff_size) == address) {
                /* merge buff_ctrl to tmp0_ctrl */
                tmp0_ctrl->buff_size += buff_ctrl->buff_size;
                buff_ctrl->buff_top  = (uint32_t)-1;
                buff_ctrl->buff_size = 0;
                if(buff_ctrl != end_ctrl) {
                    /* add buff_ctrl to the end of list */
                    tmp0_ctrl->next_info = tmp1_ctrl;
                    if(tmp1_ctrl != NULL) {
                        tmp1_ctrl->prev_info = tmp0_ctrl;
                    }
                    end_ctrl->next_info  = buff_ctrl;
                    buff_ctrl->prev_info = end_ctrl;
                    end_ctrl  = buff_ctrl;
                    buff_ctrl = tmp0_ctrl;
                }
            }
        }
        if(tmp1_ctrl != NULL) {
            if((buff_ctrl->buff_top + buff_ctrl->buff_size) == tmp1_ctrl->buff_top) {
                /* merge tmp1_ctrl to buff_ctrl */
                buff_ctrl->buff_size += tmp1_ctrl->buff_size;
                tmp1_ctrl->buff_top  = (uint32_t)-1;
                tmp1_ctrl->buff_size = 0;
                if(tmp1_ctrl != end_ctrl) {
                    /* add tmp1_ctrl to the end of list */
                    buff_ctrl->next_info = tmp1_ctrl->next_info;
                    if(tmp1_ctrl->next_info != NULL) {
                        tmp1_ctrl->next_info->prev_info = buff_ctrl;
                    }
                    end_ctrl->next_info  = tmp1_ctrl;
                    tmp1_ctrl->prev_info = end_ctrl;
                    end_ctrl = tmp1_ctrl;
                }
            }
        }
    }
    else {
        pManage->free_area = end_ctrl = buff_ctrl;
    }
    /* set end of list */
    end_ctrl->next_info = NULL;
    pManage->free_area->prev_info = end_ctrl;

    /*--------- UNLOCK ---------*/
    up(&pManage->sem);
}

/*
*****************************************************************************
** \brief Private malloc/free implementation.
** 
** This function allocates the given amount of memory bytes from the system
** heap.
**
** \param size  Number of bytes to allocate from the system heap.
**
** \retval >0  the pointer to the allocated memory area
** \retval 0   error
*****************************************************************************
*/
void *FAPIDRVSYS_AllocateMemory(uint32_t size)
{
    uint32_t  *tmp_ptr;

    size += 4 * sizeof(uint32_t);

    if(size <= FAPIDRVSYS_KMALLOC_SIZE_MAX) {
        tmp_ptr = (uint32_t*)kmalloc((ssize_t)size, GFP_KERNEL);

        if(tmp_ptr != NULL) {
            FAPI_SYS_PRINT_DEBUG(3, "\033[34mkmalloc %08x %d\033[30m\n",
                                    tmp_ptr, size);
            *tmp_ptr = 'k';
            tmp_ptr += 4;
        }
    }
    else {
        uint32_t     order;
        struct page  *buff_pg;

        order = PAGE_SHIFT;
        while( (order < 31) && (size > (1 << order)) ) {
            order ++;
        }
        order -= PAGE_SHIFT;

        buff_pg = alloc_pages(GFP_HIGHUSER, order);
        if(buff_pg == NULL) {
            return 0;
        }
        tmp_ptr = (uint32_t*)page_address(buff_pg);

        if(tmp_ptr != NULL) {
            FAPI_SYS_PRINT_DEBUG(3, "\033[35malloc_pages %08x %08x %d\033[30m\n",
                                    tmp_ptr, buff_pg, order);
            *tmp_ptr = 'p';
            tmp_ptr ++;
            *tmp_ptr = (uint32_t)buff_pg;
            tmp_ptr ++;
            *tmp_ptr = order;
            tmp_ptr += 2;
        }
    }
    return (void *)tmp_ptr;
}

/*!
********************************************************************************
** \brief Allocate dynamic memory from uncached heap memory.
**
** This macro calls the registered FAPI_SYS_ServicesT::mallocUncachedFunc
** (if there is one) to allocate a chunk of memory.
**
** \param size The number of bytes to allocate.
**
** \return Valid pointer to the allocated memory or NULL if no dynamic
**         memory available.
**
** \sa FAPI_SYS_FREE
********************************************************************************
*/
void *FAPIDRVSYS_AllocateMemoryUncached(uint32_t size)
{
    uint32_t  *tmp_ptr;

    size += 4 * sizeof(uint32_t);

    tmp_ptr = (uint32_t*)FAPISAF_AllocSBuffArea(saf_ua_manage, size);
    if(tmp_ptr != NULL) {
        tmp_ptr = (uint32_t*)FIO_ADDRESS(BUF, (uint32_t)tmp_ptr);

        FAPI_SYS_PRINT_DEBUG(3, "\033[34mumalloc %08x %d\033[30m\n",
                                tmp_ptr, size);
        *tmp_ptr = 'u';
        tmp_ptr += 4;
    }
    return (void *)tmp_ptr;
}

/*!
********************************************************************************
** \brief Free the given memory pointer within the system heap.
**
** This function frees the given memory pointer inside the system heap.
**
** \param ptr    Memory pointer to free, previously returned by
**               FAPIDRVSYS_AllocateMemory() or FAPIDRVSYS_AllocateMemoryUncached()
**
** \return none
********************************************************************************
*/
void FAPIDRVSYS_FreeMemory(void *ptr)
{
    uint32_t *tmp_ptr;

    if(ptr == NULL) {
        return;
    }

    tmp_ptr = (uint32_t *)ptr;
    tmp_ptr -= 4;

    /* check the allocation type */
    if(*tmp_ptr == 'k') {
        FAPI_SYS_PRINT_DEBUG(3, "\033[34mkfree %08x\033[30m\n",
                                tmp_ptr);
        kfree(tmp_ptr);
    }
    else if(*tmp_ptr == 'p') {
        uint32_t     order;
        struct page  *buff_pg;

        FAPI_SYS_PRINT_DEBUG(3, "\033[35m__free_pages %08x ",
                                tmp_ptr);
        tmp_ptr ++;
        buff_pg = (struct page*)*tmp_ptr;
        tmp_ptr ++;
        order   = *tmp_ptr;
        FAPI_SYS_PRINT_DEBUG(3, "%08x %d\033[30m\n",
                                buff_pg, order);
        __free_pages(buff_pg, order);
    }
    else {
        FAPI_SYS_PRINT_DEBUG(3, "\033[34mufree %08x\033[30m\n",
                                tmp_ptr);
        tmp_ptr = (uint32_t *)IO_VIRT_TO_PHYS((uint32_t)tmp_ptr);
        FAPISAF_FreeSBuffArea(saf_ua_manage, (uint32_t)tmp_ptr);
    }
    return;
}

#ifdef DEBUG
/*
*****************************************************************************
** \brief Private printf implementation.
** 
** This function simply disables the interrupts, calls the system vprintf
** function and re-enables the interrupts again.
**
** \param format The printf style format string to be used
** \param ...    Optional, format string specific arguments.
**
** \return The number of bytes printed.
*****************************************************************************
*/
static int32_t FAPIDRVSYS_Printf( const char* formatPtr, ... )
{
    va_list argp;
    int32_t ret;

    va_start( argp, formatPtr );
    ret = vprintk( formatPtr, argp );
    va_end( argp );

    return ret;
}
#endif

//***************************************************************************
//***************************************************************************
//** API Functions
//***************************************************************************
//***************************************************************************

/*!
*****************************************************************************
** \brief Invalidate data cache lines
**
** This function invalidates the data cache line holding memory addresses
** from the given address range [start..start+bytes-1].
**
** \param startAddress Start address of the memory block to invalidate
**                     within one/more data cache line
** \param bytes        Number of bytes of the memory block to invalidate
**                     within one/more data cache line
*****************************************************************************
*/
void FAPI_SYS_InvalidateDataCache( uint32_t startAddress, uint32_t bytes )
{
    /*lint -save -e529 -e550 -e957 */

    uint32_t          status;
    register uint32_t end_address = ( startAddress + bytes ) - 1;

    if( startAddress == 0UL )
        return;

    //////////////////////////////
    status = FAPI_SYS_DISABLE_IRQ;
    //////////////////////////////

    startAddress = startAddress & ~0x3;
    end_address  = end_address  & ~0x3;

    #if defined(__arm__) && !defined(__GNUC__) && !defined(__polyspace__)
        __asm { mcrr p15,0,end_address,startAddress,c6 }
    #endif

    #if defined(__arm__) && defined(__GNUC__) && !defined(__polyspace__)
        asm volatile( "mcrr p15,0,%0,%1,c6" : : "r"(end_address), "r"(startAddress) );
    #endif

    #if defined(_ARC) && !defined(__polyspace__)
        for( ; startAddress <= end_address; startAddress+=16 )
            _sr( startAddress, 0x4A );
        while( ( _lr( 0x48 ) & 0x100 ) == 0x100 )
            ;
    #endif

    //////////////////////////////
    FAPI_SYS_ENABLE_IRQ( status );
    //////////////////////////////

    /*lint -restore */
}

/*!
*****************************************************************************
** \brief Flush data cache lines
**
** This function flushes the data cache line holding memory addresses
** from the given address range [start..start+bytes-1].
**
** \param startAddress Start address of the memory block to flush
**                     from one/more data cache line
** \param bytes        Number of bytes of the memory block to flush
**                     from one/more data cache line
*****************************************************************************
*/
void FAPI_SYS_FlushDataCache( uint32_t startAddress, uint32_t bytes )
{
    /*lint -save -e529 -e550 -e957 */

    uint32_t          status;
    register uint32_t end_address = ( startAddress + bytes ) - 1;

    if( startAddress == 0UL )
        return;

    //////////////////////////////
    status = FAPI_SYS_DISABLE_IRQ;
    //////////////////////////////

    startAddress = startAddress & ~0x3;
    end_address  = end_address  & ~0x3;

    #if defined(__arm__) && !defined(__GNUC__) && !defined(__polyspace__)
        __asm { mcrr p15,0,end_address,startAddress,c12 }
    #endif

    #if defined(__arm__) && defined(__GNUC__) && !defined(__polyspace__)
        asm volatile( "mcrr p15,0,%0,%1,c12" : : "r"(end_address), "r"(startAddress) );
    #endif

    #if defined(_ARC) && !defined(__polyspace__)
        for( ; startAddress <= end_address; startAddress+=16 )
            _sr( startAddress, 0x4C );
        while( ( _lr( 0x48 ) & 0x100 ) == 0x100 )
            ;
    #endif

    //////////////////////////////
    FAPI_SYS_ENABLE_IRQ( status );
    //////////////////////////////

    /*lint -restore */
}

/*!
********************************************************************************
** \brief Invalidate data cache
**
** This function invalidates the entire instruction cache.
********************************************************************************
*/
void FAPI_SYS_InvalidateInstructionCache( void )
{
    /*lint -save -e529 -e957 */

    //////////////////////////////
    uint32_t status = FAPI_SYS_DISABLE_IRQ;
    //////////////////////////////

    #if defined(__arm__) && !defined(__GNUC__) && !defined(__polyspace__)
        register uint32_t value = 0x0;
        __asm { mcr p15,0,value,c7,c5,0 }
    #endif

    #if defined(__arm__) && defined(__GNUC__) && !defined(__polyspace__)
        uint32_t value = 0x0;
        asm volatile( "mcr p15,0,%0,c7,c5,0" : : "r"(value) );
    #endif

    #if defined(_ARC) && !defined(__polyspace__)
        register uint32_t value = 0x1;
        _sr( value, 0x10 );
    #endif

    //////////////////////////////
    FAPI_SYS_ENABLE_IRQ( status );
    //////////////////////////////

    /*lint -restore */
}

/*!
*****************************************************************************
** \brief Retrieve the current device id
**
** This function retrieves the current device id by reading the efuse
** section for various device pattern.
**
** \return
** - #FAPI_SYS_DEVICE_ID_MB86H61_ES1  if a MB86H61 ES1 device was detected
** - #FAPI_SYS_DEVICE_ID_MB86H61_ES2  if a MB86H61 ES2 device was detected
** - #FAPI_SYS_DEVICE_ID_MB86H615_ES1 if a MB86H615 ES1 device was detected
** - #FAPI_SYS_DEVICE_ID_UNKNOWN if no valid device id could be extracted
*****************************************************************************
*/
uint32_t FAPI_SYS_GetDeviceId( void )
{
    volatile uint32_t* info_ptr;

    /*lint -save -e923 */
    info_ptr = (volatile uint32_t*)FIO_ADDRESS(EFUSE,FAPI_SYS_EFUSE_BASE_ADDRESS);
    /*lint -restore */

    switch( info_ptr[3] & 0x0000FFFFUL )
    {
        case 0x0000FEF0UL : return( FAPI_SYS_DEVICE_ID_MB86H61_ES1 );
        case 0x0000FEFFUL : return( FAPI_SYS_DEVICE_ID_MB86H61_ES2 );
        case 0x0000FDF0UL : return( FAPI_SYS_DEVICE_ID_MB86H615_ES1 );
        //  case 0x0000FDFFUL : return( FAPI_SYS_DEVICE_ID_MB86H615_ES2 );
        //  case 0x0000FCF0UL : return( FAPI_SYS_DEVICE_ID_MB86H612_ES1 );
        //  case 0x0000FCFFUL : return( FAPI_SYS_DEVICE_ID_MB86H612_ES2 );
        //  case 0x0000FBF0UL : return( FAPI_SYS_DEVICE_ID_MB86H81_ES1 );
        //  case 0x0000FBFFUL : return( FAPI_SYS_DEVICE_ID_MB86H81_ES2 );
        default           : return( FAPI_SYS_DEVICE_ID_UNKNOWN );
    }
}

/*!
********************************************************************************
** \brief Retrieve the current device name
**
** This function retrieves the current device name by reading the efuse
** section for various device pattern.
**
** \return
** - "MB86H61/ES1"  if a MB86H61 ES1 device was detected
** - "MB86H61/ES2"  if a MB86H61 ES2 device was detected
** - "MB86H615/ES1" if a MB86H615 ES1 device was detected
** - "Unknown"      if no valid device id could be extracted
********************************************************************************
*/
char_t* FAPI_SYS_GetDeviceName( void )
{
    switch( FAPI_SYS_GetDeviceId() )
    {
        case FAPI_SYS_DEVICE_ID_MB86H61_ES1  : return("MB86H61/ES1");
        case FAPI_SYS_DEVICE_ID_MB86H61_ES2  : return("MB86H61/ES2");
        case FAPI_SYS_DEVICE_ID_MB86H615_ES1 : return("MB86H615/ES1");
        default                              : return("Unknown");
    }
}

/*!
********************************************************************************
** \brief Set the master cpu flag to the current cpu.
**
** This function sets the internal master cpu flag to the current cpu.
** By default it is initialised to '0' which identifies the ARM cpu.
**
********************************************************************************
*/
void FAPI_SYS_SetMasterCpu( void )
{
    #if defined(__arm__)
        sysMasterCpuId = FAPI_SYS_CPU_ID_ARM;
    #endif
    #if defined(_ARC)
        sysMasterCpuId = FAPI_SYS_CPU_ID_ARC;
    #endif
}

/*!
********************************************************************************
** \brief Check whether the current cpu is the master cpu.
**
** This function checks whether the current cpu equals the master cpu by simply
** comparing the internal master cpu flag with the current cpu flag.
**
** \return
** '1' (true) if the current cpu is the master cpu.
** '0' (false) if the current cpu is not the master cpu.
**
********************************************************************************
*/
uint32_t FAPI_SYS_IsMasterCpu( void )
{
    #if defined(__arm__)
        return( sysMasterCpuId == FAPI_SYS_CPU_ID_ARM );
    #endif
    #if defined(_ARC)
        return( sysMasterCpuId == FAPI_SYS_CPU_ID_ARC );
    #endif
    #if !defined(__arm__) && !defined(_ARC)
        return( 0 );
    #endif
}

/*!
*****************************************************************************
** \brief Delay execution for a certain time.
**
** This function represents the \b default (OS-less) implementation for the 
** service function FAPIDRVSYS_ServicesT::sleepFunc. It is applied until the 
** service will be overwritten.
** 
** \param mSecs The sleep time in milliseconds. 
**
** \sa FAPIDRVSYS_SetServices
*****************************************************************************
*/   
void  FAPIDRVSYS_Sleep(uint32_t mSecs)
{
    if(mSecs < FAPIDRVSYS_SLEEP_TIMOUT_MIN) {
        mSecs = FAPIDRVSYS_SLEEP_TIMOUT_MIN;
    }
    set_current_state(TASK_INTERRUPTIBLE);
    schedule_timeout(mSecs * HZ / 1000);
}

/*!
*****************************************************************************
** \brief Disable interrupts
**
** This function disables \b FIQ and \b IRQ of the ARM11 core. It represents 
** the \b default implementation for the service function 
** FAPIDRVSYS_ServicesT::disableIrqFunc. 
** It is applied until the service will be overwritten.
**
** \return The IRQ flags before thery were disabled. This is needed to exactly 
**         re-store the interrupts by a later call to \ref FAPIDRVSYS_EnableIrq.
**
** \sa FAPIDRVSYS_SetServices
** \sa FAPIDRVSYS_EnableIrq
**
** \note Lint error 529 temporarilly disabled 
**       within inline assembler context 
*****************************************************************************
*/ 
uint32_t FAPIDRVSYS_DisableIrq(void)
{
    uint32_t irqFlags;

    local_irq_save((unsigned long)irqFlags);
    return irqFlags;
}

/*!
*****************************************************************************
** \brief Enable interrupts
**
** This function enables respectively restores \b FIQ and \b IRQ of the ARM11 
** core. It represents the \b default implementation for the service function 
** FAPIDRVSYS_ServicesT::enableIrqFunc. 
** It is applied until the service will be overwritten.
**
** \param irqFlags The IRQ flags to re-store, usually returned by a
**                 previous call to \ref FAPIDRVSYS_DisableIrq.
**
** \sa FAPIDRVSYS_SetServices
** \sa FAPIDRVSYS_DisableIrq
**
** \note Lint errors 529 and 550 temporarilly disabled 
**       within inline assembler context 
*****************************************************************************
*/ 
void FAPIDRVSYS_EnableIrq(uint32_t irqFlags)
{
    local_irq_restore((unsigned long)irqFlags);
    return;
}

/*!
*****************************************************************************
** \brief Create a semaphore.
**
** This function represents the \b default (OS-less) implementation for the 
** service function FAPIDRVSYS_ServicesT::createSemaphoreFunc. 
** It is applied until the service will be overwritten.
** The implementation supports up to \b 256 semaphores.
**
** \param val The initial value of the semaphore. It must be >= 0.
**
** \return 
**         - Handle to the new semaphore (>0) if sucessful.
**         - (0) if no more semaphore could be created since total number
**           of semaphores has been exceeded. 
**
** \sa FAPIDRVSYS_SetServices
** \sa FAPIDRVSYS_DestroySemaphore
** \sa FAPIDRVSYS_GetSemaphore
** \sa FAPIDRVSYS_SetSemaphore
*****************************************************************************
*/ 
FAPI_SYS_SemaphoreT FAPIDRVSYS_CreateSemaphore(int32_t val)
{
    struct semaphore *pSem;

    pSem = (struct semaphore *)kmalloc(sizeof(struct semaphore),GFP_KERNEL);
    if(pSem != NULL) {
        sema_init(pSem, val);
    }

    return (FAPI_SYS_SemaphoreT)pSem;
}

/*!
*****************************************************************************
** \brief Destroy a semaphore.
**
** This function represents the \b default (OS-less) implementation for the 
** service function FAPIDRVSYS_ServicesT::destroySemaphoreFunc. 
** It is applied until the service will be overwritten.
**
** \param sem The semaphore to destroy. It must come from a previous 
**            call to \ref FAPIDRVSYS_CreateSemaphore.
**
** \sa FAPIDRVSYS_SetServices
** \sa FAPIDRVSYS_CreateSemaphore
** \sa FAPIDRVSYS_GetSemaphore
** \sa FAPIDRVSYS_SetSemaphore
*****************************************************************************
*/ 
void FAPIDRVSYS_DestroySemaphore(FAPI_SYS_SemaphoreT sem)
{
    struct semaphore *pSem;

    pSem = (struct semaphore *)sem;

    kfree(pSem);

    return;
}

/*!
*****************************************************************************
** \brief Get a semaphore.
**
** This function represents the \b default (OS-less) implementation for the 
** service function FAPIDRVSYS_ServicesT::getSemaphoreFunc. 
** It is applied until the service will be overwritten.
**
** \param sem The semaphore to get. It must come from a previous call
**            to \ref createSemaphoreFunc
**
** \param suspend
**        - #FAPI_SYS_SUSPEND if current thread should wait until the
**           semaphore is free (blocking mode).
**        - #FAPI_SYS_NO_SUSPEND if the thread should not be blocked
**          (non-blocking mode).
**        - Any other value represents the time in milliseconds to
**          (at most) wait for the semaphore (timeout mode).
** \return
** - The current value (>=0) of the semaphore if successful.
** - (-1) if the semaphore is not available (especially non-
**   blocking or timeout mode).
** - any other negative value to indicate an operating
**   environment specific error condition.
**
** \sa FAPIDRVSYS_SetServices
** \sa FAPIDRVSYS_CreateSemaphore
** \sa FAPIDRVSYS_DestroySemaphore
** \sa FAPIDRVSYS_SetSemaphore
*****************************************************************************
*/ 
int32_t FAPIDRVSYS_GetSemaphore(FAPI_SYS_SemaphoreT sem, uint32_t suspend)
{
    struct semaphore *pSem;
    uint32_t rest, step;
    int32_t  ret;

    pSem = (struct semaphore *)sem;

    if(suspend != FAPI_SYS_NO_SUSPEND) {
        if(suspend == FAPI_SYS_SUSPEND) {
            /* blocking */
            ret = down_interruptible(pSem);
        }
        else {
            rest = suspend * HZ / 1000;
            if(rest > 10) {
                step = 10;
            }
            else {
                if(rest == 0) {
                    rest = 1;
                }
                step = rest;
            }
            while(1) {
               /* non-blocking */
                ret = down_trylock(pSem);
                if(ret == 0) {
                    return 0;
                }
                else if(rest == 0) {
                    return -1;
                }
                rest -= step;
                if(rest < step) {
                    step = rest;
                }
                /* sleep */
                set_current_state(TASK_INTERRUPTIBLE);
                schedule_timeout(step);
            }
        }
    }
    else {
        /* non-blocking */
        ret = down_trylock(pSem);
        if(ret != 0) {
            ret = -1;
        }
    }
    return ret;
}

/*!
*****************************************************************************
** \brief Set a semaphore.
**
** This function represents the \b default (OS-less) implementation for the 
** service function FAPIDRVSYS_ServicesT::setSemaphoreFunc. 
** It is applied until the service will be overwritten.
**
** \param sem The semaphore to set. It must come from a previous call to
**            \ref createSemaphoreFunc.
** \param suspend
**            - #FAPI_SYS_SUSPEND if current thread should wait until the
**               semaphore is ready to sent (blocking mode).
**            - #FAPI_SYS_NO_SUSPEND if the thread should not be blocked
**              (non-blocking mode).
**            - Any other value represents the time in milliseconds to
**              (at most) wait for the semaphore to be sent (timeout mode).
**
** \sa FAPIDRVSYS_SetServices
** \sa FAPIDRVSYS_CreateSemaphore
** \sa FAPIDRVSYS_DestroySemaphore
** \sa FAPIDRVSYS_GetSemaphore
*****************************************************************************
*/
void FAPIDRVSYS_SetSemaphore(FAPI_SYS_SemaphoreT sem, uint32_t suspend)
{
    struct semaphore *pSem;

    pSem = (struct semaphore *)sem;

    up(pSem);

    return;
}

#if 1 /* FSL, for USB DMA */
/*!
*******************************************************************************
**
** \brief  create the spin lock
**
** \param  *lock  pointer for spin lock context
**
******************************************************************************/
void FAPIDRVSYS_spinLockCreate(FAPI_SYS_SpinLockT *pLock)
{
    spin_lock_init((spinlock_t *)pLock);
}

/*!
*******************************************************************************
**
** \brief  delete the spin lock
**
** \param  *lock  pointer for spin lock context
**
******************************************************************************/
void FAPIDRVSYS_spinLockDelete(FAPI_SYS_SpinLockT *pLock)
{
}

/*!
*******************************************************************************
**
** \brief  spin lock
**
** \param  *lock  pointer for spin lock context
** \param  flags  lock flag
**
******************************************************************************/
void FAPIDRVSYS_spinLock(FAPI_SYS_SpinLockT *pLock, uint32_t *pFlags)
{
    spin_lock_irqsave((spinlock_t *)pLock, *(unsigned long*)pFlags);
}

/*!
*******************************************************************************
**
** \brief  spin unlock
**
** \param  *lock  pointer for spin lock context
** \param  flags  lock flag
**
******************************************************************************/
void FAPIDRVSYS_spinUnlock(FAPI_SYS_SpinLockT *pLock, uint32_t *pFlags)
{
    spin_unlock_irqrestore((spinlock_t *)pLock, (unsigned long)*pFlags);
}
#endif

/*!
*****************************************************************************
** \brief Get physcial address.
**
** This function represents the \b default implementation for the service 
** function FAPIDRVSYS_ServicesT::getPhysAddressFunc. 
** It is applied until the service will be overwritten.
**
** \param virtAddress The virtual address to covert.
**
** \return The valid physical address.
**
** \sa FAPIDRVSYS_SetServices
** \sa FAPIDRVSYS_GetVirtAddress
*****************************************************************************
*/  
uint32_t FAPIDRVSYS_GetPhysAddress(uint32_t virtAddress)
{
    uint32_t  physAddress;

    if(virtAddress == 0) return 0;

    physAddress = IO_VIRT_TO_PHYS(virtAddress);
    if(physAddress == virtAddress) {
        struct page    *page;
        unsigned long  base;

        page = virt_to_page(virtAddress);
        base = (unsigned long)page_to_phys(page);
        physAddress = (base + (virtAddress & ~PAGE_MASK));
    }
    FAPI_SYS_PRINT_DEBUG(3, "\033[34mV2P %08x -> %08x\033[30m\n",
                            virtAddress, physAddress);
    return physAddress;
}

/*!
*****************************************************************************
** \brief Get virtual address.
**
** This function represents the \b default implementation for the service 
** function FAPIDRVSYS_ServicesT::getVirtAddressFunc. 
** It is applied until the service will be overwritten.
**
** \param physAddress The physical address to covert.
**
** \return The valid virtual address.
**
** \sa FAPIDRVSYS_SetServices
** \sa FAPIDRVSYS_GetPhysAddress
*****************************************************************************
*/  
uint32_t FAPIDRVSYS_GetVirtAddress(uint32_t physAddress)
{
    uint32_t  virtAddress;

    if(physAddress == 0) return 0;

    virtAddress = IO_PHYS_TO_VIRT(physAddress);
    
    /* NOTE: the translation of physical address to virtual address in
     * kernel address space is only valid for the memory area of kmalloc.
     * Not for the memory area of vmalloc.
     */
    if(virtAddress == physAddress) {
        virtAddress = physAddress + (PAGE_OFFSET-PHYS_OFFSET);
    }

    FAPI_SYS_PRINT_DEBUG(3, "\033[31mP2V %08x -> %08x\033[30m\n",
                            physAddress, virtAddress);
    return virtAddress;
}

//***************************************************************************
FAPI_SYS_ServicesT newSysFunctions = {
    NULL,                               /* lockSchedulerFunc    */
    NULL,                               /* unlockSchedulerFunc  */
    FAPIDRVSYS_Sleep,                   /* sleepFunc            */
    FAPIDRVSYS_DisableIrq,              /* disableIrqFunc       */
    FAPIDRVSYS_EnableIrq,               /* enableIrqFunc        */
    FAPIDRVSYS_CreateSemaphore,         /* createSemaphoreFunc  */
    FAPIDRVSYS_DestroySemaphore,        /* destroySemaphoreFunc */
    FAPIDRVSYS_GetSemaphore,            /* getSemaphoreFunc     */
    FAPIDRVSYS_SetSemaphore,            /* setSemaphoreFunc     */
#if 1 /* FSL, for USB DMA */
    FAPIDRVSYS_spinLockCreate,
    FAPIDRVSYS_spinLockDelete,
    FAPIDRVSYS_spinLock,
    FAPIDRVSYS_spinUnlock,
#endif
    NULL,                               /* createMailqueueFunc  */
    NULL,                               /* destroyMailqueueFunc */
    NULL,                               /* getMailqueueFunc     */
    NULL,                               /* setMailqueueFunc     */
    FAPIDRVSYS_GetPhysAddress,          /* getPhysAddressFunc   */
    FAPIDRVSYS_GetVirtAddress,          /* getVirtAddressFunc   */
    FAPIDRVSYS_AllocateMemory,          /* mallocFunc           */
    FAPIDRVSYS_AllocateMemoryUncached,  /* mallocUncachedFunc    */
    FAPIDRVSYS_AllocateMemory,          /* mallocCachedFunc     */
    NULL,                               /* mallocSegmentFunc    */
    FAPIDRVSYS_FreeMemory,              /* freeFunc             */
#ifndef DEBUG
    NULL,                               /* printfFunc           */
#else
    FAPIDRVSYS_Printf,                  /* printfFunc           */
#endif
    NULL                                /* initDriversFunc      */
};

/*!
*******************************************************************************
**
** \brief  Initialize process of the FAPI_SYS module.
**
** \param  void (can't set any argument)
**
******************************************************************************/
int32_t FAPISAF_InitSystemFunctions(void)
{
    FAPI_SYS_PRINT_DEBUG(1, "[SYS/FAPISAF_InitSystemFunctions]\n");

    /* register system function */
    FAPI_SYS_SetServices(&newSysFunctions);

    saf_ua_manage = FAPISAF_InitSBuffArea(
                        FAPEX_SYS_MMAP_UNCASHED_HEAP_START,
                        FAPEX_SYS_MMAP_UNCASHED_HEAP_SIZE,
                        FDSYS_UNCASHED_BUFF_ALIGNED_SIZE,
                        FDSYS_UNCASHED_BUFF_AREA_MAX);
    if(saf_ua_manage == NULL) {
        return -ENOMEM;
    }

    FAPI_SYS_PRINT_DEBUG(1, "[SYS/FAPISAF_InitSystemFunctions] FAPI_SYS is loaded ,saf_ua_mangage = %xh\n",saf_ua_manage);

    return 0;
}

/*!
*******************************************************************************
**
** \brief  Cleanup process of the FAPI_SYS module.
**
** \param  void (can't set any argument)
**
******************************************************************************/
void FAPISAF_CleanupSystemFunctions(void)
{
    FAPI_SYS_PRINT_DEBUG(1, "[SYS/FAPISAF_CleanupSystemFunctions]\n");

    FAPISAF_ExitSBuffArea(saf_ua_manage);
    saf_ua_manage = NULL;

    return;
}

#else /* __LINUX__ */

#include "rtos/rtos.h"
#include "fapi/drv_intr.h"
#include "fapi/drv_timer.h"

/*!
*******************************************************************************
**
** \brief  Initialize process of the FAPI_SYS module.
**
** \param  void (can't set any argument)
**
******************************************************************************/
int32_t FAPISAF_InitSystemFunctions(void)
{
#if (DEBUG >= 1)
    printf("[SYS/FAPISAF_InitSystemFunctions]\n");
#endif
    RTOS_InitServices();

#if (DEBUG >= 1)
    printf("[SYS/FAPISAF_InitSystemFunctions] FAPI_SYS is loaded\n");
#endif
    return 0;
}

/*!
*******************************************************************************
**
** \brief  Cleanup process of the FAPI_SYS module.
**
** \param  void (can't set any argument)
**
******************************************************************************/
void FAPISAF_CleanupSystemFunctions(void)
{
#if (DEBUG >= 1)
    printf("FAPI_SYS is removed\n");
#endif
    return;
}

#endif /* __LINUX__ */

#ifdef __LINUX__
/*!
*******************************************************************************
**
** \brief  64bit division
**
** \param  dividend     64 bits dividend
** \param  divisor      64 bits divisor
**
******************************************************************************/
int64_t FAPI_DIV64(int64_t dividend, uint64_t divisor)
{
    uint64_t result;

    if(dividend >= 0) {
        result = dividend;
    }
    else {
        result = -dividend;
    }

    do_div(result, divisor);

    if(dividend >= 0) {
        return result;
    }
    else {
        return -result;
    }
}

/*!
*******************************************************************************
**
** \brief  64bit absolute value
**
** \param  n    64 bits integer value
**
******************************************************************************/
int64_t FAPI_ABS64(int64_t n)
{
    if(n >= 0) {
        return n;
    }
    else {
        return -n;
    }
}
void FAPI_SYS_Copy4ByteStreamToWord( const uint8_t* streamPtr, uint32_t* wordPtr )
{
    *wordPtr = ( ( 0x000000FFUL & streamPtr[0] ) << 24 )
             | ( ( 0x000000FFUL & streamPtr[1] ) << 16 )
             | ( ( 0x000000FFUL & streamPtr[2] ) <<  8 )
             | ( ( 0x000000FFUL & streamPtr[3] ) <<  0 )
             ;
}

void FAPI_SYS_CopyByteStreamToWordArray( const uint8_t* streamPtr,
                                         uint32_t*      wordArray,
                                         uint32_t       wordCount )
{
    uint32_t word_index;

    streamPtr += ( wordCount * sizeof(uint32_t) );
    for( word_index=0; word_index < wordCount; word_index++ )
    {
        streamPtr -= 4;
        FAPI_SYS_Copy4ByteStreamToWord( streamPtr, &wordArray[word_index] );
    }
}

/*!
********************************************************************************
** \brief Retrieve the current memory configuration
**
** This function retrieves the current memory configuration for system and
** video memory.
**
** \return
** - #FAPI_ERR_BAD_PARAMETER if memInfoPtr points to NULL.
** - #FAPI_ERR_FEATURE_NOT_SUPPORTED in case invalid configuration from the
**                                   DDR memory interface is detected.
** - #FAPI_OK if successfull, all fields in memInfoPtr can be used.
********************************************************************************
*/
int32_t FAPI_SYS_GetMemoryInfo( FAPI_SYS_GetMemoryInfoT* memInfoPtr )
{
    uint32_t* config_register1;  // to detect memory1 size
    uint32_t* config_register2;  // to detect memory2 size
    uint32_t* ac_spec_register1; // to detect memory1 installed
    uint32_t* ac_spec_register2; // to detect memory2 installed
    uint32_t* dummy_register2;   // used to store video memory offset
    uint32_t  video_offset;      // for the current video memory offset
    uint32_t  video_percentage;  // to calculate the percentage of video usage
    uint32_t  video_min_percentage = 20; // min. allowed % of video usage
    uint32_t  video_max_percentage = 80; // max. allowed % of video usage

    if( !memInfoPtr )
    {
        return( FAPI_ERR_BAD_PARAMETER );
    }

#if defined(MB86H61)

    /*lint -save -e923 */
    config_register1  = (uint32_t*)FREG_DDR_RAMIF_S_SDRAM_TYPE;
    ac_spec_register1 = (uint32_t*)FREG_DDR_RAMIF_S_AC_SPEC_2;
    config_register2  = (uint32_t*)FREG_DDR_RAMIF_V_SDRAM_TYPE;
    ac_spec_register2 = (uint32_t*)FREG_DDR_RAMIF_V_AC_SPEC_2;
    dummy_register2   = (uint32_t*)FREG_DDR_RAMIF_V_DUMMY_FF;
    /*lint -restore */

#endif

#if defined(MB86H615)

    /*lint -save -e923 */
    config_register1  = 0;
    ac_spec_register1 = 0;
    config_register2  = (uint32_t*)FREG_DDR_RAMIF_V_SDRAM_TYPE;
    ac_spec_register2 = (uint32_t*)FREG_DDR_RAMIF_V_AC_SPEC_2;
    dummy_register2   = (uint32_t*)FREG_DDR_RAMIF_V_DUMMY_FF;
    /*lint -restore */

#endif

    memset( memInfoPtr, 0, sizeof(FAPI_SYS_GetMemoryInfoT) );

#if defined(MB86H61)

    if( *ac_spec_register1 != 0 )
    {
        memInfoPtr->physicalMemory1Start = 0x20000000UL;
        switch( *config_register1 & 0xFUL )
        {
            case 0x0UL: memInfoPtr->physicalMemory1Bytes =  32 << 20; break;
            case 0x1UL: memInfoPtr->physicalMemory1Bytes =  64 << 20; break;
            case 0x2UL: memInfoPtr->physicalMemory1Bytes = 128 << 20; break;
            case 0x3UL: memInfoPtr->physicalMemory1Bytes = 256 << 20; break;
            default:
                memset( memInfoPtr, 0, sizeof(FAPI_SYS_GetMemoryInfoT) );
                return( FAPI_ERR_FEATURE_NOT_SUPPORTED );
        }
    }

#endif

#if defined(MB86H615)

    config_register1  = config_register1;
    ac_spec_register1 = ac_spec_register1;

#endif

    if(  *ac_spec_register2 != 0 )
    {
        memInfoPtr->physicalMemory2Start = 0x40000000UL;
        switch( *config_register2 & 0xFUL )
        {
            case 0x0UL: memInfoPtr->physicalMemory2Bytes =  32 << 20; break;
            case 0x1UL: memInfoPtr->physicalMemory2Bytes =  64 << 20; break;
            case 0x2UL: memInfoPtr->physicalMemory2Bytes = 128 << 20; break;
            case 0x3UL: memInfoPtr->physicalMemory2Bytes = 256 << 20; break;
            default:
                memset( memInfoPtr, 0, sizeof(FAPI_SYS_GetMemoryInfoT) );
                return( FAPI_ERR_FEATURE_NOT_SUPPORTED );
        }
    }

    // both memory interfaces are used and configured by RVS, the first
    // interface is used as system (applictaion) memory, the second interface
    // is used for video decoding
    if( ( memInfoPtr->physicalMemory1Start != 0 )
    &&  ( memInfoPtr->physicalMemory2Start != 0 ) )
    {
        memInfoPtr->systemMemoryStart = memInfoPtr->physicalMemory1Start;
        memInfoPtr->systemMemoryBytes = memInfoPtr->physicalMemory1Bytes;
        memInfoPtr->videoMemoryStart  = memInfoPtr->physicalMemory2Start;
        memInfoPtr->videoMemoryBytes  = memInfoPtr->physicalMemory2Bytes;
    }
    // only the second memory interfcae is configured to host both
    // the system memory (for applications) as well as the videoMemoryBytes
    // memory used by the video decoder
    // allowed values are 20..80% of physical memory2 for video usage
    else if( ( memInfoPtr->physicalMemory1Start == 0 )
         &&  ( memInfoPtr->physicalMemory2Start != 0 ) )
    {
        video_offset     = (*dummy_register2 & 0x0000FFFFUL) << 20;
        video_percentage = ( video_offset * 100 ) / memInfoPtr->physicalMemory2Bytes;
        if( ( video_percentage < video_min_percentage )
        ||  ( video_percentage > video_max_percentage ) )
        {
            memset( memInfoPtr, 0, sizeof(FAPI_SYS_GetMemoryInfoT) );
            return( FAPI_ERR_FEATURE_NOT_SUPPORTED );
        }

        memInfoPtr->videoMemoryOffset = video_offset;
        memInfoPtr->systemMemoryStart = memInfoPtr->physicalMemory2Start;
        memInfoPtr->systemMemoryBytes = memInfoPtr->videoMemoryOffset;
        memInfoPtr->videoMemoryStart  = memInfoPtr->physicalMemory2Start + video_offset;
        memInfoPtr->videoMemoryBytes  = memInfoPtr->physicalMemory2Bytes - video_offset;
    }
    // error condition, it is not allowed to configure only the first memory
    // interface, for a single memory system the second memory interface must
    // be used to host system and video memory
    else if( ( memInfoPtr->physicalMemory1Start != 0 )
         &&  ( memInfoPtr->physicalMemory2Start == 0 ) )
    {
        return( FAPI_ERR_FEATURE_NOT_SUPPORTED );
    }
    // error condition, it is not allowed to configure neither the first memory
    // interface nor the second memory interface
    else if( ( memInfoPtr->physicalMemory1Start == 0 )
         &&  ( memInfoPtr->physicalMemory2Start == 0 ) )
    {
        return( FAPI_ERR_FEATURE_NOT_SUPPORTED );
    }

    return( FAPI_OK );
}

/*******************************
 MACRO (Linux dep.)
********************************/
#ifdef BUILD_AS_MODULE
module_init(FAPISAF_InitSystemFunctions);
module_exit(FAPISAF_CleanupSystemFunctions);

MODULE_LICENSE("Proprietary. Fujitsu Semiconductor.");
MODULE_DESCRIPTION("FAPI System Abstraction Layer (Linux extension).");

EXPORT_SYMBOL(FAPISAF_InitSBuffArea);
EXPORT_SYMBOL(FAPISAF_ExitSBuffArea);
EXPORT_SYMBOL(FAPISAF_AllocSBuffArea);
EXPORT_SYMBOL(FAPISAF_FreeSBuffArea);
EXPORT_SYMBOL(FAPI_SYS_GetErrorString);
EXPORT_SYMBOL(FAPI_SYS_Services);
EXPORT_SYMBOL(FAPI_DIV64);
EXPORT_SYMBOL(FAPI_ABS64);
EXPORT_SYMBOL(FAPI_SYS_InvalidateDataCache);
EXPORT_SYMBOL(FAPI_SYS_FlushDataCache);
EXPORT_SYMBOL(FAPI_SYS_InvalidateInstructionCache);
EXPORT_SYMBOL(FAPI_SYS_GetDeviceId);
EXPORT_SYMBOL(FAPI_SYS_GetDeviceName);
EXPORT_SYMBOL(FAPI_SYS_SetMasterCpu);
EXPORT_SYMBOL(FAPI_SYS_IsMasterCpu);
EXPORT_SYMBOL(FAPI_SYS_CopyByteStreamToWordArray);
EXPORT_SYMBOL(FAPI_SYS_Copy4ByteStreamToWord);
EXPORT_SYMBOL(FAPI_SYS_GetMemoryInfo);
#endif /* BUILD_AS_MODULE */

#endif /* __LINUX__ */
