/*
 *
 * \attention  This program is provided as is. You can redistribute it and/or
 * modify it. Fujitsu Semiconductor Limited accepts no responsibility or
 * liability for any errors or omissions.
 *
 * Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
 */
/*
 CFDL module
 system abstraction
*/

#include "fapex.h"
#include "fapex/abst_error.h"
#include "fapex_cfdl_priv.h"

#ifndef __LINUX__

#include "fapex/abst_types.h"

#define FAPEX_SYS_SLEEP_TIME  1   /* [ms] */

#else /* !__LINUX__ */
#include <linux/slab.h>    /* kmalloc */
#include <linux/kernel.h>  /* printk */
#include <linux/wait.h>    /* wait queue */
#include <asm/uaccess.h>   /* copy user<->kernel */
#include <asm/page.h>      /* paddr<->vaddr */
#include <linux/semaphore.h> /* semaphore */
#include <linux/poll.h>    /* poll */
#include <linux/ioport.h>
#include <linux/spinlock.h>
#include <asm/io.h>
#endif /* !__LINUX__ */

/*!
*******************************************************************************
**
** \brief  Get a device identifier of the driver internal.
**
** On Linux, inode is file descripter. On RTOS, may be, inode is same as id.
**
** \param  inode    On Linux, inode, On RTOS, device identifier
**
******************************************************************************/
int32_t FAPEX_SYS_getID(FAPEXinode_S inode)
{
#ifndef __LINUX__
    return inode;
#else /* !__LINUX__ */
    return MINOR(inode->i_rdev);
#endif /* !__LINUX__ */
}

void FAPEX_SYS_clearMemory(void *ptr, int32_t size)
{
    uint8_t *tmpptr;
    int32_t count;

    tmpptr = (uint8_t*)(ptr);

    for(count=0;count<size;count++){
        *tmpptr = 0;
        tmpptr++;
    }
    return;
}

/*!
*******************************************************************************
**
** \brief  Create event.
**
** \param  *pIvt   event management structure
**
******************************************************************************/
int32_t FAPEX_SYS_createEvent(FAPEXCFDLSAF_EventStrT *pIvt)
{
    /* create spin lock */
    FAPEXCFDLSAF_SPIN_LOCK_CREATE( &pIvt->lock );

#ifndef __LINUX__

    /* create mail_box */
    pIvt->ivt = RTOS_CreateMailqueue(8, sizeof(uint32_t));
    if(pIvt->ivt == NULL) {
        return -1;
    }
    pIvt->wai_bitptn = 0;
    pIvt->evt_bitptn = 0;
    pIvt->mode       = 0;
    pIvt->read_idx   = 0;
    pIvt->write_idx  = 0;
    pIvt->tout       = 0;

#else /* !__LINUX__ */

    /* create wait_queue */
    init_waitqueue_head(&(pIvt->ivt));

#endif /* !__LINUX__ */

    FAPEXDebugPrint(4, "[CFDL/FAPEX_SYS_createEvent]\n");

    pIvt->initialized = 1;

    return 0;
}

/*!
*******************************************************************************
**
** \brief  Release event.
**
** \param  *pIvt   event management structure
**
******************************************************************************/
int32_t FAPEX_SYS_deleteEvent(FAPEXCFDLSAF_EventStrT *pIvt)
{
    pIvt->initialized = 0;

#ifndef __LINUX__

    if((pIvt->wai_bitptn != 0) && (pIvt->tout == 0)) {
        uint32_t event_bitptn;

        event_bitptn     = pIvt->wai_bitptn;
        pIvt->evt_bitptn = event_bitptn;

        RTOS_SetMailqueue(pIvt->ivt, &event_bitptn, 0, RTOS_NO_SUSPEND);

        FAPI_SYS_SLEEP( 100 );
    }

    /* delete mail_box */
    RTOS_DestroyMailqueue( pIvt->ivt );

#else /* !__LINUX__ */

    /* delete wait_queue */

#endif /* !__LINUX__ */

    /* delate spin lock */
    FAPEXCFDLSAF_SPIN_LOCK_DELETE( &pIvt->lock );

    FAPEXDebugPrint(4, "[CFDL/FAPEX_SYS_deleteEvent]\n");

    return 0;
}

/*!
*******************************************************************************
**
** \brief  Wait event for queue
**
** \param  *pIvt  event management structure
** \param  tout   timeout [msec] (0 : without timeout)
**
******************************************************************************/
int32_t fapex_sys_waitEventQueue(FAPEXCFDLSAF_EventStrT *pIvt, uint32_t tout)
{
    int32_t  ret;
#ifndef __LINUX__
    uint32_t  receive_data;
#endif

    if(pIvt->mode & FAPEXCFDLSAF_WAITEVENT_ASYNC) {
        /* poll event */
        if(pIvt->read_idx == pIvt->write_idx) {
            /* poll_event/unmatch */
            return FAPI_ERR_DEVICE_BUSY;
        }
    }
    else {
        /* wait queue event */
        if(tout != 0) {
            /* with timeout */
#ifndef __LINUX__
            while(1) {
                ret = RTOS_GetMailqueue(pIvt->ivt, (void *)&receive_data, tout);
                if(ret == 0) {
                    /* timeout */
                    return FAPI_ERR_TIMEOUT;
                }
                if(pIvt->read_idx != pIvt->write_idx) {
                    break;
                }
            }
#else
            ret = wait_event_interruptible_timeout(
                    pIvt->ivt,
                    (pIvt->read_idx != pIvt->write_idx),
                    tout);
            if(ret == 0) {
                /* timeout */
                return FAPI_ERR_TIMEOUT;
            }
            else if(ret == -ERESTARTSYS) {
                /* error */
                return FAPI_ERR_BAD_PARAMETER;
            }
#endif /* !__LINUX__ */
        }
        else {
            /* without timeout */
#ifndef __LINUX__
            while(1) {
                ret = RTOS_GetMailqueue(pIvt->ivt, (void *)&receive_data, RTOS_SUSPEND);
                if(ret == 0) {
                    /* error */
                    return FAPI_ERR_BAD_PARAMETER;
                }
                if(pIvt->read_idx != pIvt->write_idx) {
                    break;
                }
            }
#else
            /* without timeout */
            ret = wait_event_interruptible(
                    pIvt->ivt,
                    (pIvt->read_idx != pIvt->write_idx));
            if(ret != 0) {
                /* error */
                return FAPI_ERR_BAD_PARAMETER;
            }
#endif /* !__LINUX__ */
        }
    }
    return 0;
}

/*!
*******************************************************************************
**
** \brief  Wait event (AND type)
**
** \param  *pIvt  event management structure
** \param  tout   timeout [msec] (0 : without timeout)
**
******************************************************************************/
int32_t fapex_sys_waitEventAnd(FAPEXCFDLSAF_EventStrT *pIvt, uint32_t tout)
{
    int32_t   ret = 0;
    uint32_t  wai_bitptn = pIvt->wai_bitptn;
#ifndef __LINUX__
    uint32_t  receive_data;
#endif

    if(pIvt->mode & FAPEXCFDLSAF_WAITEVENT_ASYNC) {
        /* poll event */
        if((wai_bitptn & (pIvt->evt_bitptn)) != wai_bitptn) {
            /* poll_event/unmatch */
            return FAPI_ERR_DEVICE_BUSY;
        }
#ifndef __LINUX__
        receive_data = pIvt->evt_bitptn;
#endif
    }
    else {
        /* wait event */
        if(tout != 0) {
            /* with timeout */
#ifndef __LINUX__
            while(1) {
                ret = RTOS_GetMailqueue(pIvt->ivt, (void *)&receive_data, tout);
                if(ret == 0) {
                    /* timeout */
                    return FAPI_ERR_TIMEOUT;
                }
                if((wai_bitptn & receive_data) == wai_bitptn) {
                    break;
                }
            }
#else
            ret = wait_event_interruptible_timeout(
                    pIvt->ivt,
                    ((wai_bitptn & (pIvt->evt_bitptn)) == wai_bitptn),
                    tout);
            if(ret == 0) {
                /* timeout */
                return FAPI_ERR_TIMEOUT;
            }
            else if(ret == -ERESTARTSYS) {
                /* error */
                return FAPI_ERR_BAD_PARAMETER;
            }
#endif /* !__LINUX__ */
        }
        else {
            /* without timeout */
#ifndef __LINUX__
            while(1) {
                ret = RTOS_GetMailqueue(pIvt->ivt, (void *)&receive_data, RTOS_SUSPEND);
                if(ret == 0) {
                    /* error */
                    return FAPI_ERR_BAD_PARAMETER;
                }
                if((wai_bitptn & receive_data) == wai_bitptn) {
                    break;
                }
            }
#else
            /* without timeout */
            ret = wait_event_interruptible(
                    pIvt->ivt,
                    ((wai_bitptn & (pIvt->evt_bitptn)) == wai_bitptn));
            if(ret != 0) {
                /* error */
                return FAPI_ERR_BAD_PARAMETER;
            }
#endif /* !__LINUX__ */
        }
    }
#ifndef __LINUX__
    ret = (int32_t)receive_data;
#endif
    return ret;
}

/*!
*******************************************************************************
**
** \brief  Wait event (OR type)
**
** \param  *pIvt  event management structure
** \param  tout   timeout [msec] (0 : without timeout)
**
******************************************************************************/
int32_t fapex_sys_waitEventOr(FAPEXCFDLSAF_EventStrT *pIvt, uint32_t tout)
{
    int32_t   ret = 0;
    uint32_t  wai_bitptn = pIvt->wai_bitptn;
#ifndef __LINUX__
    uint32_t  receive_data;
#endif

    if(pIvt->mode & FAPEXCFDLSAF_WAITEVENT_ASYNC) {
        /* poll event */
        if((wai_bitptn & (pIvt->evt_bitptn)) == 0) {
            /* poll_event/unmatch */
            return FAPI_ERR_DEVICE_BUSY;
        }
#ifndef __LINUX__
        receive_data = pIvt->evt_bitptn;
#endif
    }
    else {
        /* wait event */
        if(tout != 0) {
            /* with timeout */
#ifndef __LINUX__
            while(1) {
                ret = RTOS_GetMailqueue(pIvt->ivt, (void *)&receive_data, tout);
                if(ret == 0) {
                    /* timeout */
                    return FAPI_ERR_TIMEOUT;
                }
                if(wai_bitptn & receive_data) {
                    break;
                }
            }
#else
            ret = wait_event_interruptible_timeout(
                    pIvt->ivt,
                    (wai_bitptn & (pIvt->evt_bitptn)),
                    tout);
            if(ret == 0) {
                /* timeout */
                return FAPI_ERR_TIMEOUT;
            }
            else if(ret == -ERESTARTSYS) {
                /* error */
                return FAPI_ERR_BAD_PARAMETER;
            }
#endif /* !__LINUX__ */
        }
        else {
            /* without timeout */
#ifndef __LINUX__
            while(1) {
                ret = RTOS_GetMailqueue(pIvt->ivt, (void *)&receive_data, RTOS_SUSPEND);
                if(ret == 0) {
                    /* error */
                    return FAPI_ERR_BAD_PARAMETER;
                }
                if(wai_bitptn & receive_data) {
                    break;
                }
            }
#else
            ret = wait_event_interruptible(
                    pIvt->ivt,
                    (wai_bitptn & (pIvt->evt_bitptn)));
            if(ret != 0) {
                /* error */
                return FAPI_ERR_BAD_PARAMETER;
            }
#endif /* !__LINUX__ */
        }
    }
#ifndef __LINUX__
    ret = (int32_t)receive_data;
#endif
    return ret;
}

/*!
*******************************************************************************
**
** \brief  Wait event.
**
** \param  *pIvt   event management structure
**
******************************************************************************/
int32_t FAPEX_SYS_waitEvent(FAPEXCFDLSAF_EventStrT *pIvt)
{
    int32_t   ret;
    uint32_t  mode = pIvt->mode;
    uint32_t  tout;
    uint32_t  flags;
#ifndef __LINUX__
    tout = pIvt->tout;
#else
    tout = msecs_to_jiffies(pIvt->tout);
#endif

    FAPEXDebugPrint(4, "[CFDL/FAPEX_SYS_waitEvent]\n");

    if(mode & FAPEXCFDLSAF_WAITEVENT_QUEUE) {
        return fapex_sys_waitEventQueue( pIvt, tout );
    }
    else {
        if(mode & FAPEXCFDLSAF_WAITEVENT_AND) {
            ret = fapex_sys_waitEventAnd( pIvt, tout );
        }
        else {
            ret = fapex_sys_waitEventOr( pIvt, tout );
        }

        if(ret >= 0) {
            /*** LOCK ***/
            FAPEXCFDLSAF_SPIN_LOCK( &pIvt->lock, &flags );

#ifdef __LINUX__
            ret = (int32_t)pIvt->evt_bitptn;
#endif
            pIvt->evt_bitptn = ret & ~(pIvt->wai_bitptn);

            /*** UNLOCK ***/
            FAPEXCFDLSAF_SPIN_UNLOCK( &pIvt->lock, &flags );
        }
        return ret;
    }
}

/*!
*******************************************************************************
**
** \brief  Set (inform) event.
**
** \param  *pIvt       event management structure
** \param  set_bitptn  a bit pattern when it inform a event
** \param  set_mode    a mode when it set a bit flag (and/or/clear set)
**
******************************************************************************/
int32_t FAPEX_SYS_setEvent(FAPEXCFDLSAF_EventStrT *pIvt, uint32_t set_bitptn,
                           uint32_t set_mode)
{
#ifndef __LINUX__
    int32_t  ret;
#endif /* !__LINUX__ */
    uint32_t flags;
    uint32_t event_bitptn;

    /*** LOCK ***/
    FAPEXCFDLSAF_SPIN_LOCK( &pIvt->lock, &flags );

    switch(set_mode){
        case FAPEXCFDLSAF_WAITEVENT_OR:
            event_bitptn = pIvt->evt_bitptn | set_bitptn;
            break;
        case FAPEXCFDLSAF_WAITEVENT_AND:
            event_bitptn = pIvt->evt_bitptn & set_bitptn;
            break;
        case FAPEXCFDLSAF_WAITEVENT_QUEUE_WRITE:
            event_bitptn = set_bitptn;
            pIvt->write_idx ++;
            break;
        case FAPEXCFDLSAF_WAITEVENT_QUEUE_READ:
            event_bitptn = set_bitptn;
            pIvt->read_idx ++;
            break;
        default:
            event_bitptn = set_bitptn;
    }
    pIvt->evt_bitptn = event_bitptn;

    /*** UNLOCK ***/
    FAPEXCFDLSAF_SPIN_UNLOCK( &pIvt->lock, &flags );

    if(event_bitptn == 0) {
        /* nothing event */
        return 0;
    }

#ifndef __LINUX__

    ret = RTOS_SetMailqueue(pIvt->ivt, &event_bitptn, 0, RTOS_NO_SUSPEND);
    if(ret == 0) {
        return -1;
    }

#else

    wake_up_interruptible(&(pIvt->ivt));

#endif /* !__LINUX__ */
    return 0;
}

/*!
*******************************************************************************
**
** \brief  Poll event.
** This function wait all devices which is set by user. Waiting is released if 
** there is even one of the specified devices event. 
** On Linux, use poll_wait(). On RTOS, may be, use same event flag for waiting, 
** and have event status separately (indepent on flag).
**
** \param  *pIvt_read       event management structure (for read)
** \param  *pIvt_write      event management structure (for write)
** \param  ptbl             internal data. On RTOS, don't care.
**
******************************************************************************/
int32_t FAPEX_SYS_pollEvent(FAPEXfile_S *file,
                            FAPEXCFDLSAF_EventStrT *pIvt_read,
                            FAPEXCFDLSAF_EventStrT *pIvt_write,
                            FAPEXFDLPLARG2_T ptbl)
{
#ifndef __LINUX__

    if(pIvt_read != NULL){
        FAPEXDebugPrint(4, "pIvt_read->wai_bitptn:%d\npIvt_read->evt_bitptn:%d\n",
                           pIvt_read->wai_bitptn, pIvt_read->evt_bitptn);
    }
    if(pIvt_write != NULL){
        FAPEXDebugPrint(4, "pIvt_write->wai_bitptn:%d\npIvt_write->evt_bitptn:%d\n",
                           pIvt_write->wai_bitptn, pIvt_write->evt_bitptn);
    }

#else /* !__LINUX__ */

    if(pIvt_read != NULL){
        FAPEXDebugPrint(4, "pIvt_read->wai_bitptn:%d\npIvt_read->evt_bitptn:%d\n",
                            pIvt_read->wai_bitptn, pIvt_read->evt_bitptn);

        poll_wait(file, &(pIvt_read->ivt), ptbl);
    }
    if(pIvt_write != NULL){
        FAPEXDebugPrint(4, "pIvt_write->wai_bitptn:%d\npIvt_write->evt_bitptn:%d\n",
                           pIvt_write->wai_bitptn, pIvt_write->evt_bitptn);

        poll_wait(file, &(pIvt_write->ivt), ptbl);
    }

#endif /* __LINUX__ */

    FAPEXDebugPrint(4, "[CFDL/FAPEX_SYS_pollEvent] return\n");

    return 0;
}

/*!
*******************************************************************************
**
** \brief  memory copy (kernel -> user).
**
** On operating system without the restriction, simply copies.
**
** \param  *usr_addr  user address
** \param  *sys_addr  system address
** \param  byte       size
**
******************************************************************************/
int32_t FAPEX_SYS_copyToUser(void *usr_addr, void *sys_addr, uint32_t byte)
{
#ifndef __LINUX__
    uint32_t count;
    uint8_t *psys,*pusr;

    psys = (uint8_t*)sys_addr;
    pusr = (uint8_t*)usr_addr;
    for(count=0; count<byte; count++) {
        *pusr = *psys;
        psys++;
        pusr++;
    }
    return 0;
#else /* !__LINUX__ */
    int32_t ret;

/* Todo: check addr space */
    ret = copy_to_user(usr_addr, sys_addr, byte);

    FAPEXDebugPrint(4, "[CFDL/FAPEX_SYS_copyToUser]\n");

    return ret;
#endif /* !__LINUX__ */
}

/*!
*******************************************************************************
**
** \brief  memory copy (user -> kernel).
**
** On operating system without the restriction, simply copies.
**
** \param  *usr_addr  user address
** \param  *sys_addr  system address
** \param  byte       size
**
******************************************************************************/
int32_t FAPEX_SYS_copyFromUser(void *sys_addr, void *usr_addr, uint32_t byte)
{
#ifndef __LINUX__
    uint32_t count;
    uint8_t *psys,*pusr;

    psys = (uint8_t*)sys_addr;
    pusr = (uint8_t*)usr_addr;
    for(count=0; count<byte; count++) {
        *psys = *pusr;
        psys++;
        pusr++;
    }
    return 0;
#else /* !__LINUX__ */
    int32_t ret;

/* Todo: check addr space */
    ret = copy_from_user(sys_addr, usr_addr, byte);

    FAPEXDebugPrint(4, "[CFDL/FAPEX_SYS_copyFromUser]\n");

    return ret;
#endif /* !__LINUX__ */
}

/*!
*******************************************************************************
**
** \brief  create the spin lock
**
** \param  *lock  pointer for spin lock context
**
******************************************************************************/
void FAPEX_SYS_spinLockCreate(FAPEXCFDLSAF_SpnLockStrT *pLock)
{
#ifndef __LINUX__
    *pLock = FAPEX_SYS_CreateSemaphore(1);
    return;
#else
    spin_lock_init(pLock);

#endif /* !__LINUX__ */

    FAPEXDebugPrint(4, "[CFDL/FAPEX_SYS_spinLockCreate]\n");
}

/*!
*******************************************************************************
**
** \brief  delete the spin lock
**
** \param  *lock  pointer for spin lock context
**
******************************************************************************/
void FAPEX_SYS_spinLockDelete(FAPEXCFDLSAF_SpnLockStrT *pLock)
{
#ifndef __LINUX__
    FAPEX_SYS_DestroySemaphore(pLock);
    *pLock = 0;
#endif /* !__LINUX__ */

    FAPEXDebugPrint(4, "[CFDL/FAPEX_SYS_spinLockDelete]\n");
}

/*!
*******************************************************************************
**
** \brief  spin lock
**
** \param  *lock  pointer for spin lock context
** \param  flags  lock flag
**
******************************************************************************/
void FAPEX_SYS_spinLock(FAPEXCFDLSAF_SpnLockStrT *pLock, uint32_t *pFlags)
{
#ifndef __LINUX__
    FAPEX_SYS_GetSemaphore( pLock, FAPEX_SYS_NO_SUSPEND );
    return;
#else
    spin_lock_irqsave(pLock, *(unsigned long*)pFlags);

#endif /* !__LINUX__ */

    FAPEXDebugPrint(4, "[CFDL/FAPEX_SYS_spinLock]\n");
}

/*!
*******************************************************************************
**
** \brief  spin unlock
**
** \param  *lock  pointer for spin lock context
** \param  flags  lock flag
**
******************************************************************************/
void FAPEX_SYS_spinUnlock(FAPEXCFDLSAF_SpnLockStrT *pLock, uint32_t *pFlags)
{
#ifndef __LINUX__
    FAPEX_SYS_SetSemaphore( pLock, FAPEX_SYS_NO_SUSPEND );
    return;
#else
    spin_unlock_irqrestore(pLock, (unsigned long)*pFlags);

#endif /* !__LINUX__ */

    FAPEXDebugPrint(4, "[CFDL/FAPEX_SYS_spinUnlock]\n");
}

/* definition of the FAPEX System Abstraction Functions (SAF) */
FAPEXCFDLSAF_ServicesT FAPEXCFDLSAF_Services_EXT = {
    FAPEX_SYS_getID,            // getid
    FAPEX_SYS_clearMemory,      // clear_memory
    FAPEX_SYS_createEvent,      // event_create
    FAPEX_SYS_deleteEvent,      // event_delete
    FAPEX_SYS_waitEvent,        // event_wait
    FAPEX_SYS_setEvent,         // event_set
    FAPEX_SYS_pollEvent,        // event_poll
    FAPEX_SYS_copyToUser,       // copy_to_user
    FAPEX_SYS_copyFromUser,     // copy_to_drv
    FAPEX_SYS_spinLockCreate,   // spinlock_create
    FAPEX_SYS_spinLockDelete,   // spinlock_delete
    FAPEX_SYS_spinLock,         // spinlock
    FAPEX_SYS_spinUnlock        // spinunlock
};

#ifdef __LINUX__

/*******************************
 MACRO (Linux dep.)
********************************/
#ifdef BUILD_AS_MODULE
//EXPORT_SYMBOL(FAPEX_SYS_getID);
//EXPORT_SYMBOL(FAPEX_SYS_clearMemory);
//EXPORT_SYMBOL(FAPEX_SYS_createEvent);
//EXPORT_SYMBOL(FAPEX_SYS_deleteEvent);
//EXPORT_SYMBOL(FAPEX_SYS_waitEvent);
//EXPORT_SYMBOL(FAPEX_SYS_setEvent);
//EXPORT_SYMBOL(FAPEX_SYS_pollEvent);
//EXPORT_SYMBOL(FAPEX_SYS_copyToUser);
//EXPORT_SYMBOL(FAPEX_SYS_copyFromUser);
#endif /* BUILD_AS_MODULE */
#endif /* __LINUX__ */
