/*!
********************************************************************************
**
** \file        ./boards/fmc/mb86h61_pdkv1/mb86h61_pdkv1_gpio_cisl.c
**
** \version     $Id: mb86h61_pdkv1_gpio_cisl.c  $
**
** \brief       Function to initialize some of the dev.kit GPIOs for 'CiSlave'
**              
**              The GPIOs of the J700 (digital RGB connector) are configured
**              to operate as a 'CI-Module CPU-Bus-Interface' 
**              There exists a special 'CI-ModuleEmulatorProbe-PCB'
**              which can be attached to this connector.
**
** \note        this source-file is usually not 'linked into' applications
**              and is primarily intended for test-purpose
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU MICROELECTRONICS
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Microelectronics Europe GmbH
** (C) Copyright 2010 by Fujitsu Semiconductor Europe GmbH
**
********************************************************************************
*/

/*lint -save -e923 */

#include <fapi/reg_gpio.h>
#include <fapi/drv_gpio.h>

#include <boards/boards_common.h>          // some defines needed for 'mb86h61_dk01.h'
#include <boards/boards_com_dirx.h>
#include <boards/com_cisl_access.h>
#include <boards/mb86h61_pdkv1.h>

//#include "btestlib/btest.h"            // for diagnostic-logger

#define OPIN_CI_DET     69      // NOTE: we use GPIO-Ouput 'active-0' to simulate CI-Adapter was inserted
#define IPIN_CI_RES     86
#define IPIN_CI_CS1     76
#define OPIN_CI_WT      88      // 
#define OPIN_CI_IRQ     82      // 
#define IPIN_CI_IOWR    79      // 
#define IPIN_CI_IORD    78      // 
#define IPIN_CI_M_WE    81      // 
#define IPIN_CI_M_OE    77      // 
#define IOPIN_CI_D7     75      // 
#define IOPIN_CI_D6     74      // 
#define IOPIN_CI_D5     73      // 
#define IOPIN_CI_D4     72      // 
#define IOPIN_CI_D3     70      // 
#define IOPIN_CI_D2     95      // 
#define IOPIN_CI_D1     94      // 
#define IOPIN_CI_D0     93      // 
#define IPIN_CI_A0      92      // 
#define IPIN_CI_A1      91      // 
#define IPIN_CI_A2      90      // 
#define IPIN_CI_A3      89      // 
#define IPIN_CI_A4      87      // 
#define IPIN_CI_A5      85      // 
#define IPIN_CI_A6      84      // 
#define IPIN_CI_A7      83      // 
#define IPIN_CI_A8      80      // 

static void BOARD_FMC_H61PDKV1_SimulateCiOutIn(uint32_t flgOutOrIn);
static uint32_t BOARD_FMC_H61PDKV1_CtrlPollWaitPin(uint32_t flgNotOrActive);
static uint32_t BOARD_FMC_H61PDKV1_PollBackIrqOutpPin(void);

static const char_t INFOSTR_CiSlaveHw[] = "H61PDevKit-CiSlaveAdapter";
static BRDCOM_USI_CiSlaveHwCtrlT  BOARD_FMC_H61PDKV1_CiSlaveInterface = {
 /* CiSlaveTargetHwInfoStrPtr */    INFOSTR_CiSlaveHw
,/* SimCamOutInHook           */    BOARD_FMC_H61PDKV1_SimulateCiOutIn
,/* CtrlWaitSignalPin         */    BOARD_FMC_H61PDKV1_CtrlPollWaitPin
,/* PollBackIrqSignalPin      */    BOARD_FMC_H61PDKV1_PollBackIrqOutpPin
};

static  BRDCOM_DX_GpioInputConfigDataT  InpPins_CiSlave[] = {
 { 2+IPIN_CI_RES,  BDEF_GPIO_IN_USI_RESET   }
,{ 2+IPIN_CI_CS1,  BDEF_GPIO_IN_USI_IO_CS0  }
,{ 1,              BDEF_GPIO_IN_USI_IO_CS1  }   // NOTE: for CI-Emu.'CS1' must be internally set to '1'
,{ 2+IPIN_CI_IOWR, BDEF_GPIO_IN_USI_IO_WR   }
,{ 2+IPIN_CI_IORD, BDEF_GPIO_IN_USI_IO_RD   }
,{ 2+IPIN_CI_M_WE, BDEF_GPIO_IN_USI_ADDR_2  }   // NOTE: for IR_RD/WR, the level of 'WE,OE==1' is still
,{ 2+IPIN_CI_M_OE, BDEF_GPIO_IN_USI_DMA_ACK }   //       used by HW as 'IO-Addr2', so R4..7 should be used !!!
,{ 2+IOPIN_CI_D7,  BDEF_GPIO_IN_USI_DATA_7  }
,{ 2+IOPIN_CI_D6,  BDEF_GPIO_IN_USI_DATA_6  }
,{ 2+IOPIN_CI_D5,  BDEF_GPIO_IN_USI_DATA_5  }
,{ 2+IOPIN_CI_D4,  BDEF_GPIO_IN_USI_DATA_4  }
,{ 2+IOPIN_CI_D3,  BDEF_GPIO_IN_USI_DATA_3  }
,{ 2+IOPIN_CI_D2,  BDEF_GPIO_IN_USI_DATA_2  }
,{ 2+IOPIN_CI_D1,  BDEF_GPIO_IN_USI_DATA_1  }
,{ 2+IOPIN_CI_D0,  BDEF_GPIO_IN_USI_DATA_0  }
,{ 2+IPIN_CI_A0,   BDEF_GPIO_IN_USI_ADDR_0  }     // I/O-Address A0
,{ 2+IPIN_CI_A1,   BDEF_GPIO_IN_USI_ADDR_1  }     // I/O-Address A1
,{ 2+IPIN_CI_A1,   BDEF_GPIO_IN_USI_DATA_8  }     // MEM-Address A1
,{ 2+IPIN_CI_A2,   BDEF_GPIO_IN_USI_DATA_9  }     // MEM-Address A2
,{ 2+IPIN_CI_A3,   BDEF_GPIO_IN_USI_DATA_10 }     // MEM-Address A3
,{ 2+IPIN_CI_A4,   BDEF_GPIO_IN_USI_DATA_11 }     // MEM-Address A4
,{ 2+IPIN_CI_A5,   BDEF_GPIO_IN_USI_DATA_12 }     // MEM-Address A5
,{ 2+IPIN_CI_A6,   BDEF_GPIO_IN_USI_DATA_13 }     // MEM-Address A6
,{ 2+IPIN_CI_A7,   BDEF_GPIO_IN_USI_DATA_14 }     // MEM-Address A7
,{ 2+IPIN_CI_A8,   BDEF_GPIO_IN_USI_DATA_15 }     // MEM-Address A8
};
#define N_INP_PINS (sizeof(InpPins_CiSlave) / sizeof(BRDCOM_DX_GpioInputConfigDataT))

static  BRDCOM_DX_GpioOutputConfigDataT  OutPins_CiSlave[] = {
 { OPIN_CI_DET, 0,                  BDEF_GPIO_OUT_1          }  // default: simulate CAM extracted
,{ OPIN_CI_WT,  0,                  BDEF_GPIO_OUT_USI_WAIT   }
,{ OPIN_CI_IRQ, BDEF_GPIO_OUT_INV,  BDEF_GPIO_OUT_USI_IRQ    }  // -> convert to 'low active IRQ outp.' 
//,{ OPIN_CI_IRQ, 0,                  BDEF_GPIO_OUT_USI_IRQ    }    // (would be 'high-active IRQ'
,{ IOPIN_CI_D7, 0,                  BDEF_GPIO_OTR_USI_DATA_7 }
,{ IOPIN_CI_D6, 0,                  BDEF_GPIO_OTR_USI_DATA_6 }
,{ IOPIN_CI_D5, 0,                  BDEF_GPIO_OTR_USI_DATA_5 }
,{ IOPIN_CI_D4, 0,                  BDEF_GPIO_OTR_USI_DATA_4 }
,{ IOPIN_CI_D3, 0,                  BDEF_GPIO_OTR_USI_DATA_3 }
,{ IOPIN_CI_D2, 0,                  BDEF_GPIO_OTR_USI_DATA_2 }
,{ IOPIN_CI_D1, 0,                  BDEF_GPIO_OTR_USI_DATA_1 }
,{ IOPIN_CI_D0, 0,                  BDEF_GPIO_OTR_USI_DATA_0 }
};               
#define N_OUT_PINS (sizeof(OutPins_CiSlave) / sizeof(BRDCOM_DX_GpioOutputConfigDataT))

static void BOARD_FMC_H61PDKV1_SimulateCiOutIn(uint32_t flgOutOrIn)
{
    volatile uint32_t * h6xRegPtr;
    uint32_t regVal;

//    BTLIB_IO_PrStrHexStr(1, (const char_t*)"\n-> SimInOut:", flgOutOrIn, (const char_t*)"\n");
    regVal = FREG_GPIO_DATA_OUT + (OPIN_CI_DET*4);
    h6xRegPtr = (volatile uint32_t *) regVal;
    if (!flgOutOrIn) regVal = 1;
    else             regVal = 0;
    *h6xRegPtr = regVal;
}

/*
void BRDCOM_CISL_ControlIrqOutputPin(uint32_t pinLevel)     // NOTE: just test via 'direct GPIO'
{
    volatile uint32_t * h6xRegPtr;
    uint32_t reg_val;

    reg_val = FREG_GPIO_DATA_OUT + (OPIN_CI_IRQ*4);
    h6xRegPtr = (volatile uint32_t *) reg_val;
    *h6xRegPtr = pinLevel;
}
*/

// BRDCOM_USI_CtrlPollWaitPinHook
static uint32_t BOARD_FMC_H61PDKV1_CtrlPollWaitPin(uint32_t flgNotOrActive)
{
    volatile uint32_t * h6xRegPtr;
    uint32_t regVal;

//    BTLIB_IO_PrStrHexStr(1, (const char_t*)"\n-> CtrlWaitPin:", flgNotOrActive, (const char_t*)"\n");

    regVal = FREG_GPIO_DATA_OUT + (OPIN_CI_WT*4);  // GPIO-output-config for 'WaitOutputPin'
    h6xRegPtr = (volatile uint32_t *) regVal;
    if (!flgNotOrActive) regVal = 1;
    else                 regVal = 0;
    // todo: enable/disable 'wait-GPIO-pin'

//    regVal = FREG_GPIO_GetInput_95_64();
    h6xRegPtr = (volatile uint32_t *) FREG_GPIO_INPUT_95_64;
    regVal = *h6xRegPtr;
    regVal = (regVal >> (OPIN_CI_WT-64)) & 1;
    return regVal;
}

// BRDCOM_USI_PollBackIrqPinHook
static uint32_t BOARD_FMC_H61PDKV1_PollBackIrqOutpPin(void)
{
    volatile uint32_t * h6xRegPtr;
    uint32_t regVal;


    h6xRegPtr = (volatile uint32_t *) FREG_GPIO_INPUT_95_64;
    regVal = *h6xRegPtr;
//    BTLIB_IO_PrStrHexStr(8, (const char_t*)"\n-> Irq:", regVal, (const char_t*)"\n");
    regVal = (regVal >> (OPIN_CI_IRQ-64)) & 1;
    return regVal;
}
                 
/*!              
********************************************************************************
** \brief initialize all specific GPIO for 'CiSlave' testing
**
********************************************************************************
*/
void BOARD_FMC_H61PDKV1_BoardIfc_Register_CiSlave(void)
{
    BRDCOM_DX_ConfigureInputGpios(N_INP_PINS, InpPins_CiSlave);
    BRDCOM_DX_ConfigureOutputGpios(N_OUT_PINS, OutPins_CiSlave);

    BRDCOM_DX_CiSlaveInterfacePtr = &BOARD_FMC_H61PDKV1_CiSlaveInterface;
}

/*
uint32_t BOARD_FMC_H61PDKV1_GenLinkError_ViolatesWithNormalGpioInit;

void BOARD_FMC_H61PDKV1_GenerateLinkError_ViolatesWithNormalGpioInit(void)
{

}
*/
