/*
 *
 * \attention  This program is provided as is. You can redistribute it and/or
 * modify it. Fujitsu Microelectronics Limited accepts no responsibility or
 * liability for any errors or omissions.
 *
 * Copyright (C) 2008-2010 Fujitsu Microelectronics Limited, All Rights Reserved.
 */
/*
 FDL module skel_drv (icc)
*/

#include "fapex.h"
#include "fapex/abst_libicc.h"
#include "fapex/sys_config.h"
#include "fapex_fdlicc_priv.h"
#include "fapex_config_cmn.h"

/* for icc_funcStr, timer_funcStr */
#include "fapex_func_ext.h"

/*
*******************************************************************************
*******************************************************************************
** Global Data
*******************************************************************************
*******************************************************************************/

/*
*******************************************************************************
*******************************************************************************
** Local Data
*******************************************************************************
*******************************************************************************/
#ifndef ALLOCATE_DYNAMIC_MEMORY
static FAPEXFDL_ICC_CmnStrT  FAPIGLD_FDLICCCmnCtx[FAPEXDEV_ICC_CMN_DESCRIPTOR_NUM];
static FAPEXFDL_ICC_SubStrT  FAPIGLD_FDLICCSubCtx[FAPEXDEV_ICC_SUB_DESCRIPTOR_NUM];
#endif /* !ALLOCATE_DYNAMIC_MEMORY */

/*!
*******************************************************************************
**
** \brief  allocate data buffer
**
** \param  pfunc_sub    pointer to sub context
** \param  size         allocation size
**
******************************************************************************/
static int32_t fdlicc_AllocateDataBuffer(FAPEXFDL_ICC_SubStrT *pfunc_sub, uint32_t size)
{
    /*---------- LOCK ----------*/
    FAPI_SYS_GET_SEMAPHORE(pfunc_sub->buff_sem, FAPI_SYS_SUSPEND);

    if(size > pfunc_sub->dataBufferSize) {
        if(pfunc_sub->dataBuffer != NULL) {
            FAPI_SYS_FREE(pfunc_sub->dataBuffer);
        }
        pfunc_sub->dataBuffer = (uint8_t*)FAPI_SYS_MALLOC(sizeof(uint8_t) * size);
        if(pfunc_sub->dataBuffer == NULL) {
            return FAPEX_ICC_ERR_OUT_OF_MEMORY;
        }
    }

    /*--------- UNLOCK ---------*/
    FAPI_SYS_SET_SEMAPHORE(pfunc_sub->buff_sem, FAPI_SYS_NO_SUSPEND);

    return 0;
}

/*!
*******************************************************************************
**
** \brief  Read method of this id's FDL.
**         This function is called with the systemcall "read".
**
** If copy data to the argument's buffer, use FAPI_SYS_copyFromUser().
**
** \param  file    The pointer to the struct file.
** \param  buf     The pointer to the base address of the buffer.
** \param  size    The buffer size.
** \param  offset  The offset (now, not supported).
**
******************************************************************************/
FAPEXFDLREADRET_T FAPEXFDLICC_read(FAPEXFDLREADARG1_T file, FAPEXFDLREADARG2_T buf,
                                   FAPEXFDLREADARG3_T size, FAPEXFDLREADARG4_T offset)
{
    FAPEXFDL_ICC_SubStrT *pfunc_sub;
    uint8_t   data;
    uint64_t  end_time;
    int64_t   delta_time;
    int32_t   read_bytes, ret = 0;

    FAPEXDebugPrint(3, "[FAPEXFDLICC_read] size:%d\n",size);

    /* Get the context base of this id's FDL. */
    pfunc_sub = (FAPEXFDL_ICC_SubStrT*)(file->private_data);

    read_bytes = 0;
    end_time = timer_funcStr.GetTimeStamp(FAPI_TIMER_RESOLUTION_1_MSEC)
               + FAPEXFDL_ICC_MAX_TIMEOUT;

    while(read_bytes < size) {
#ifdef FAPI_ICC_CONNECT
        ret = icc_funcStr.ReadByte(pfunc_sub->pFAPICtxHdl, 0);
#endif
        if(ret < 0) {
            FAPEXDebugPrint(2, "[FAPEXFDLICC_read] FAPI_ICC_ReadByte:error %d\n", ret);
            if(ret == FAPI_ICC_ERR_TIMEOUT) {
                delta_time = timer_funcStr.GetTimeStamp(FAPI_TIMER_RESOLUTION_1_MSEC)
                             - end_time;
                if(delta_time > 0) {
                    break;
                }
                FAPI_SYS_SLEEP(1);
            }
            else {
                break;
            }
        }
        data = (uint8_t)ret;

        ret = FAPEXCFDLSAF_COPY_TO_USR((uint8_t*)((uint32_t)buf + read_bytes), &data, sizeof(uint8_t));
        if(ret != 0) {
            return ret;
        }
        read_bytes ++;
    }
    return read_bytes;
}

/*!
*******************************************************************************
**
** \brief  Write method of this id's FDL.
**         This function is called with the systemcall "write".
**
** If copy data from the argument's buffer, use FAPI_SYS_copyToUser().
**
** \param  file    The pointer to the struct file.
** \param  buf     The pointer to the base address of the buffer.
** \param  size    The buffer size.
** \param  offset  The offset (now, not supported).
**
******************************************************************************/
FAPEXFDLWRTRET_T FAPEXFDLICC_write(FAPEXFDLWRTARG1_T file, FAPEXFDLWRTARG2_T buf,
                                   FAPEXFDLWRTARG3_T size, FAPEXFDLWRTARG4_T offset)
{
    FAPEXFDL_ICC_SubStrT *pfunc_sub;
    uint64_t  end_time;
    int64_t   delta_time;
    int32_t   wrote_bytes, rest_bytes, ret = 0;

    FAPEXDebugPrint(3, "[FAPEXFDLICC_write] size:%d\n",size);

    /* Get the context base of this id's FDL. */
    pfunc_sub = (FAPEXFDL_ICC_SubStrT*)(file->private_data);

    ret = fdlicc_AllocateDataBuffer(pfunc_sub, size);
    if(ret != 0) {
        return ret;
    }

    /* copy parameters from an user space to the kernel space */
    FAPEXCFDLSAF_COPY_TO_DRV(pfunc_sub->dataBuffer, (void*)buf, (sizeof(uint32_t) * size));

    rest_bytes  = size;
    wrote_bytes = 0;
    end_time = timer_funcStr.GetTimeStamp(FAPI_TIMER_RESOLUTION_1_MSEC)
               + FAPEXFDL_ICC_MAX_TIMEOUT;

    while(wrote_bytes < size) {
#ifdef FAPI_ICC_CONNECT
        ret = icc_funcStr.WriteBytes(pfunc_sub->pFAPICtxHdl,
                                     pfunc_sub->dataBuffer + wrote_bytes,
                                     rest_bytes, 0);
#endif
        if(ret < 0) {
            FAPEXDebugPrint(2, "[FAPEXFDLICC_write] FAPI_ICC_WriteBytes:error %d\n", ret);
            if(ret == FAPI_ICC_ERR_TIMEOUT) {
                delta_time = timer_funcStr.GetTimeStamp(FAPI_TIMER_RESOLUTION_1_MSEC)
                             - end_time;
                if(delta_time > 0) {
                    break;
                }
                FAPI_SYS_SLEEP(1);
            }
            else {
                break;
            }
        }
        wrote_bytes += ret;
        rest_bytes  -= ret;
    }
    return wrote_bytes;
}

/*!
*******************************************************************************
**
** \brief  Event Notifer
**
** \param   handle       Valid driver instance handle previously returned by
**                       #FAPI_ICC_Open.
** \param   optData      Optional data to be passed to the notify function.
**
******************************************************************************/
static void fdlicc_CardNotifier(uint32_t cardIndex, uint32_t inOutFlag, void* optData)
{
    FAPEXFDL_ICC_SubStrT *pfunc_sub = (FAPEXFDL_ICC_SubStrT *)optData;
    uint32_t  evt_flag;

    switch(inOutFlag) {
        case FAPI_ICC_CARD_IN:
            evt_flag = FAPEX_ICC_STATUS_CARD_IN;
            FAPEXDebugPrint(2, "[fdlicc_CardNotifier] FAPI_ICC_CARD_IN \n");
            break;
        case FAPI_ICC_NO_CARD:
            evt_flag = FAPEX_ICC_STATUS_CARD_OUT;
            FAPEXDebugPrint(2, "[fdlicc_CardNotifier] FAPI_ICC_NO_CARD \n");
            break;
        default:
            return;
    }

    /* set event */
    FAPEXCFDLSAF_SET_EVENT(&pfunc_sub->dt_event_card, evt_flag, FAPEXCFDLSAF_WAITEVENT_OR);
}

static void fdlicc_WarmResetFinishNotifier(int32_t status, void* optData)
{
    FAPEXFDL_ICC_SubStrT *pfunc_sub = (FAPEXFDL_ICC_SubStrT *)optData;

    /* set event */
    FAPEXCFDLSAF_SET_EVENT(&pfunc_sub->dt_event_reset,
                           FAPEX_ICC_STATUS_FINISHED_RESET_WARM,
                           FAPEXCFDLSAF_WAITEVENT_OR);
}

static void fdlicc_ColdResetFinishNotifier(int32_t status, void* optData)
{
    FAPEXFDL_ICC_SubStrT *pfunc_sub = (FAPEXFDL_ICC_SubStrT *)optData;

    /* set event */
    FAPEXCFDLSAF_SET_EVENT(&pfunc_sub->dt_event_reset,
                           FAPEX_ICC_STATUS_FINISHED_RESET_COLD,
                           FAPEXCFDLSAF_WAITEVENT_OR);
}

/*!
*******************************************************************************
**
** \brief  ICC common descriptor open
**
** \param  pfunc_cmn    pointer to common context
** \param  pArg         pointer to FAPEX_ICC_OpenParamStrT parameter
**
******************************************************************************/
static int32_t fdlicc_OpenCmn(FAPEXFDL_ICC_CmnStrT *pfunc_cmn, FAPEX_ICC_OpenParamStrT *pArg)
{
    FAPEX_ICC_OpenParamStrT  params;
    int32_t   ret = FAPI_OK;

    if(pArg == NULL) {
        return FAPEX_ICC_ERR_BAD_PARAMETER;
    }
    /* copy parameters from an user space to the kernel space */
    FAPEXCFDLSAF_COPY_TO_DRV(&params, &pArg, sizeof(FAPEX_ICC_OpenParamStrT));

    if (params.protocolMode == 0){
        ret = icc_funcStr.T0Init();
        if(ret != FAPI_OK) {
            FAPEXDebugPrint(2, "[fdlicc_OpenCmn] FAPI_ICC_T0_Init error:%d\n", ret);
            return ret;
        }
    }
    else{
        ret = icc_funcStr.T1Init();
        if(ret != FAPI_OK) {
            FAPEXDebugPrint(2, "[fdlicc_OpenCmn] FAPI_ICC_T1_Init error:%d\n", ret);
            return ret;
        }
    }
    pfunc_cmn->protocolMode = params.protocolMode;

    pfunc_cmn->enable_channel = 0;

    return ret;
}

/*!
*******************************************************************************
**
** \brief  ICC sub descriptor open
**
** \param  pfunc_cmn    pointer to common context
** \param  pfunc_sub    pointer to sub context
** \param  id           Device identifier
** \param  pArg         pointer to FAPEX_ICC_OpenSubIdParamStrT parameter
**
******************************************************************************/
static int32_t fdlicc_OpenSub(FAPEXFDL_ICC_CmnStrT *pfunc_cmn, FAPEXFDL_ICC_SubStrT *pfunc_sub,
                              uint32_t id, FAPEX_ICC_OpenSubIdParamStrT *pArg)
{
    FAPI_ICC_T0_OpenParamsT    params;
    FAPEXCFDLSAF_EventStrT     *dt_event_ptr;
    FAPEXFDL_IccT0_HandleT     *icc_t0_handle_ptr;
    int32_t  ret;

    if(pfunc_sub->pFAPICtxHdl != NULL) {
        return 0;
    }
    if(pArg == NULL) {
        return FAPEX_ICC_ERR_BAD_PARAMETER;
    }
    /* copy parameters from an user space to the kernel space */
    FAPEXCFDLSAF_COPY_TO_DRV(&(params.cardIndex), &(pArg->cardIndex), sizeof(uint32_t));

    params.version    = FAPI_ICC_T0_VERSION;
    params.cardNotify = fdlicc_CardNotifier;
    params.usePPS     = 0;
    params.optDataPtr = pfunc_sub;
                                
#ifdef FAPI_ICC_CONNECT
    if ( pfunc_cmn->protocolMode == 0 ){
        pfunc_sub->IccT0Handle = icc_funcStr.T0Open(&params, &ret);
        icc_t0_handle_ptr = (FAPEXFDL_IccT0_HandleT*)pfunc_sub->IccT0Handle;
        pfunc_sub->pFAPICtxHdl = icc_t0_handle_ptr->iccHandle;
    }
    FAPEXDebugPrint(3, "[fdlicc_OpenSub] ICC context:0x%08x\n",
                           (uint32_t)pfunc_sub->pFAPICtxHdl);
    if(pfunc_sub->pFAPICtxHdl == NULL) {
        FAPEXDebugPrint(2, "[fdlicc_OpenSub] FAPI_ICC_T0_Open/FAPI_ICC_T1_Open error:%d\n", ret);
        return ret;
    }
#endif

    dt_event_ptr = &pfunc_sub->dt_event_card;
    if(dt_event_ptr->initialized == 0) {
        FAPEXCFDLSAF_CREATE_EVENT(dt_event_ptr);
    }

    dt_event_ptr = &pfunc_sub->dt_event_reset;
    if(dt_event_ptr->initialized == 0) {
        FAPEXCFDLSAF_CREATE_EVENT(dt_event_ptr);
    }

    if(pfunc_sub->buff_sem == 0) {
        pfunc_sub->buff_sem = FAPI_SYS_CREATE_SEMAPHORE(1);
        if(pfunc_sub->buff_sem == 0) {
            return FAPEX_ICC_ERR_SEMAPHORE_CREATE;
        }
    }

    /*---------- LOCK ----------*/
    FAPI_SYS_GET_SEMAPHORE(pfunc_cmn->cmn_sem, FAPI_SYS_SUSPEND);

    /* update status */
    pfunc_cmn->enable_channel |= 1 << (pfunc_sub->id - FAPEXDEV_ICC_SUB_MINOR_FIRST);

    /*--------- UNLOCK ---------*/
    FAPI_SYS_SET_SEMAPHORE(pfunc_cmn->cmn_sem, FAPI_SYS_NO_SUSPEND);

    pfunc_sub->status = FAPEX_STATUS_OPENED;

    return 0;
}

/*!
*******************************************************************************
**
** \brief  ICC sub descriptor close
**
** \param  pfunc_cmn    pointer to common context
** \param  pfunc_sub    pointer to sub context
**
******************************************************************************/
static int32_t fdlicc_CloseSub(FAPEXFDL_ICC_CmnStrT *pfunc_cmn, FAPEXFDL_ICC_SubStrT *pfunc_sub,
                               uint32_t id)
{
    uint32_t ch_mask;
    int32_t  ret;

    if(pfunc_sub->pFAPICtxHdl == NULL) {
        return 0;
    }

    /* set end event */
    ret = FAPEXCFDLSAF_SET_EVENT( &pfunc_sub->dt_event_card,
                                  FAPEX_ICC_STATUS_CLOSED,
                                  FAPEXCFDLSAF_WAITEVENT_OR );
    if(ret < 0) {
        return FAPEX_ICC_ERR_INVALID_STATUS;
    }

    ret = FAPEXCFDLSAF_SET_EVENT( &pfunc_sub->dt_event_reset,
                                  FAPEX_ICC_STATUS_CLOSED,
                                  FAPEXCFDLSAF_WAITEVENT_OR );
    if(ret < 0) {
        return FAPEX_ICC_ERR_INVALID_STATUS;
    }

#ifdef FAPI_ICC_CONNECT
    /* call FAPI (close) */
    ret = icc_funcStr.Close(pfunc_sub->pFAPICtxHdl);
    if(ret != FAPI_OK) {
        return ret;
    }
#endif /* FAPI_VIOUT_CONNECT */
    pfunc_sub->pFAPICtxHdl = NULL;

    FAPEXCFDLSAF_DELETE_EVENT(&pfunc_sub->dt_event_card);
    FAPEXCFDLSAF_DELETE_EVENT(&pfunc_sub->dt_event_reset);

    /*---------- LOCK ----------*/
    FAPI_SYS_GET_SEMAPHORE(pfunc_sub->buff_sem, FAPI_SYS_SUSPEND);

    if(pfunc_sub->dataBuffer != NULL) {
        FAPI_SYS_FREE(pfunc_sub->dataBuffer);
        pfunc_sub->dataBuffer = NULL;
    }

    /*--------- UNLOCK ---------*/
    FAPI_SYS_SET_SEMAPHORE(pfunc_sub->buff_sem, FAPI_SYS_NO_SUSPEND);

    FAPI_SYS_DESTROY_SEMAPHORE(pfunc_sub->buff_sem);
    pfunc_sub->buff_sem = (FAPI_SYS_SemaphoreT)0;

    ch_mask = ~(1 << (id - FAPEXDEV_ICC_SUB_MINOR_FIRST));

    /*---------- LOCK ----------*/
    FAPI_SYS_GET_SEMAPHORE(pfunc_cmn->cmn_sem, FAPI_SYS_SUSPEND);

    /* update status */
    pfunc_cmn->enable_channel &= ch_mask;

    /*--------- UNLOCK ---------*/
    FAPI_SYS_SET_SEMAPHORE(pfunc_cmn->cmn_sem, FAPI_SYS_NO_SUSPEND);

    pfunc_sub->status = FAPEX_STATUS_LOADED;

    return 0;
}

/*!
*******************************************************************************
**
** \brief  ICC set I/O port parameter
**
** \param  pfunc_sub    pointer to sub context
** \param  pArg         pointer to FAPEX_ICC_IOportParamStrT parameter
**
******************************************************************************/
static int32_t fdlicc_SetIOport(FAPEXFDL_ICC_SubStrT *pfunc_sub, FAPEX_ICC_IOportParamStrT *pArg)
{
    FAPEX_ICC_IOportParamStrT  *params;
    int32_t  ret = 0;

    params = &pfunc_sub->ioport;

    /* setup open parameters */
    FAPEXCFDLSAF_COPY_TO_DRV(params, pArg, sizeof(FAPEX_ICC_IOportParamStrT));

#ifdef FAPI_ICC_CONNECT
    ret = icc_funcStr.SetBaudRate(pfunc_sub->pFAPICtxHdl, params->baudRate);
    if(ret != FAPI_OK) {
        FAPEXDebugPrint(2, "[fdlicc_SetIOport] FAPI_ICC_SetBaudRate error:%d\n", ret);
        return ret;
    }

    ret = icc_funcStr.SetClockFrequency(pfunc_sub->pFAPICtxHdl, (params->cardClock/1000));
    if(ret != FAPI_OK) {
        FAPEXDebugPrint(2, "[fdlicc_SetIOport] FAPI_ICC_SetClockFrequency error:%d\n", ret);
        return ret;
    }
#endif

    pfunc_sub->status = FAPEX_STATUS_RUNNING;

    return 0;
}

/*!
*******************************************************************************
**
** \brief  ICC get I/O port parameter
**
** \param  pfunc_sub    pointer to sub context
** \param  pArg         pointer to FAPEX_ICC_IOportParamStrT parameter
**
******************************************************************************/
static int32_t fdlicc_GetIOport(FAPEXFDL_ICC_SubStrT *pfunc_sub, FAPEX_ICC_IOportParamStrT *pArg)
{
    FAPEX_ICC_IOportParamStrT  *params;
    uint32_t tmp_value;
    int32_t  ret = 0;

    params = &pfunc_sub->ioport;

#ifdef FAPI_ICC_CONNECT
    ret = icc_funcStr.GetBaudRate(pfunc_sub->pFAPICtxHdl, &tmp_value);
    if(ret != FAPI_OK) {
        FAPEXDebugPrint(2, "[fdlicc_GetIOport] FAPI_ICC_SetBaudRate error:%d\n", ret);
        return ret;
    }
    params->baudRate = tmp_value;

    ret = icc_funcStr.GetClockFrequency(pfunc_sub->pFAPICtxHdl, &tmp_value);
    if(ret != FAPI_OK) {
        FAPEXDebugPrint(2, "[fdlicc_GetIOport] FAPI_ICC_SetClockFrequency error:%d\n", ret);
        return ret;
    }
    params->cardClock = tmp_value * 1000;
#endif

    /* copy parameters from the kernel space to an user space */
    FAPEXCFDLSAF_COPY_TO_USR(pArg, params, sizeof(FAPEX_ICC_IOportParamStrT));

    return 0;
}

/*!
*******************************************************************************
**
** \brief  ICC reset
**
** \param  pfunc_sub    pointer to sub context
** \param  resetMode    reset mode
**
******************************************************************************/
static int32_t fdlicc_Reset(FAPEXFDL_ICC_SubStrT *pfunc_sub, int32_t resetMode)
{
    int32_t  ret = FAPEX_ICC_ERR_INVALID_STATUS;
    FAPEXCFDLSAF_EventStrT *dt_event_ptr;
    FAPEXFDL_IccT0_HandleT *icc_t0_handle_ptr;

    if(pfunc_sub->status != FAPEX_STATUS_RUNNING) {
        return ret;
    }

    switch(resetMode) {
        case FAPEX_ICC_RESET_MODE_WARM:
            {
                FAPI_ICC_WarmResetParamsT  *wreset_params;

                wreset_params = (FAPI_ICC_WarmResetParamsT*)&pfunc_sub->ioport;
#ifdef FAPI_ICC_CONNECT
                ret = icc_funcStr.WarmReset(pfunc_sub->pFAPICtxHdl,
                                            wreset_params,
                                            fdlicc_WarmResetFinishNotifier,
                                            pfunc_sub);
                if(ret != FAPI_OK) {
                    FAPEXDebugPrint(2, "[fdlicc_Reset] FAPI_ICC_WarmReset error:%d\n", ret);
                }
#endif
            }
            break;

        case FAPEX_ICC_RESET_MODE_COLD:
            {
                FAPI_ICC_ColdResetParamsT  *creset_params;
                
                icc_t0_handle_ptr = (FAPEXFDL_IccT0_HandleT*)pfunc_sub->IccT0Handle;
                
                FAPEXDebugPrint(2, "[fdlicc_Reset] cardInitialized=%d\n", icc_t0_handle_ptr->cardInitialized);
                
                creset_params = (FAPI_ICC_ColdResetParamsT*)&pfunc_sub->ioport;
#ifdef FAPI_ICC_CONNECT

                ret = icc_funcStr.ColdReset(pfunc_sub->pFAPICtxHdl,
                                            creset_params,
                                            fdlicc_ColdResetFinishNotifier,
                                            pfunc_sub);
                if(ret != FAPI_OK) {
                    FAPEXDebugPrint(2, "[fdlicc_Reset] FAPI_ICC_ColdReset error:%d\n", ret);
                }
#endif
            }
            break;

        default:
            ret = FAPEX_ICC_ERR_BAD_PARAMETER;
    }

    if(ret == 0) {
        /* check whether it has waited for finish of reset */
        dt_event_ptr = &pfunc_sub->dt_event_reset;
        dt_event_ptr->mode       = FAPEXCFDLSAF_WAITEVENT_SYNC | FAPEXCFDLSAF_WAITEVENT_OR;
        dt_event_ptr->wai_bitptn = FAPEX_ICC_STATUS_FINISHED_RESET | FAPEX_ICC_STATUS_CLOSED;
        dt_event_ptr->tout       = 1000;

        ret = FAPEXCFDLSAF_WAIT_EVENT(dt_event_ptr);
        if(ret > 0) {
            FAPEXDebugPrint(2, "FAPEXCFDLSAF_WAIT_EVENT %d",ret);
            if ( (ret & FAPEX_ICC_STATUS_FINISHED_RESET_COLD) == FAPEX_ICC_STATUS_FINISHED_RESET_COLD ){
                FAPEXDebugPrint(2,"FAPEX_ICC_STATUS_FINISHED_RESET_COLD ret:%d\n",ret);

                return FAPEX_ICC_STATUS_FINISHED_RESET_COLD;

            }
            else if ( (ret & FAPEX_ICC_STATUS_FINISHED_RESET_WARM) == FAPEX_ICC_STATUS_FINISHED_RESET_WARM ){
                FAPEXDebugPrint(2, "FAPEX_ICC_STATUS_FINISHED_RESET_WARM ret:%d\n",ret);

                return FAPEX_ICC_STATUS_FINISHED_RESET_WARM;
            }
        }
        else if(ret == FAPI_ERR_TIMEOUT) {
            return FAPEX_ICC_ERR_TIMEOUT;
        }
    }
    return ret;
}

/*!
*******************************************************************************
**
** \brief  ICC get ATR parameter
**
** \param  pfunc_sub    pointer to sub context
** \param  pArg         pointer to FAPEX_ICC_AtrParamStrT parameter
**
******************************************************************************/
static int32_t fdlicc_GetATR(FAPEXFDL_ICC_SubStrT *pfunc_sub, FAPEX_ICC_AtrParamStrT *pArg)
{
    FAPI_ICC_AtrParamsT  params;
    int32_t  ret = 0;

#ifdef FAPI_ICC_CONNECT
    ret = icc_funcStr.GetAtrParams(pfunc_sub->pFAPICtxHdl, &params);
    if(ret != FAPI_OK) {
        FAPEXDebugPrint(2, "[fdlicc_GetATR] FAPI_ICC_GetAtrParams error:%d\n", ret);
        return ret;
    }
#endif

    /* copy parameters from the kernel space to an user space */
    FAPEXCFDLSAF_COPY_TO_USR(&(pArg->convention), &(params.convention), sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_USR(&(pArg->protocol),   &(params.protocol),   sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_USR(&(pArg->FI),         &(params.FI),         sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_USR(&(pArg->DI),         &(params.DI),         sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_USR(&(pArg->PI1),        &(params.PI1),        sizeof(uint8_t) );
    FAPEXCFDLSAF_COPY_TO_USR(&(pArg->PI2),        &(params.PI2),        sizeof(uint8_t) );
    FAPEXCFDLSAF_COPY_TO_USR(&(pArg->II),         &(params.II),         sizeof(uint8_t) );
    FAPEXCFDLSAF_COPY_TO_USR(&(pArg->N),          &(params.N),          sizeof(uint8_t) );
    FAPEXCFDLSAF_COPY_TO_USR(&(pArg->XI),         &(params.XI),         sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_USR(&(pArg->UI),         &(params.UI),         sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_USR(&(pArg->histBytes),  &(params.histBytes),  sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_USR(&(pArg->atrBytes),   &(params.atrBytes),   sizeof(uint32_t));

    FAPEXCFDLSAF_COPY_TO_USR(pArg->histBuffer, params.histBuffer, (sizeof(uint8_t) * params.histBytes));
    FAPEXCFDLSAF_COPY_TO_USR(pArg->atrBuffer,  params.atrBuffer,  (sizeof(uint8_t) * params.atrBytes) );

    return 0;
}

/*!
*******************************************************************************
**
** \brief  ICC get data
**
** \param  pfunc_sub    pointer to sub context
** \param  pArg         pointer to FAPEX_ICC_DataParamStrT parameter
**
******************************************************************************/
static int32_t fdlicc_GetData(FAPEXFDL_ICC_SubStrT *pfunc_sub, FAPEX_ICC_DataParamStrT *pArg)
{
    int32_t   ret;
    uint32_t  sendLength;
    uint32_t  readLength;
    FAPI_ICC_TOcommandT         cmd  = { 0 };
    FAPI_ICC_TOresponseT        resp = { 0 };

    /* copy parameters from an user space to the kernel space */
    FAPEXCFDLSAF_COPY_TO_DRV(&sendLength,        &(pArg->sendSize),      sizeof(uint32_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&(cmd.Class),       &(pArg->sendBuffer[0]), sizeof(uint8_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&(cmd.Instruction), &(pArg->sendBuffer[1]), sizeof(uint8_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&(cmd.Param1),      &(pArg->sendBuffer[2]), sizeof(uint8_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&(cmd.Param2),      &(pArg->sendBuffer[3]), sizeof(uint8_t));
    FAPEXCFDLSAF_COPY_TO_DRV(&(cmd.Length),      &(pArg->sendBuffer[4]), sizeof(uint8_t));

    if (sendLength > 5){
        // SEND CMD
        FAPEXCFDLSAF_COPY_TO_DRV(cmd.data, (pArg->sendBuffer+5), cmd.Length);
        resp.Length = 0;
    }
    else{
        // READ CMD
        resp.Length = cmd.Length;
    }
    
#ifdef FAPI_ICC_CONNECT
    ret = icc_funcStr.T0SendCmd( (FAPI_SYS_HandleT)pfunc_sub->IccT0Handle, &cmd, &resp);
    if(ret != FAPI_OK) {
        FAPEXDebugPrint(2, "[fdlicc_GetData] FAPI_ICC_T0_SendCmd error:0x%02x\n", ret);
        return ret;
    }
#endif
    
    /* copy parameters from the kernel space to an user space */
    if (resp.Length > 0){
        // READ CMD
        readLength = resp.Length + 2;
        FAPEXCFDLSAF_COPY_TO_USR(&(pArg->receiveSize),              &readLength, sizeof(uint32_t));
        FAPEXCFDLSAF_COPY_TO_USR(pArg->receiveBuffer,               resp.data,   resp.Length);
        FAPEXCFDLSAF_COPY_TO_USR(pArg->receiveBuffer+resp.Length,   &resp.SW1,    sizeof(uint8_t));
        FAPEXCFDLSAF_COPY_TO_USR(pArg->receiveBuffer+resp.Length+1, &resp.SW2,    sizeof(uint8_t));
    }
    else{
        // SEND CMD
        readLength = 2;
        FAPEXCFDLSAF_COPY_TO_USR(&(pArg->receiveSize),  &readLength, sizeof(uint32_t));
        FAPEXCFDLSAF_COPY_TO_USR(pArg->receiveBuffer,   &resp.SW1,  sizeof(uint8_t));
        FAPEXCFDLSAF_COPY_TO_USR(pArg->receiveBuffer+1, &resp.SW2,  sizeof(uint8_t));
    }

    return 0;
}

/*!
*******************************************************************************
**
** \brief  ICC get status
**
** \param  pfunc_sub    pointer to sub context
** \param  pArg         pointer to status flag
**
******************************************************************************/
static int32_t fdlicc_GetStatus(FAPEXFDL_ICC_SubStrT *pfunc_sub, uint32_t *pArg)
{
    int32_t  status = 0;
    int32_t  ret = 0;

#ifdef FAPI_ICC_CONNECT
    ret = icc_funcStr.GetSlotStatus(pfunc_sub->pFAPICtxHdl);
#endif

    switch(ret) {
        case FAPI_OK:
            status |= FAPEX_ICC_STATUS_NORMAL;
            break;
        case FAPI_ICC_ERR_SLOT_EMPTY:
            status |= FAPEX_ICC_STATUS_ERR_SLOT_EMPTY;
            break;
        case FAPI_ICC_ERR_RUNNING:
            status |= FAPEX_ICC_STATUS_ERR_RUNNING;
            break;
        case FAPI_ICC_ERR_OVERFLOW:
            status |= FAPEX_ICC_STATUS_ERR_OVERFLOW;
            break;
        case FAPI_ICC_ERR_NO_ANSWER:
            status |= FAPEX_ICC_STATUS_ERR_NO_ANSWER;
            break;
        case FAPI_ICC_ERR_TIMEOUT:
            status |= FAPEX_ICC_STATUS_ERR_TIMEOUT;
            break;
        case FAPI_ICC_ERR_UNKNOWN:
        default:
            status |= FAPEX_ICC_STATUS_ERR_UNKNOWN;
    }

    /* copy parameters from the kernel space to an user space */
    FAPEXCFDLSAF_COPY_TO_USR(pArg, &status, sizeof(uint32_t));

    return 0;
}
/*!
*******************************************************************************
**
** \brief   wait for the evnet of card in/out notify
**
** \param  pfunc_sub    pointer to sub context
** \param  pArg           pointer to status flag
**
******************************************************************************/
static int32_t fdlicc_GetCardNotify(FAPEXFDL_ICC_SubStrT *pfunc_sub, uint32_t *pArg)
{
    int32_t  ret = 0;
    FAPEXCFDLSAF_EventStrT *dt_event_ptr;

    dt_event_ptr = &pfunc_sub->dt_event_card;
    /* wait */
    dt_event_ptr->mode       = FAPEXCFDLSAF_WAITEVENT_SYNC | FAPEXCFDLSAF_WAITEVENT_OR;
    dt_event_ptr->wai_bitptn = FAPEX_ICC_STATUS_CARD_IN_OUT | FAPEX_ICC_STATUS_CLOSED;
    dt_event_ptr->tout       = 0;

    ret = FAPEXCFDLSAF_WAIT_EVENT(dt_event_ptr);
    if(ret > 0) {
        FAPEXCFDLSAF_COPY_TO_USR(pArg, &ret, sizeof(uint32_t));
    }
    else if(ret == FAPI_ERR_TIMEOUT) {
        return FAPEX_ICC_ERR_TIMEOUT;
    }
    return 0;
}
/*!
*******************************************************************************
**
** \brief  General control of this id's FDL.
**         This function is called with systemcall "ioctl".
**
** \param  inode   The pointer to the struct inode.
** \param  file    The pointer to the struct file.
** \param  cmd     The ioctl command. This is set by user application. On Linux,
**                 the kernel copy this data to the kernel space. On RTOS, this
**                 memory area is shared with the user.
**                 memory of the user
** \param  arg     The argument of the ioctl command. This is set by user
**                 application. On Linux, the kernel copy this data to the kernel
**                 space. On RTOS, thismemory area is shared with the user.
**
******************************************************************************/
FAPEXFDLIOCRET_T FAPEXFDLICC_ioctl(FAPEXFDLIOCARG1_T inode, FAPEXFDLIOCARG2_T file,
                                   FAPEXFDLIOCARG3_T cmd, FAPEXFDLIOCARG4_T arg)
{
    FAPEXFDL_ICC_CmnStrT *pfunc_cmn;
    FAPEXFDL_ICC_SubStrT *pfunc_sub;
    uint32_t  id;
    int32_t   ret = FAPEX_ICC_ERR_BAD_PARAMETER;

    FAPEXDebugPrint(3, "[FAPEXFDLICC_ioctl] cmd:%08x\n",cmd);

    id = FAPEXCFDLSAF_GETID(inode);

    /* Get the context base of this id's FDL. */
    if(id < FAPEXDEV_ICC_SUB_MINOR_FIRST) {
        pfunc_cmn = (FAPEXFDL_ICC_CmnStrT *)(file->private_data);
        pfunc_sub = NULL;
    }
    else {
        pfunc_sub = (FAPEXFDL_ICC_SubStrT *)(file->private_data);
        if(pfunc_sub != NULL) {
            pfunc_cmn = (FAPEXFDL_ICC_CmnStrT *)pfunc_sub->pCmnHdl;
        }
        else {
            pfunc_cmn = NULL;
        }
    }

    switch(cmd) {
        /* actual code which is depend on function is implimented here. */
        case FAPEX_IOCCOM_ICC_OPEN_SUBID:
            if(pfunc_sub != NULL) {
                ret = fdlicc_OpenSub(pfunc_cmn, pfunc_sub, id,
                                     (FAPEX_ICC_OpenSubIdParamStrT*)arg);
            }
            break;

        case FAPEX_IOCCOM_CLOSE_SUBID:
            if(pfunc_sub != NULL) {
                ret = fdlicc_CloseSub(pfunc_cmn, pfunc_sub, id);
            }
            break;

        case FAPEX_IOCCOM_ICC_OPEN:
            if( (pfunc_cmn != NULL) && (pfunc_sub == NULL) ) {
                ret = fdlicc_OpenCmn(pfunc_cmn, (FAPEX_ICC_OpenParamStrT *)arg);
            }
            break;

        case FAPEX_IOCCOM_ICC_SET_IOPORT:
            if( (pfunc_sub != NULL) && (arg != (FAPEXFDLIOCARG4_T)NULL) ) {
                ret = fdlicc_SetIOport(pfunc_sub, (FAPEX_ICC_IOportParamStrT *)arg);
            }
            break;

        case FAPEX_IOCCOM_ICC_GET_IOPORT:
            if( (pfunc_sub != NULL) && (arg != (FAPEXFDLIOCARG4_T)NULL) ) {
                ret = fdlicc_GetIOport(pfunc_sub, (FAPEX_ICC_IOportParamStrT *)arg);
            }
            break;

        case FAPEX_IOCCOM_RESET:
            if(pfunc_sub != NULL) {
                ret = fdlicc_Reset(pfunc_sub, (int32_t)arg);
            }
            break;

        case FAPEX_IOCCOM_ICC_GET_ATR:
            if( (pfunc_sub != NULL) && (arg != (FAPEXFDLIOCARG4_T)NULL) ) {
                ret = fdlicc_GetATR(pfunc_sub, (FAPEX_ICC_AtrParamStrT *)arg);
            }
            break;

        case FAPEX_IOCCOM_ICC_GET_DATA:
            if( (pfunc_sub != NULL) && (arg != (FAPEXFDLIOCARG4_T)NULL) ) {
                ret = fdlicc_GetData(pfunc_sub, (FAPEX_ICC_DataParamStrT *)arg);
            }
            break;

        case FAPEX_IOCCOM_ICC_GET_STATUS:
            if( (pfunc_sub != NULL) && (arg != (FAPEXFDLIOCARG4_T)NULL) ) {
                ret = fdlicc_GetStatus(pfunc_sub, (uint32_t*)arg);
            }
            break;

        case FAPEX_IOCCOM_ICC_GET_CARDNOTIFY:
            if( (pfunc_sub != NULL) && (arg != (FAPEXFDLIOCARG4_T)NULL) ) {
                ret = fdlicc_GetCardNotify(pfunc_sub, (uint32_t*)arg);
            }

        default:
            break;
    }

    return ret;
}

/*!
*******************************************************************************
**
** \brief  Poll method of this id's FDL.
**         This function is called with the systemcall "select".
**
** \param  file    The pointer to the struct file.
** \param  ptbl    An internal data.
**
******************************************************************************/
FAPEXFDLPLRET_T FAPEXFDLICC_poll(FAPEXFDLPLARG1_T file, FAPEXFDLPLARG2_T ptbl)
{
    FAPEXFDLPLRET_T        mask = 0;
    FAPEXFDL_ICC_SubStrT   *pfunc_sub;
    FAPEXCFDLSAF_EventStrT *dt_event_ptr = NULL;

    /* Get the context base of this id's FDL. */
    pfunc_sub = (FAPEXFDL_ICC_SubStrT*)(file->private_data);
    dt_event_ptr = &pfunc_sub->dt_event_card;

    FAPEXDebugPrint(4, "[FAPEXFDLICC_poll]\n");

    FAPEXCFDLSAF_POLL_EVENT(file, dt_event_ptr, NULL, ptbl);

    /* actual code which is depend on function is implimented here. */

//  if(dt_event_ptr->evt_bitptn & FAPEX_ICC_STATUS_WRITE) {
//      mask |= POLLOUT | POLLWRNORM;
//  }
//  if(dt_event_ptr->evt_bitptn & FAPEX_ICC_STATUS_READ) {
//      mask |= POLLIN | POLLRDNORM;
//  }
    if(dt_event_ptr->evt_bitptn & FAPEX_ICC_STATUS_NORMAL_CAHNGE) {
        mask |= POLLMSG;
    }
    if(dt_event_ptr->evt_bitptn & FAPEX_ICC_STATUS_WARNING) {
        mask |= POLLPRI;
    }
//  if(dt_event_ptr->evt_bitptn & FAPEX_ICC_STATUS_ERROR) {
//      mask |= POLLERR;
//  }

    return mask;
}

#if 0 /* not supported */

/*!
*******************************************************************************
**
** \brief  Mmap method of this id's FDL.
**         This function is called with the systemcall "mmap".
**
** \param  file    The pointer to the struct file.
** \param  vma     system depend
**
******************************************************************************/
FAPEXFDLMMPRET_T FAPEXFDLICC_mmap(FAPEXFDLMMPARG1_T file, FAPEXFDLMMPARG2_T vma)
{
#ifdef __LINUX__
    int32_t ret;

    if (vma->vm_pgoff != 0){
        return -EINVAL;
    }

    /* This area doen't swapout */
    vma->vm_flags |= VM_IO | VM_RESERVED;

    /* This area is IO area. Set noncache, and it doesn't dump to a core
       file of a process */
    if (file->f_flags & O_SYNC){
        vma->vm_page_prot = pgprot_noncached(vma->vm_page_prot);
        vma->vm_flags |= VM_IO;
    }

    ret = remap_pfn_range(vma, vma->vm_start, vma->vm_pgoff,
                  vma->vm_end - vma->vm_start, vma->vm_page_prot);

    if(ret != 0){
        return -EAGAIN;
    }
    return 0;
#else
    return -1;
#endif /* __LINUX__ */
}
#endif

/*!
*******************************************************************************
**
** \brief  Open this id's FDL.
**         This function is called with the systemcall "open". And this id's FDL
**         are opened.
**
** \param  inode   The pointer to the struct inode.
** \param  file    The pointer to the struct file.
**
******************************************************************************/
FAPEXFDLOPENRET_T FAPEXFDLICC_open (FAPEXFDLOPENARG1_T inode, FAPEXFDLOPENARG2_T file)
{
    FAPEXFDL_ICC_CmnStrT *pfunc_cmn;
    void    *pCfdlHdl;
    int32_t id;

    FAPEXDebugPrint(3, "[FAPEXFDLICC_open] file:0x%08x,private_data:0x%08x\n",
                       (uint32_t)file, (uint32_t)file->private_data);

    if(file->private_data == NULL){
        id = FAPEXCFDLSAF_GETID(inode);
        FAPEXDebugPrint(3, "[FAPEXFDLICC_open] minor_no:%d\n", id);

        /* Get the context base of the CFDL. */
        pCfdlHdl = (void*)FAPEXCFDL_GetCfdlHdl();
        FAPEXDebugPrint(3, "[FAPEXFDLICC_open] pCfdlHdl:0x%08x\n", (uint32_t)(pCfdlHdl));

        /* Get the context base of this id's FDL. */
        pfunc_cmn = (FAPEXFDL_ICC_CmnStrT*)FAPEXCFDL_GetFdlHdl(pCfdlHdl, id);
        file->private_data = pfunc_cmn;
        FAPEXDebugPrint(3, "[FAPEXFDLICC_open] pFdlHdl:0x%08x\n", (uint32_t)(pfunc_cmn));

        /* open sequence */
        /* ... */

        /* set parameter */
        pfunc_cmn->pCfdlHdl = pCfdlHdl;
        pfunc_cmn->id       = id;
#ifdef __LINUX__
        kref_init(&pfunc_cmn->kref);
#endif
    }
    else {
        pfunc_cmn = (FAPEXFDL_ICC_CmnStrT *)file->private_data;
#ifdef __LINUX__
        kref_get(&pfunc_cmn->kref);
#endif
    }

    /* increment our usage count for the device */
    pfunc_cmn->used_count++;

    FAPEXDebugPrint(3, "[FAPEXFDLICC_open] fapex_fdlicc was opened : used_counter %d.\n",
                       pfunc_cmn->used_count);

    return 0;
}

#ifdef __LINUX__
/*!
*******************************************************************************
**
** \brief  delete this FDL id's module.
**         This function is called with the systemcall "close". And this id's FDL
**         are released.
**
** \param  kref    The pointer to the kref structure.
**
******************************************************************************/
static void fdlicc_delete(struct kref *kref)
{
    return;
}
#endif

/*!
*******************************************************************************
**
** \brief  Release this FDL id's module.
**         This function is called with the systemcall "close". And this id's FDL
**         are released.
**
** \param  inode   The pointer to the struct inode.
** \param  file    The pointer to the struct file.
**
******************************************************************************/
FAPEXFDLRELRET_T FAPEXFDLICC_release (FAPEXFDLRELARG1_T inode, FAPEXFDLRELARG2_T file)
{
    FAPEXFDL_ICC_CmnStrT  *pfunc_cmn;
    FAPEXFDLRELRET_T      ret = 0;

    pfunc_cmn = (FAPEXFDL_ICC_CmnStrT*)(file->private_data);

    if(pfunc_cmn != NULL) {
        /* check common id */
        if(pfunc_cmn->id < FAPEXDEV_ICC_SUB_MINOR_FIRST) {
            if(pfunc_cmn->enable_channel != 0) {
                FAPEXFDL_RETVAL( FAPEX_ICC_ERR_INVALID_STATUS );
            }
        }

        /* decrement the count on our device */
        pfunc_cmn->used_count--;
#ifdef __LINUX__
        kref_put(&pfunc_cmn->kref, fdlicc_delete);
#endif

        FAPEXDebugPrint(3, "[FAPEXFDLICC_release] id %d, used_count %d\n",
                           pfunc_cmn->id, pfunc_cmn->used_count);
    }
    FAPEXFDL_RETVAL(ret);
}

/*--------------------------------------------------------*/
/* The file operations of this FDL module.                */
/*--------------------------------------------------------*/
struct file_operations fapex_fdlicc_fops;

/*!
*******************************************************************************
**
** \brief  Initialize process of the FDL module.
**         This function is called when the FDL module is loaded. It is executed
**         only one times.
**         On Linux, this function is registered to the kernel using "module_init".
**         And user application doesn't call this function. If do "insmod" command,
**         this is called by the kernel.
**         On RTOS, user application has to call this function.
**
** \param  void (can't set any argument)
**
******************************************************************************/
int32_t FAPEXFDLICC_init(void)
{
    int32_t ret;
    int32_t count;
    void    *pCfdlHdl;
    FAPEXFDL_ICC_CmnStrT  *pfunc_cmn;
    FAPEXFDL_ICC_SubStrT  *pfunc_sub;
    FAPEXCFDL_UtilStrT    utility;
#ifdef ALLOCATE_DYNAMIC_MEMORY
    FAPEXFDL_ICC_SubStrT  *pfunc_sub0;
#endif

    FAPEXDebugPrint(1, "[FAPEXFDLICC_init]\n");

    /* set struct file_operations */
    fapex_fdlicc_fops.owner   = THIS_MODULE;
    fapex_fdlicc_fops.open    = FAPEXFDLICC_open;
    fapex_fdlicc_fops.read    = FAPEXFDLICC_read;
    fapex_fdlicc_fops.write   = FAPEXFDLICC_write;
    fapex_fdlicc_fops.ioctl   = FAPEXFDLICC_ioctl;
    fapex_fdlicc_fops.poll    = FAPEXFDLICC_poll;
//  fapex_fdlicc_fops.mmap    = FAPEXFDLICC_mmap;
    fapex_fdlicc_fops.release = FAPEXFDLICC_release;

#ifdef ALLOCATE_DYNAMIC_MEMORY
    pfunc_cmn = (FAPEXFDL_ICC_CmnStrT*)FAPI_SYS_MALLOC(sizeof(FAPEXFDL_ICC_CmnStrT));
    if(pfunc_cmn == NULL) {
        return FAPEX_ICC_ERR_OUT_OF_MEMORY;
    }
    pfunc_sub = (FAPEXFDL_ICC_SubStrT *)FAPI_SYS_MALLOC(sizeof(FAPEXFDL_ICC_SubStrT)*FAPEXDEV_ICC_SUB_DESCRIPTOR_NUM);
    if(pfunc_sub == NULL) {
        FAPI_SYS_FREE(pfunc_cmn);
        return FAPEX_ICC_ERR_OUT_OF_MEMORY;
    }
    pfunc_sub0 = pfunc_sub;
#else
    pfunc_cmn = FAPIGLD_FDLICCCmnCtx;
    pfunc_sub = FAPIGLD_FDLICCSubCtx;
#endif /* ALLOCATE_DYNAMIC_MEMORY */
    FAPEXCFDLSAF_CLEAR_MEMORY(pfunc_cmn, sizeof(FAPEXFDL_ICC_CmnStrT)*(FAPEXDEV_ICC_CMN_DESCRIPTOR_NUM));
    FAPEXCFDLSAF_CLEAR_MEMORY(pfunc_sub, sizeof(FAPEXFDL_ICC_SubStrT)*(FAPEXDEV_ICC_SUB_DESCRIPTOR_NUM));

    pCfdlHdl = (void*)FAPEXCFDL_GetCfdlHdl();

    pfunc_cmn->pCfdlHdl = pCfdlHdl;
    pfunc_cmn->cmn_sem  = FAPI_SYS_CREATE_SEMAPHORE(1);
    if(pfunc_cmn->cmn_sem == 0) {
        ret = FAPEX_ICC_ERR_SEMAPHORE_CREATE;
        goto FAPEXFDLICC_init_error;
    }

    /* Register this id's FDL device to CFDL. */
    ret = FAPEXCFDL_RegisterFdl(&fapex_fdlicc_fops, pfunc_cmn, &utility, FAPEXDEV_ICC_CMN_MINOR);
    if(ret) {
        goto FAPEXFDLICC_init_error;
    }

    for(count=FAPEXDEV_ICC_SUB_MINOR_FIRST; count<=FAPEXDEV_ICC_SUB_MINOR_LAST; count++) {
        pfunc_sub->pCfdlHdl = pCfdlHdl;
        pfunc_sub->pCmnHdl  = pfunc_cmn;
//      pfunc_sub->status   = FAPEX_STATUS_LOADED;

        /* Register this id's FDL device to CFDL. */
        ret = FAPEXCFDL_RegisterFdl(&fapex_fdlicc_fops, pfunc_sub, &utility, count);
        if(ret) {
            goto FAPEXFDLICC_init_error;
        }
        pfunc_sub ++;
    }

#ifdef FAPI_ICC_CONNECT
    FAPEXDebugPrint(1, "call FAPEXCFG_ICC_Init\n");

    /* call FAPI (initialize) */
    ret = FAPEXCFG_ICC_Init();
#else
    ret = 0;
#endif /* FAPI_ICC_CONNECT */

    FAPEXDebugPrint(1, "[FAPEXFDLICC_init] fapex_fdlicc was loaded. %d\n", ret);

    return ret;

FAPEXFDLICC_init_error:
#ifdef ALLOCATE_DYNAMIC_MEMORY
    FAPI_SYS_FREE(pfunc_cmn);
    FAPI_SYS_FREE(pfunc_sub0);
#endif
    FAPEXDebugPrint(2, "[FAPEXFDLICC_init] FAPEXCFDL_RegisterFdl:error %d\n", ret);
    return ret;
}

/*!
*******************************************************************************
**
** \brief  Cleanup process of the FDL module.
**         This function is called when the FDL module is unloaded. It is executed
**         only one times.
**         On Linux, this function is registered to the kernel using "module_exit".
**         And user application doesn't call this function. If do "rmmod" command,
**         this is called by the kernel.
**         On RTOS, user application has to call this function.
**
** \param  void (can't set any argument)
**
******************************************************************************/
void FAPEXFDLICC_cleanup(void)
{
    int32_t count;
    void    *pCfdlHdl;
    FAPEXFDL_ICC_CmnStrT *pfunc_cmn;

    FAPEXDebugPrint(1, "[FAPEXFDLICC_cleanup]\n");

#ifdef FAPI_ICC_CONNECT
    /* call FAPI API (cleanup) */
    FAPEXCFG_ICC_Cleanup();

    FAPEXDebugPrint(1, "end FAPEXCFG_ICC_Cleanup\n");
#endif /* FAPI_ICC_CONNECT */

    /* cleanup sequence (common this FDL) */
    pCfdlHdl = (void*)FAPEXCFDL_GetCfdlHdl();

    /* relase the FDL context */
    pfunc_cmn = (FAPEXFDL_ICC_CmnStrT*)FAPEXCFDL_GetFdlHdl(pCfdlHdl, FAPEXDEV_ICC_CMN_MINOR);
    if(pfunc_cmn != NULL) {
        FAPI_SYS_DESTROY_SEMAPHORE(pfunc_cmn->cmn_sem);
        pfunc_cmn->cmn_sem = (FAPI_SYS_SemaphoreT)0;

#ifdef ALLOCATE_DYNAMIC_MEMORY
        FAPI_SYS_FREE(pfunc_cmn);
    }
    {
        FAPEXFDL_ICC_SubStrT *pfunc_sub;

        pfunc_sub = (FAPEXFDL_ICC_SubStrT*)FAPEXCFDL_GetFdlHdl(pCfdlHdl, FAPEXDEV_ICC_SUB_MINOR_FIRST);
        if(pfunc_sub != NULL) {
            FAPI_SYS_FREE(pfunc_sub);
        }
#endif /* ALLOCATE_DYNAMIC_MEMORY */
    }

    /* Unregister this id's FDL device to CFDL. */
    FAPEXCFDL_UnregisterFdl(FAPEXDEV_ICC_CMN_MINOR);

    for(count=FAPEXDEV_ICC_SUB_MINOR_FIRST; count<=FAPEXDEV_ICC_SUB_MINOR_LAST; count++) {
        /* Unregister this id's FDL device to CFDL. */
        FAPEXCFDL_UnregisterFdl(count);
    }

    FAPEXDebugPrint(1, "[FAPEXFDLVIENC_cleanup] fapex_fdlvienc was unloaded.\n");

    return;
}

#ifdef __LINUX__
/*******************************
 MACRO (Linux dep.)
********************************/
#ifdef BUILD_AS_MODULE
module_init(FAPEXFDLICC_init);
module_exit(FAPEXFDLICC_cleanup);

MODULE_LICENSE("Proprietary. Fujitsu Microelectronics.");
MODULE_DESCRIPTION("FAPI driver (fapex_fdlicc object)");
#endif /* BUILD_AS_MODULE */
#endif /* __LINUX__ */
