/*!
*************************************************************************
**
** \file    fapex/abst_libadc.h
**
** \brief   abstraction library header file (ADC driver)
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/


#ifndef _FAPEX_ABST_LIBADC_H
#define _FAPEX_ABST_LIBADC_H

#include "fapex/abst_lib.h"
#include "fapex/abst_error.h"

/*
*************************************************************************
*************************************************************************
** Defines and Macros
*************************************************************************
*************************************************************************/


/*
*************************************************************************
** Standard error codes for the driver
*************************************************************************/

/*! FAPex ADC error code base value */
#define FAPEX_ADC_ERR_BASE            (FAPEX_ADC_MODULE_BASE + FAPEX_RET_ERR_BASE)

/*! Bad parameter passed. */
#define FAPEX_ADC_ERR_BAD_PARAMETER (FAPEX_ADC_ERR_BASE + FAPEX_ERR_BAD_PARAMETER)

/*! Memory allocation failed. */
#define FAPEX_ADC_ERR_OUT_OF_MEMORY (FAPEX_ADC_ERR_BASE + FAPEX_ERR_OUT_OF_MEMORY)

/*! Device already initialised. */
#define FAPEX_ADC_ERR_ALREADY_INITIALIZED (FAPEX_ADC_ERR_BASE + FAPEX_ERR_ALREADY_INITIALIZED)

/*! Device not initialised. */
#define FAPEX_ADC_ERR_NOT_INITIALIZED (FAPEX_ADC_ERR_BASE + FAPEX_ERR_NOT_INITIALIZED)

/*! Feature or function is not available. */
#define FAPEX_ADC_ERR_FEATURE_NOT_SUPPORTED (FAPEX_ADC_ERR_BASE + FAPEX_ERR_FEATURE_NOT_SUPPORTED)

/*! Timeout occured. */
#define FAPEX_ADC_ERR_TIMEOUT (FAPEX_ADC_ERR_BASE + FAPEX_ERR_TIMEOUT)

/*! The device is busy, try again later. */
#define FAPEX_ADC_ERR_DEVICE_BUSY (FAPEX_ADC_ERR_BASE + FAPEX_ERR_DEVICE_BUSY)

/*! Invalid handle was passed. */
#define FAPEX_ADC_ERR_INVALID_HANDLE (FAPEX_ADC_ERR_BASE + FAPEX_ERR_INVALID_HANDLE)

/*! Semaphore could not be created. */
#define FAPEX_ADC_ERR_SEMAPHORE_CREATE (FAPEX_ADC_ERR_BASE + FAPEX_ERR_SEMAPHORE_CREATE)

/*! The driver's used version is not supported. */
#define FAPEX_ADC_ERR_UNSUPPORTED_VERSION (FAPEX_ADC_ERR_BASE + FAPEX_ERR_UNSUPPORTED_VERSION)

/*! The mailqueue could not be created. */
#define FAPEX_ADC_ERR_MAILQUEUE_CREATE (FAPEX_ADC_ERR_BASE + FAPEX_ERR_MAILQUEUE_CREATE)

/*! The adc have not been opened. */
#define FAPEX_ADC_ERR_NOT_OPEN (FAPEX_ADC_ERR_BASE + FAPEX_ERR_NOT_OPEN)

/*! The adc have been opend. */
#define FAPEX_ADC_ERR_ALREADY_OPEN (FAPEX_ADC_ERR_BASE + FAPEX_ERR_ALREADY_OPEN)

/*! \brief The device cannot be used because it has been powered off. */
#define FAPEX_ADC_ERR_POWERED_OFF            (FAPEX_ADC_ERR_BASE - 100)

/*
*************************************************************************
*************************************************************************
** Data types
*************************************************************************
*************************************************************************/

/*!
*************************************************************************
** \brief open parameters for the ADC driver
**
** \sa FAPEX_ADC_Ioctl 
*************************************************************************/
typedef enum
{
    FAPEX_ADC_CHANNEL_0     = 0,           //!< \b   ADC_VIN[0] pin is selected (default)
    FAPEX_ADC_CHANNEL_1,                   //!< \b   ADC_VIN[1] pin is selected
    FAPEX_ADC_CHANNEL_2,                   //!< \b   ADC_VIN[2] pin is selected
    FAPEX_ADC_CHANNEL_3,                   //!< \b   ADC_VIN[3] pin is selected
} FAPEX_ADC_OpenParamEnumT;

/*!
*************************************************************************
** \brief set clock parameters for the ADC driver ioctl FAPEX_IOCCOM_ADC_SETCLOCK command
**
** \sa FAPEX_ADC_Ioctl
*************************************************************************/
typedef enum
{
    FAPEX_ADC_DIVIDER_COUNT_8     = 2,           //!< \b  divide by 8 (12.375MHz) (default)
    FAPEX_ADC_DIVIDER_COUNT_16,                  //!< \b  divide by 16 (6.1875MHz)
    FAPEX_ADC_DIVIDER_COUNT_32,                  //!< \b  divide by 32 (3.09375MHz)
    FAPEX_ADC_DIVIDER_COUNT_64,                  //!< \b  divide by 64 (1.546875MHz)
    FAPEX_ADC_DIVIDER_COUNT_128,                 //!< \b  divide by 128 (0.7734375MHz)
    FAPEX_ADC_DIVIDER_COUNT_256,                 //!< \b  divide by 256 (0.38671875MHz)
} FAPEX_ADC_DividerCountEnumT;


/*!
*************************************************************************
** \brief ioctl specific command for the ADC driver
*************************************************************************/
// IO
#define FAPEX_IOCCOM_ADC_POWEROFF   _IO(FAPEX_DEV1_TYPE, FAPEX_IOCCOM_NB_ADC + 0)
// IOR
#define FAPEX_IOCCOM_ADC_READ      _IOR(FAPEX_DEV1_TYPE, FAPEX_IOCCOM_NB_ADC + 0, uint32_t)
// IOW
#define FAPEX_IOCCOM_ADC_OPEN      _IOW(FAPEX_DEV1_TYPE, FAPEX_IOCCOM_NB_ADC + 0, FAPEX_ADC_OpenParamEnumT)
#define FAPEX_IOCCOM_ADC_SETCLOCK  _IOW(FAPEX_DEV1_TYPE, FAPEX_IOCCOM_NB_ADC + 1, FAPEX_ADC_DividerCountEnumT)


/*
*************************************************************************
*************************************************************************
** prototype for application
*************************************************************************
*************************************************************************/
#ifndef __LINUX__
#include "fapex/abst_types.h"

#ifdef __cplusplus
extern "C" {
#endif

FAPEXOPENRET_T FAPEX_ADC_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags, FAPEXOPENARG3_T argptr);
FAPEXRELRET_T  FAPEX_ADC_Close(FAPEXRELARG1_T id);
FAPEXIOCRET_T  FAPEX_ADC_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg);

#ifdef __cplusplus
}
#endif
#endif /* !__LINUX__ */

#endif /* !_FAPEX_ABST_LIBADC_H */

