/*!
*****************************************************************************
**
** \file        fapex/abst_libaudec_heaac.h
** 
** \version     $Id: drv_audec_heaac.h 4281 2009-07-10 13:22:36Z spurke $
**
** \brief       HE-AAC decoder specific parameters.
**
** This file contains the declaration of specific parameters for the HE-AAC decoder.
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU SEMICONDUCTOR
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2006-2010 by Fujitsu Semiconductor Europe GmbH
** (C) Copyright 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*****************************************************************************
*/

#ifndef FAPEX_AUDEC_HEAAC_H
    #define FAPEX_AUDEC_HEAAC_H

    #include <fapex/abst_lib.h>

    //***********************************************************************
    //***********************************************************************
    //** Defines and Macros
    //***********************************************************************
    //***********************************************************************


    //***********************************************************************
    //***********************************************************************
    //** Data types
    //***********************************************************************
    //***********************************************************************


    /*!
    *************************************************************************
    ** \brief Parameters for the HE-AAC decoder.
    **
    ** This data structure covers all parameters which can be set for the HE-AAC
    ** decoder.
    ** Setting these parameters is optional. If not the decoder applies the 
    ** mentioned defaults. 
    **
    ** \sa FAPEX_AUDEC_Ioctl/FAPEX_IOCCOM_AUDEC_SET_CODEC_PARAMS
    *************************************************************************
    */
    typedef struct {

        /*!
           The audio decoder ID.
           \note This field must be set to FAPEX_AUDEC_DECODER_HEAAC.
        */
        int32_t decoder;
        
        /*!
           The external sampling rate for raw decoding (Decoder's default: -1).
        */
        int32_t extSamplingRate;
        
        /*!
           The maximum output sampling rate (Decoder's default: 96000).
        */
        int32_t maxOutSamplingRate;
        
        /*!
           Enables the DownSampled SBR processing. 
           - 0: disable
           - 1: enable
           (Decoder's default: 0).
        */
        uint32_t enableDSSBR;
        
        /*!
           Enables the DownMixed SBR processing. 
           - 0: disable
           - 1: enable
           (Decoder's default: 0).
        */
        uint32_t enableDMSBR;

        /*!
           The SBR decoding mode.
           - 0: SBR decoding disabled
           - 1: SBR decoding enabled
           - 2: SBR decoding automatically when SBR is actually present 
           (Decoder's default: 2).
        */
        uint32_t sbrMode;
        
        /*!
           Enables the stereo DOWNMIX for the decoder. 
           - 0: disable
           - 1: enable
           (Decoder's default: 1).
        */
        uint32_t enableDOWNMIX;
        
        /*!
           If stereo DOWNMIX is enabled the following modes can be applied. 
           - 0: Lo/Ro
           - 1: Lt/Rt
           (Decoder's default: 0).
        */
        uint32_t downmixMode;
        
        /*!
           Enables DRC and consideration of the Program Reference Level for the decoder. 
           - 0: disable
           - 1: enable
           (Decoder's default: 1).
        */
        uint32_t enableDRC;
        
        /*!
           If DRC is enabled the following target reference level will be applied. 
           - 0 - 127 which corresponds to an attenuation of 0 to -31.25dBFS in steps of
           -0.25dB.
           (Decoder's default: 127).
        */
        uint32_t drcTargetRefLevel;
        
        /*!
           Number of frames for falling slope when fadeOut is performed to enter
           the mute state in case of faulty frames. A value between 1 and 8.
           (Decoder's default: 5).
        */
        uint32_t fadeOutFrameNum;
        
        /*!
           Number of frames for rising slope when fadeIn is performed to leave
           the mute state in case the signal has recovered. A value between 1 and 8.
           (Decoder's default: 5).
        */
        uint32_t fadeInFrameNum;
        
        /*!
           Minimum number of error free frames until rising slope starts to leave 
           mute state. A value between 0 and 255.
           (Decoder's default: 5).
        */
        uint32_t muteFrameCount;
        
        /*!
           Number of consecutive frames with same header which must sucessfully 
           decode before the decoder will start. (Codec's default: 1)
           This parameter allows applications to improve the audio error 
           recognition when the decoder is started with bad streams. 
        */
        uint32_t startInsyncFrameCount;

    } FAPEX_AUDEC_HeaacParamsT;

#endif
