/*!
*************************************************************************
**
** \file    fapex/abst_libauout.h
**
** \brief   abstraction library header file (Audio Output driver)
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#ifndef _FAPEX_ABST_LIBAUOUT_H
#define _FAPEX_ABST_LIBAUOUT_H

#include "fapex/abst_lib.h"
#include "fapex/abst_error.h"

/*
*************************************************************************
*************************************************************************
** Defines and Macros
*************************************************************************
*************************************************************************/

/*
*************************************************************************
** Standard error codes for the driver
*************************************************************************/
/*! FAPex AUOUT error code base value */
#define FAPEX_AUOUT_ERR_BASE            (FAPEX_AUOUT_MODULE_BASE + FAPEX_RET_ERR_BASE)

/*! Bad parameter passed. */
#define FAPEX_AUOUT_ERR_BAD_PARAMETER   (FAPEX_AUOUT_ERR_BASE + FAPEX_ERR_BAD_PARAMETER)

/*! Memory allocation failed. */
#define FAPEX_AUOUT_ERR_OUT_OF_MEMORY   (FAPEX_AUOUT_ERR_BASE + FAPEX_ERR_OUT_OF_MEMORY)


/*
*************************************************************************
*************************************************************************
** Data types
*************************************************************************
*************************************************************************/

/*!
    *************************************************************************
    ** \brief Audio Synchronization State.
    **
    ** This enumeration contains the various synchronization states of the
    ** audio output module.
    **
    ** \sa FAPI_AUOUT_GetSyncState
    *************************************************************************
    */
    typedef enum {
        FAPEX_AUOUT_SYNC_UNDEF           = (int32_t)0, //!< SYNC state is undefined.
        FAPEX_AUOUT_SYNC_SYNCHRONISING,                //!< Synchronization is ongoing.
        FAPEX_AUOUT_SYNC_SYNCHRONIZED                  //!< Synchronization achieved.
    } FAPEX_AUOUT_SyncStateEnumT;





/*!
*************************************************************************
** \brief open parameters for the AUOUT driver
**
** \sa FAPEX_AUOUT_Open
*************************************************************************/
typedef struct {
    /*!
        Audio Output Type enable flag<br>
        The value of enableFlag is the bitwise-inclusive OR of values from the following list.<br>
        \param #FAPEX_AUOUT_ENABLE_I2S   Enable I2S output
        \param #FAPEX_AUOUT_ENABLE_SPDIF Enable SPDIF output
        \param #FAPEX_AUOUT_ENABLE_HDMI  Enable HDMI output
        \param #FAPEX_AUOUT_ENABLE_PCM Enable PCM buffer output
    */
    uint32_t enableFlag;
    /*!
        Audio Output Channel enable flag<br>
        The value of channelFlag is the bitwise-inclusive OR of values from the following list.<br>
        \param #FAPEX_AUOUT_CHANNEL_LEFT       Enable Left channel
        \param #FAPEX_AUOUT_CHANNEL_CENTER     Enable Center channel
        \param #FAPEX_AUOUT_CHANNEL_RIGHT      Enable Right channel
        \param #FAPEX_AUOUT_CHANNEL_SUR_LEFT   Enable Left Surround channel
        \param #FAPEX_AUOUT_CHANNEL_SUR_RIGHT  Enable Right Surround channel
        \param #FAPEX_AUOUT_CHANNEL_BACK_LEFT  Enable Left Back Surround channel
        \param #FAPEX_AUOUT_CHANNEL_BACK_RIGHT Enable Right Back Surround channel
        \param #FAPEX_AUOUT_CHANNEL_LFE        Enable Low Frequency Effects channel
    */
    uint32_t channelFlag;

} FAPEX_AUOUT_OpenParamStrT;

#define FAPEX_AUOUT_ENABLE_I2S               (0x0001) //!< Enable I2S output
#define FAPEX_AUOUT_ENABLE_SPDIF             (0x0002) //!< Enable SPDIF output
#define FAPEX_AUOUT_ENABLE_HDMI              (0x0004) //!< Enable HDMI output
#define FAPEX_AUOUT_ENABLE_PCM              (0x0008) //!< Enable PCM buffer output
#define FAPEX_AUOUT_ENABLE_TYPE_MASK         (FAPEX_AUOUT_ENABLE_I2S | FAPEX_AUOUT_ENABLE_SPDIF | FAPEX_AUOUT_ENABLE_PCM)

#define FAPEX_AUOUT_CHANNEL_LEFT             0x00000001     //!< Enable Left channel
#define FAPEX_AUOUT_CHANNEL_CENTER           0x00000002     //!< Enable Center channel
#define FAPEX_AUOUT_CHANNEL_RIGHT            0x00000004     //!< Enable Right channel
#define FAPEX_AUOUT_CHANNEL_SUR_LEFT         0x00000008     //!< Enable Left Surround channel
#define FAPEX_AUOUT_CHANNEL_SUR_RIGHT        0x00000010     //!< Enable Right Surround channel
#define FAPEX_AUOUT_CHANNEL_BACK_LEFT        0x00000020     //!< Enable Left Back Surround channel
#define FAPEX_AUOUT_CHANNEL_BACK_RIGHT       0x00000040     //!< Enable Right Back Surround channel
#define FAPEX_AUOUT_CHANNEL_LFE              0x00000080     //!< Enable Low Frequency Effects channel
#define FAPEX_AUOUT_CHANNEL_ALL              \
 (FAPEX_AUOUT_CHANNEL_LEFT | FAPEX_AUOUT_CHANNEL_CENTER | FAPEX_AUOUT_CHANNEL_RIGHT | \
  FAPEX_AUOUT_CHANNEL_SUR_LEFT | FAPEX_AUOUT_CHANNEL_SUR_RIGHT |                      \
  FAPEX_AUOUT_CHANNEL_BACK_LEFT | FAPEX_AUOUT_CHANNEL_BACK_RIGHT | FAPEX_AUOUT_CHANNEL_LFE)

#define FAPEX_AUOUT_CHANNEL_MIN_ID           (FAPEX_AUOUT_CHANNEL_LEFT)
#define FAPEX_AUOUT_CHANNEL_MAX_ID           (FAPEX_AUOUT_CHANNEL_LFE)
#define FAPEX_AUOUT_SPEAKER_MIN_ID           (0)
#define FAPEX_AUOUT_SPEAKER_MAX_ID           (7)

#define FAPEX_AUOUT_CHANNEL_STEREO           (FAPEX_AUOUT_CHANNEL_LEFT | FAPEX_AUOUT_CHANNEL_RIGHT)
#define FAPEX_AUOUT_CHANNEL_2BK              (FAPEX_AUOUT_CHANNEL_BACK_LEFT | FAPEX_AUOUT_CHANNEL_BACK_RIGHT)
#define FAPEX_AUOUT_CHANNEL_2SD              (FAPEX_AUOUT_CHANNEL_SUR_LEFT | FAPEX_AUOUT_CHANNEL_SUR_RIGHT)
#define FAPEX_AUOUT_CHANNEL_2P1CH            (FAPEX_AUOUT_CHANNEL_STEREO | FAPEX_AUOUT_CHANNEL_LFE)
#define FAPEX_AUOUT_CHANNEL_3CH              (FAPEX_AUOUT_CHANNEL_STEREO | FAPEX_AUOUT_CHANNEL_CENTER)
#define FAPEX_AUOUT_CHANNEL_3P1CH            (FAPEX_AUOUT_CHANNEL_3CH | FAPEX_AUOUT_CHANNEL_LFE)
#define FAPEX_AUOUT_CHANNEL_2S2CH            (FAPEX_AUOUT_CHANNEL_STEREO | FAPEX_AUOUT_CHANNEL_2BK)
#define FAPEX_AUOUT_CHANNEL_4P1CH            (FAPEX_AUOUT_CHANNEL_2S2CH | FAPEX_AUOUT_CHANNEL_LFE)
#define FAPEX_AUOUT_CHANNEL_3S2CH            (FAPEX_AUOUT_CHANNEL_3CH | FAPEX_AUOUT_CHANNEL_2BK)
#define FAPEX_AUOUT_CHANNEL_5P1CH            (FAPEX_AUOUT_CHANNEL_3S2CH | FAPEX_AUOUT_CHANNEL_LFE)
#define FAPEX_AUOUT_CHANNEL_7P1CH            (FAPEX_AUOUT_CHANNEL_5P1CH | FAPEX_AUOUT_CHANNEL_2SD)


/*!
*************************************************************************
** \brief PCM information parameters for the AUOUT driver
**
** The related ioctl command type :
** \ref auout_cmd_set_pcm_info "FAPEX_IOCCOM_AUOUT_SET_PCM_INFO",
** \ref auout_cmd_get_pcm_info "FAPEX_IOCCOM_AUOUT_GET_PCM_INFO"
**
** \sa FAPEX_AUOUT_Ioctl
*************************************************************************/
typedef struct {
    /*!
        sampling frequency [Hz]<br>
        -1 if unknown.
    */
    int32_t  sampleFreq;
    /*!
        number of bits for PCM sample. -1 if unknown.
    */
    int32_t  sampleBits;
    /*!
        number of channels. -1 if unknown.
    */
    int32_t  channelCount;

} FAPEX_AUOUT_PcmInfoStrT;

/*!
*************************************************************************
** \brief stream type parameter for the AUOUT driver
**
** The related ioctl command type :
** \ref auout_cmd_set_stream_info "FAPEX_IOCCOM_AUOUT_SET_STREAM_TYPE",
** \ref auout_cmd_get_stream_info "FAPEX_IOCCOM_AUOUT_GET_STREAM_TYPE"
**
** \sa FAPEX_AUOUT_Ioctl
*************************************************************************/
#define FAPEX_AUOUT_STREAM_UNDEF    (-1)    //!< Audio stream undefined.
#define FAPEX_AUOUT_STREAM_AAC      (0)     //!< AAC-LC compressed audio.
#define FAPEX_AUOUT_STREAM_AC3      (1)     //!< AC3 compressed audio.
#define FAPEX_AUOUT_STREAM_MPEG_L1  (2)     //!< MPEG1/2 Layer 1 compressed audio.
#define FAPEX_AUOUT_STREAM_MPEG_L2  (3)     //!< MPEG1/2 Layer 2 compressed audio.
#define FAPEX_AUOUT_STREAM_MPEG_L3  (4)     //!< MPEG1/2 Layer 3 (MP3) compressed audio.
#define FAPEX_AUOUT_STREAM_WMA      (5)     //!< WMA compressed audio.
#define FAPEX_AUOUT_STREAM_EC3      (6)     //!< Enhanced AC3 compressed audio.
#define FAPEX_AUOUT_STREAM_HEAAC    (7)     //!< AACplus v1/v2 compressed audio.
#define FAPEX_AUOUT_STREAM_PCM (8)  //!< PCM audio

/*!
*************************************************************************
** \brief mute parameter for the AUOUT driver
**
** The related ioctl command type :
** \ref auout_cmd_set_mute "FAPEX_IOCCOM_AUOUT_SET_MUTE"
**
** \sa FAPEX_AUOUT_Ioctl
*************************************************************************/
typedef struct {
    /*!
        Audio Output Channel enable flag<br>
        The value of channelFlag is the bitwise-inclusive OR of values from the following list.<br>
        \param #FAPEX_AUOUT_CHANNEL_LEFT       Enable Left channel
        \param #FAPEX_AUOUT_CHANNEL_CENTER     Enable Center channel
        \param #FAPEX_AUOUT_CHANNEL_RIGHT      Enable Right channel
        \param #FAPEX_AUOUT_CHANNEL_SUR_LEFT   Enable Left Surround channel
        \param #FAPEX_AUOUT_CHANNEL_SUR_RIGHT  Enable Right Surround channel
        \param #FAPEX_AUOUT_CHANNEL_BACK_LEFT  Enable Left Back Surround channel
        \param #FAPEX_AUOUT_CHANNEL_BACK_RIGHT Enable Right Back Surround channel
        \param #FAPEX_AUOUT_CHANNEL_LFE        Enable Low Frequency Effects channel
    */
    uint32_t channelFlag;
    /*!
        mute flag ... 0:mute / other:Unmute
    */
    uint32_t muteFlag;

} FAPEX_AUOUT_MuteParamStrT;

/*!
*************************************************************************
** \brief volume parameter for the AUOUT driver
**
** The related ioctl command type :
** \ref auout_cmd_set_volume "FAPEX_IOCCOM_AUOUT_SET_VOLUME",
** \ref auout_cmd_get_volume "FAPEX_IOCCOM_AUOUT_GET_VOLUME"
**
** \sa FAPEX_AUOUT_Ioctl
*************************************************************************/
typedef struct {
    /*!
        Audio Output Channel enable flag<br>
        The value of channelFlag is the bitwise-inclusive OR of values from the following list.<br>
        \param #FAPEX_AUOUT_CHANNEL_LEFT       Enable Left channel
        \param #FAPEX_AUOUT_CHANNEL_CENTER     Enable Center channel
        \param #FAPEX_AUOUT_CHANNEL_RIGHT      Enable Right channel
        \param #FAPEX_AUOUT_CHANNEL_SUR_LEFT   Enable Left Surround channel
        \param #FAPEX_AUOUT_CHANNEL_SUR_RIGHT  Enable Right Surround channel
        \param #FAPEX_AUOUT_CHANNEL_BACK_LEFT  Enable Left Back Surround channel
        \param #FAPEX_AUOUT_CHANNEL_BACK_RIGHT Enable Right Back Surround channel
        \param #FAPEX_AUOUT_CHANNEL_LFE        Enable Low Frequency Effects channel
    */
    uint32_t channelFlag;
    /*!
        volume value (0-255)
    */
    int32_t  volume;

} FAPEX_AUOUT_VolumeParamStrT;

/*!
*************************************************************************
** \brief DualMono parameter for the AUOUT driver
**
** The related ioctl command type :
** \ref auout_cmd_set_dualmono "FAPEX_IOCCOM_AUOUT_SET_DUALMONO"
**
** \sa FAPEX_AUOUT_Ioctl
*************************************************************************/
typedef struct {
    /*!
        Audio Output Channel enable flag<br>
        The value of channelFlag is the bitwise-inclusive OR of values from the following list.<br>
        \param #FAPEX_AUOUT_CHANNEL_LEFT       Enable Left channel
        \param #FAPEX_AUOUT_CHANNEL_CENTER     Enable Center channel
        \param #FAPEX_AUOUT_CHANNEL_RIGHT      Enable Right channel
        \param #FAPEX_AUOUT_CHANNEL_SUR_LEFT   Enable Left Surround channel
        \param #FAPEX_AUOUT_CHANNEL_SUR_RIGHT  Enable Right Surround channel
        \param #FAPEX_AUOUT_CHANNEL_BACK_LEFT  Enable Left Back Surround channel
        \param #FAPEX_AUOUT_CHANNEL_BACK_RIGHT Enable Right Back Surround channel
        \param #FAPEX_AUOUT_CHANNEL_LFE        Enable Low Frequency Effects channel
    */
    uint32_t channelFlag;
    /*!
        DualMono enable flag ... 0:disable / other:enable
    */
    uint32_t enableFlag;

} FAPEX_AUOUT_DualMonoParamStrT;

/*!
*************************************************************************
** \brief speakers parameter for the AUOUT driver
**
** The related ioctl command type :
** \ref auout_cmd_get_speakers "FAPEX_IOCCOM_AUOUT_GET_SPEAKERS"
**
** \sa FAPEX_AUOUT_Ioctl
*************************************************************************/
typedef struct {
    /*!
        Audio Output Channel enable flag<br>
        The value of channelFlag is the bitwise-inclusive OR of values from the following list.<br>
        \param #FAPEX_AUOUT_CHANNEL_LEFT       Enable Left channel
        \param #FAPEX_AUOUT_CHANNEL_CENTER     Enable Center channel
        \param #FAPEX_AUOUT_CHANNEL_RIGHT      Enable Right channel
        \param #FAPEX_AUOUT_CHANNEL_SUR_LEFT   Enable Left Surround channel
        \param #FAPEX_AUOUT_CHANNEL_SUR_RIGHT  Enable Right Surround channel
        \param #FAPEX_AUOUT_CHANNEL_BACK_LEFT  Enable Left Back Surround channel
        \param #FAPEX_AUOUT_CHANNEL_BACK_RIGHT Enable Right Back Surround channel
        \param #FAPEX_AUOUT_CHANNEL_LFE        Enable Low Frequency Effects channel
    */
    uint32_t channelFlag;
    /*!
        number of channels
    */
    uint32_t channelCount;

} FAPEX_AUOUT_SpeakersParamStrT;

/*!
*************************************************************************
** \brief I2S buffer parameter for the AUOUT driver
**
** The related ioctl command type :
** \ref auout_cmd_get_i2sbuffers "FAPEX_IOCCOM_AUOUT_GET_I2SBUFFERS"
**
** \sa FAPEX_AUOUT_Ioctl
*************************************************************************/
typedef struct {
    /*!
        Audio Output Buffer mapping flag<br>
        The value of channelFlag[] is the bitwise-inclusive OR of values from the following list.<br>
        \param #FAPEX_AUOUT_BUFFER0_0 Enable Buffer0-0
        \param #FAPEX_AUOUT_BUFFER0_1 Enable Buffer0-1
        \param #FAPEX_AUOUT_BUFFER1_0 Enable Buffer1-0
        \param #FAPEX_AUOUT_BUFFER1_1 Enable Buffer1-1
        \param #FAPEX_AUOUT_BUFFER2_0 Enable Buffer2-0
        \param #FAPEX_AUOUT_BUFFER2_1 Enable Buffer2-1
        \param #FAPEX_AUOUT_BUFFER3_0 Enable Buffer3-0
        \param #FAPEX_AUOUT_BUFFER3_1 Enable Buffer3-1
    */
    uint32_t channelFlag[4];
    /*!
        number of channels
    */
    uint32_t bufferCount;

} FAPEX_AUOUT_BufferParamStrT;

#define FAPEX_AUOUT_BUFFER0_0                (0x0001) //!< Enable Buffer0-0
#define FAPEX_AUOUT_BUFFER0_1                (0x0002) //!< Enable Buffer0-1
#define FAPEX_AUOUT_BUFFER1_0                (0x0004) //!< Enable Buffer1-0
#define FAPEX_AUOUT_BUFFER1_1                (0x0008) //!< Enable Buffer1-1
#define FAPEX_AUOUT_BUFFER2_0                (0x0010) //!< Enable Buffer2-0
#define FAPEX_AUOUT_BUFFER2_1                (0x0020) //!< Enable Buffer2-1
#define FAPEX_AUOUT_BUFFER3_0                (0x0040) //!< Enable Buffer3-0
#define FAPEX_AUOUT_BUFFER3_1                (0x0080) //!< Enable Buffer3-1

 /*!
*************************************************************************
** \brief Volume Control Parameter.
**
** This data structure contains the parameter needed by the Volume Control
** APT.
**
** \attention The structure must be binary comaptible between ARM and ARC.
**            Therefore any change of this structure would require a re-build 
**            of the audio firmware.
*************************************************************************
*/
typedef struct {
    
    /*!
        The target loudness level (0 - 255) per speaker channel.
    */
    uint8_t targetVolume[FAPEX_AUOUT_SPEAKER_MAX_ID + 1];
    
    /*!
        The current loudness level (0 - 255) per speaker channel the tool
        has generated.
        It is maintained by the tool. The initial values must be zero for 
        fade-in. 
    */
    uint8_t currentVolume[FAPEX_AUOUT_SPEAKER_MAX_ID + 1];
    
} FAPEX_AUOUT_AptVolParamsT;

/*!
*************************************************************************
** \brief ioctl specific command for the AUOUT driver
*************************************************************************/
#define FAPEX_IOCCOM_AUOUT_OPEN             _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_AUOUT + 0, FAPEX_AUOUT_OpenParamStrT)
#define FAPEX_IOCCOM_AUOUT_SET_PCM_INFO     _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_AUOUT + 1, FAPEX_AUOUT_PcmInfoStrT)
#define FAPEX_IOCCOM_AUOUT_GET_PCM_INFO     _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_AUOUT + 1, FAPEX_AUOUT_PcmInfoStrT)
#define FAPEX_IOCCOM_AUOUT_SET_STREAM_TYPE  _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_AUOUT + 2, int32_t)
#define FAPEX_IOCCOM_AUOUT_GET_STREAM_TYPE  _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_AUOUT + 2, int32_t)
#define FAPEX_IOCCOM_AUOUT_SET_MUTE         _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_AUOUT + 3, FAPEX_AUOUT_MuteParamStrT)
#define FAPEX_IOCCOM_AUOUT_SET_VOLUME       _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_AUOUT + 4, FAPEX_AUOUT_VolumeParamStrT)
#define FAPEX_IOCCOM_AUOUT_GET_VOLUME       _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_AUOUT + 4, FAPEX_AUOUT_VolumeParamStrT)
#define FAPEX_IOCCOM_AUOUT_SET_DUALMONO     _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_AUOUT + 5, FAPEX_AUOUT_DualMonoParamStrT)
#define FAPEX_IOCCOM_AUOUT_GET_SPEAKERS     _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_AUOUT + 6, FAPEX_AUOUT_SpeakersParamStrT)
#define FAPEX_IOCCOM_AUOUT_GET_I2SBUFFERS   _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_AUOUT + 7, FAPEX_AUOUT_BufferParamStrT)
#define FAPEX_IOCCOM_AUOUT_START_HDMI       _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_AUOUT + 8, uint32_t)
#define FAPEX_IOCCOM_AUOUT_STOP_HDMI        _IO(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_AUOUT + 9)
#define FAPEX_IOCCOM_AUOUT_SET_SPDIFLBIT    _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_AUOUT + 10, uint32_t)
#define FAPEX_IOCCOM_AUOUT_GET_SPDIFLBIT    _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_AUOUT + 10, uint32_t)
#define FAPEX_IOCCOM_AUOUT_GET_SYNCSTATUS   _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_AUOUT + 11, uint32_t)
#define FAPEX_IOCCOM_AUOUT_SET_SPDIFCOPYPROTECT _IOW(FAPEX_DEV_TYPE,FAPEX_IOCCOM_NB_AUOUT + 11,uint32_t)
#define FAPEX_IOCCOM_AUOUT_SET_SPDIFPCMLOCK _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_AUOUT + 12, uint32_t)
#define FAPEX_IOCCOM_AUOUT_SET_SPDIFMODE    _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_AUOUT + 13, uint32_t)
#define FAPEX_IOCCOM_AUOUT_ADD_APT          _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_AUOUT + 14, uint32_t)
#define FAPEX_IOCCOM_AUOUT_REMOVE_APT       _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_AUOUT + 15, uint32_t)
#define FAPEX_IOCCOM_AUOUT_SET_APTVOLPAR    _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_AUOUT + 16, uint32_t)
#define FAPEX_IOCCOM_AUOUT_SET_APTSAMPLESRC    _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_AUOUT + 17, uint32_t)


/*
*************************************************************************
*************************************************************************
** prototype for application
*************************************************************************
*************************************************************************/
#ifndef __LINUX__
#include "fapex/abst_types.h"

#ifdef __cplusplus
extern "C" {
#endif

FAPEXOPENRET_T FAPEX_AUOUT_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags, FAPEXOPENARG3_T argptr);
FAPEXRELRET_T  FAPEX_AUOUT_Close(FAPEXRELARG1_T id);
FAPEXIOCRET_T  FAPEX_AUOUT_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg);

#ifdef __cplusplus
}
#endif
#endif /* !__LINUX__ */

#endif /* !_FAPEX_ABST_LIBAUOUT_H */

