/*!
*************************************************************************
**
** \file    fapex/abst_libblte.h
**
** \brief   abstraction library header file (BLTE driver)
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Microelectronics Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Microelectronics Limited, All Rights Reserved.
**
*************************************************************************
*/

#ifndef _FAPEX_ABST_LIBBLTE_H
#define _FAPEX_ABST_LIBBLTE_H

#include "fapex/abst_lib.h"
#include "fapex/abst_error.h"

/*
*************************************************************************
*************************************************************************
** Defines and Macros
*************************************************************************
*************************************************************************/


/*
*************************************************************************
** Standard error codes for the driver
*************************************************************************/
/*! FAPex BLTE error code base value */
#define FAPEX_BLTE_ERR_BASE             (FAPEX_BLTE_MODULE_BASE + FAPEX_RET_ERR_BASE)

/*! Bad parameter passed. */
#define FAPEX_BLTE_ERR_BAD_PARAMETER    (FAPEX_BLTE_ERR_BASE + FAPEX_ERR_BAD_PARAMETER)

/*! Memory allocation failed. */
#define FAPEX_BLTE_ERR_OUT_OF_MEMORY    (FAPEX_BLTE_ERR_BASE + FAPEX_ERR_OUT_OF_MEMORY)

 //**************************************************************************
    //**************************************************************************
    //** Supported  modes
    //**************************************************************************
    //**************************************************************************
    #define FAPEX_BLTE_DMA_MODE  0   //!< DMA mode is the default value
    #define FAPEX_BLTE_CMD_MODE  1   //!< CMD draw mode 

    /*!
    *******************************************************************************
    **
    ** \brief BLTE input/output format codes.
    **
    *******************************************************************************
    */
    typedef enum
    {
        FAPEX_BLTE_MODE_8BIT        = (uint8_t)0, //8bits CLUT
        FAPEX_BLTE_MODE_YCBYCR      = (uint8_t)1, //16bits YCBCR
        FAPEX_BLTE_MODE_ARGB1555    = (uint8_t)3, //16bits ARGB1555
        FAPEX_BLTE_MODE_RGB565      = (uint8_t)4, //16bits RGB565
        FAPEX_BLTE_MODE_ARGB8888    = (uint8_t)5, //32bits ARGB8888
        FAPEX_BLTE_MODE_AYCbCr8888  = (uint8_t)6 //32bits AYCbCr8888
    }FAPEX_BLTE_ColorFormatEnumT;


   /*******************************************************************************
    **                                                                             
    ** \brief BLTE input/output corlor order.                                            
    **                                                                             
    *******************************************************************************
    */
    typedef enum
    {
        /* BLTE corlor order for CLUT 8bits */
	  FAPEX_BLTE_INPUT_ORDER_CLUT_ARGB    = (uint8_t)0, //ARGB 
	  FAPEX_BLTE_INPUT_ORDER_CLUT_AYCbCr  = (uint8_t)1,  //AYCbCr
	     
	   /* BLTE corlor order for 16bits YCbCr */
	  FAPEX_BLTE_ORDER_YCbCr_YCbYCr    = (uint8_t)2, //YCbYCr
	  FAPEX_BLTE_ORDER_YCbCr_YCrCb     = (uint8_t)3,  //YCrCb
	  FAPEX_BLTE_ORDER_YCbCr_CbYCrY    = (uint8_t)4,  //CbYCrY
	  FAPEX_BLTE_ORDER_YCbCr_CrYCbY    = (uint8_t)5, //CrYCbY
	     
	  /* BLTE color order for 16bits ARGB1555 */
	  FAPEX_BLTE_ORDER_ARGB1555_ARGB    = (uint8_t)6, //ARGB
        FAPEX_BLTE_ORDER_ARGB1555_ARBG    = (uint8_t)7, //ARBG
        FAPX_BLTE_ORDER_ARGB1555_AGRB    = (uint8_t)8, //AGRB
        FAPEX_BLTE_ORDER_ARGB1555_AGBR    = (uint8_t)9, //AGBR
        FAPEX_BLTE_ORDER_ARGB1555_ABRG    = (uint8_t)10, //ABRG
        FAPEX_BLTE_ORDER_ARGB1555_ABGR    = (uint8_t)11, //AGBR
        FAPEX_BLTE_ORDER_ARGB1555_RGBA    = (uint8_t)12, //RGBA
        FAPEX_BLTE_ORDER_ARGB1555_RBGA    = (uint8_t)13, //RGBA
        FAPEX_BLTE_ORDER_ARGB1555_GRBA    = (uint8_t)14, //GRBA
        FAPEX_BLTE_ORDER_ARGB1555_GBRA    = (uint8_t)15, //GBRA
        FAPEX_BLTE_ORDER_ARGB1555_BRGA    = (uint8_t)16,//BRGA
        FAPEX_BLTE_ORDER_ARGB1555_BGRA    = (uint8_t)17, //BGRA
        
        /* BLTE color order for 16bits RGB */ 
        FAPEX_BLTE_ORDER_RGB565_RGB    = (uint8_t)18, //ARGB
        FAPEX_BLTE_ORDER_RGB565_RBG    = (uint8_t)19,  //ARBG
        FAPEX_BLTE_ORDER_RGB565_GRB    = (uint8_t)20,  //AGRB
        FAPEX_BLTE_ORDER_RGB565_GBR    = (uint8_t)21, //AGBR
        FAPEX_BLTE_ORDER_RGB565_BRG    = (uint8_t)22, //ABRG
        FAPEX_BLTE_ORDER_RGB565_BGR    = (uint8_t)23,//AGBR
        
        /* BLTE color order for 32bits ARGB */
        FAPEX_BLTE_ORDER_ARGB8888_ARGB    = (uint8_t)24, //ARGB
        FAPEX_BLTE_ORDER_ARGB8888_ARBG    = (uint8_t)25,  //ARBG
        FAPEX_BLTE_ORDER_ARGB8888_AGRB    = (uint8_t)26,  //AGRB
        FAPEX_BLTE_ORDER_ARGB8888_AGBR    = (uint8_t)27, //AGBR
        FAPEX_BLTE_ORDER_ARGB8888_ABRG    = (uint8_t)28, //ABRG
        FAPEX_BLTE_ORDER_ARGB8888_ABGR    = (uint8_t)29, //AGBR
        FAPEX_BLTE_ORDER_ARGB8888_RGBA    = (uint8_t)30, //RGBA
        FAPEX_BLTE_ORDER_ARGB8888_RBGA    = (uint8_t)31,//RGBA
        FAPEX_BLTE_ORDER_ARGB8888_GRBA    = (uint8_t)32, //GRBA
        FAPEX_BLTE_ORDER_ARGB8888_GBRA    = (uint8_t)33, //GBRA
        FAPEX_BLTE_ORDER_ARGB8888_BRGA    = (uint8_t)34, //BRGA
        FAPEX_BLTE_ORDER_ARGB8888_BGRA    = (uint8_t)35, //BGRA
        
        /* BLTE color order for 32bits AYCbCr */
        FAPEX_BLTE_ORDER_AYCbCr_AYCbCr    = (uint8_t)36, //AYCbCr
        FAPEX_BLTE_ORDER_AYCbCr_AYCrCb    = (uint8_t)37, //AYCrCb
        FAPEX_BLTE_ORDER_AYCbCr_ACbYCr    = (uint8_t)38, //ACbYCr
        FAPEX_BLTE_ORDER_AYCbCr_ACbCrY    = (uint8_t)39, //ACbCrY
        FAPEX_BLTE_ORDER_AYCbCr_ACrYCb    = (uint8_t)40, //ACrYCb
        FAPEX_BLTE_ORDER_AYCbCr_ACrCbY    = (uint8_t)41, //ACrCbY
        FAPEX_BLTE_ORDER_AYCbCr_YCbCrA    = (uint8_t)42, //YCbCrA
        FAPEX_BLTE_ORDER_AYCbCr_YCrCbA    = (uint8_t)43, //YCrCbA
        FAPEX_BLTE_ORDER_AYCbCr_CbYCrA    = (uint8_t)44, //CbYCrA
        FAPEX_BLTE_ORDER_AYCbCr_CbCrYA    = (uint8_t)45, //CbCrYA
        FAPEX_BLTE_ORDER_AYCbCr_CrYCbA    = (uint8_t)46,//CrYCbA
        FAPEX_BLTE_ORDER_AYCbCr_CrCbYA    = (uint8_t)47 //CrCbYA
	      
    }FAPEX_BLTE_ColorOrderEnumT;


/*!
     ******************************************************************************
     **
     ** \brief BLTE scale
     **
    *******************************************************************************
    */
    typedef enum{
        FAPEX_BLTE_BITBLT_SCALE_NORMAL = (uint8_t)0,
        FAPEX_BLTE_BITBLT_SCALE_2TIMES = (uint8_t)1,
        FAPEX_BLTE_BITBLT_SCALE_HALFTIMES = (uint8_t)2
    }FAPEX_BLTE_BitBltScaleEnumT;
    
    /*!
     ******************************************************************************
     **
     ** \brief BLTE blend mode
     **
    *******************************************************************************
    */
    typedef enum{
        FAPEX_BLTE_BITBLT_SCALE_INTERPOLATION = (uint8_t)0,
        FAPEX_BLTE_BITBLT_SCALE_LINEAR = (uint8_t)1
    }FAPEX_BLTE_BitBltScaleModeEnumT;
    /*!
     ******************************************************************************
     **
     ** \brief BLTE blend mode
     **
    *******************************************************************************
    */
    typedef enum
    {
        FAPEX_BLTE_BM_NORMAL =(uint8_t)0,
        FAPEX_BLTE_BM_BLEND = (uint8_t)1,
        FAPEX_BLTE_BM_LOGOP = (uint8_t)2
    }FAPEX_BLTE_BlendModeEnumT;
    /*!                                                                            
     *******************************************************************************
     **                                                                             
     ** \brief BLTE logical operation.                                            
     **                                                                             
    *******************************************************************************
    */
    typedef enum
    {
        FAPEX_BLTE_LOGOP_CLEAR     = (uint8_t)0, //0
        FAPEX_BLTE_LOGOP_AND       = (uint8_t)1, // s&d
        FAPEX_BLTE_LOGOP_AND_REV   = (uint8_t)2, // s&~d
        FAPEX_BLTE_LOGOP_COPY      = (uint8_t)3, // s
        FAPEX_BLTE_LOGOP_AND_INV   = (uint8_t)4, // ~s&d
        FAPEX_BLTE_LOGOP_NOP       = (uint8_t)5, // d 
        FAPEX_BLTE_LOGOP_XOR       = (uint8_t)6, // s^d
        FAPEX_BLTE_LOGOP_OR        = (uint8_t)7, // s|d
        FAPEX_BLTE_LOGOP_NOR       = (uint8_t)8, // ~(s|d)
        FAPEX_BLTE_LOGOP_EQUIV     = (uint8_t)9, // ~(s^d)
        FAPEX_BLTE_LOGOP_INV       = (uint8_t)10,// ~d
        FAPEX_BLTE_LOGOP_OR_REV    = (uint8_t)11, // s|~d 
        FAPEX_BLTE_LOGOP_COPY_INV  = (uint8_t)12, // ~s
        FAPEX_BLTE_LOGOP_OR_INV    = (uint8_t)13, // ~s|d
        FAPEX_BLTE_LOGOP_NAND      = (uint8_t)14, // ~(s&d)
        FAPEX_BLTE_LOGOP_SET       = (uint8_t)15  // all 1's
    }FAPEX_BLTE_LogOperationEnumT;

    /*!                                                                            
     *******************************************************************************
     **                                                                             
     ** \brief BLTE blend operation.                                            
     **                                                                             
    ********************************************************************************
    */
    typedef enum
    {
        FAPEX_BLTE_BLENDOP_ADD       = (uint8_t)0, 
        FAPEX_BLTE_BLENDOP_SUB       = (uint8_t)1, 
        FAPEX_BLTE_BLENDOP_REV_SUB   = (uint8_t)2, 
        FAPEX_BLTE_BLENDOP_MIN       = (uint8_t)3, 
        FAPEX_BLTE_BLENDOP_MAX       = (uint8_t)4
    }FAPEX_BLTE_BlendFuncEnumT;
      
    /*!                                                                                  
     *******************************************************************************     
     **                                                                                  
     ** \brief BLTE blend coefficient .                                                     
     **                                                                                  
    ********************************************************************************
    */      
    typedef enum
    {
        FAPEX_BLTE_COEFF_ZERO  = (uint8_t)0,
        FAPEX_BLTE_COEFF_ONE   = (uint8_t)1,
        FAPEX_BLTE_COEFF_SRC_COLOR = (uint8_t)2,
        FAPEX_BLTE_COEFF_ONE_MINUS_SRC_COLOR = (uint8_t)3, 
        FAPEX_BLTE_COEFF_DST_COLOR = (uint8_t)4,
        FAPEX_BLTE_COEFF_ONE_MINUS_DST_COLOR = (uint8_t)5,
        FAPEX_BLTE_COEFF_SRC_ALPHA = (uint8_t)6,
        FAPEX_BLTE_COEFF_ONE_MINUS_SRC_ALPHA = (uint8_t)7,
        FAPEX_BLTE_COEFF_DST_ALPHA = (uint8_t)8,
        FAPEX_BLTE_COEFF_ONE_MINUS_DST_ALPHA = (uint8_t)9,
        FAPEX_BLTE_COEFF_CONSTANT_COLOR = (uint8_t)10,
        FAPEX_BLTE_COEFF_ONE_MINUS_CONSTANT_COLOR = (uint8_t)11,
        FAPEX_BLTE_COEFF_CONSTANT_ALPHA = (uint8_t)12,
        FAPEX_BLTE_COEFF_ONE_MINUS_CONSTANT_ALPHA = (uint8_t)13,
        FAPEX_BLTE_COEFF_SRC_ALPHA_SATURATE = (uint8_t)14
    }FAPEX_BLTE_BlendCoefEnumT;   
    
    /*!                                                                                  
     *******************************************************************************     
     **                                                                                  
     ** \brief BLTE shading mode.                                                     
     **                                                                                  
    ********************************************************************************
    */      
    typedef enum
    {
        FAPEX_BLTE_SHADING_NORMAL  = (uint8_t)0,
        FAPEX_BLTE_SHADING_GROUND  = (uint8_t)1
    }FAPEX_BLTE_ShadingModeEnumT;   
    
    /*!
    *******************************************************************************
    **
    ** \brief BLTE GenThumbnail mode
    **
    */
    typedef enum{
        FAPEX_BLTE_THUMBNAIL_MODE_NORM = (uint8_t)0,
        FAPEX_BLTE_THUMBNAIL_MODE_POSTARY = (uint8_t)1,
        FAPEX_BLTE_THUMBNAIL_MODE_BINARY = (uint8_t)2
    }FAPEX_BLTE_ThumbnailModeEnumT;
    
    
    /*!
    *****************************************************************************
    ** Structure of color parameter
    *****************************************************************************
    */
    typedef struct{
        /* color format */
        FAPEX_BLTE_ColorFormatEnumT format;
        /* color order */
        FAPEX_BLTE_ColorOrderEnumT order;
    } FAPEX_BLTE_LayoutT;
    
    /*!
    *****************************************************************************
    ** Structure of a rectangular area
    *****************************************************************************
    */
    typedef struct{
        /* x position of top left point */
        uint32_t x;
        /* y position of top left point */
        uint32_t y;
        /* width of the given area */
        uint32_t w;
        /* height of the given area */
        uint32_t h;
    
    } FAPEX_BLTE_RectT;

        /*!
    ***************************************************************************
    **
    ** \brief blte key color parameter
    **
    ***************************************************************************
    */
    typedef struct{
        uint8_t   enable;
        uint32_t  keycolor;
    }FAPEX_BLTE_ColorKeyParamT;
    
    /*!
    ***************************************************************************
    **
    ** \brief blte ground shading parameter
    **
    ***************************************************************************
    */
    typedef struct{
        /* color value of top right vertex specifying in RGBA order */
        uint32_t color0;
        /* right side R component step size */
        int16_t  rightStepR;
        /* right side G component step size */
        int16_t  rightStepG;
        /* right side B component step size */
        int16_t  rightStepB;
        /* right side A component step size */
        int16_t  rightStepA;
        /* color value at top left vertex specifying in RGBA order */
        uint32_t color1;
        /* left side R component step size */
        int16_t  leftStepR;
        /* left side G component step size */
        int16_t  leftStepG;
        /* left side B component step size */
        int16_t  leftStepB;
        /* left side A component step size */
        int16_t  leftStepA;
        /* enable Gouraud shading on alpha component also. MDR4_As */
        uint32_t enableAlphaShading;
    } FAPEX_BLTE_GouraudShadingParamsT;
   
    /*!
    ***************************************************************************
    **
    ** \brief blte shading parameter
    **
    ***************************************************************************
    */
    typedef struct{
        /* mode: flat or Gouraud shading */
        FAPEX_BLTE_ShadingModeEnumT mode;
        FAPEX_BLTE_GouraudShadingParamsT gouraudParams;
    
    } FAPEX_BLTE_ShadingParamsT;
    
    /*!
    ***************************************************************************
    **
    ** \brief blte blend parameter
    **
    ***************************************************************************
    */
    typedef struct{
        FAPEX_BLTE_BlendCoefEnumT      dstFunc;
        FAPEX_BLTE_BlendCoefEnumT      srcFunc;
        FAPEX_BLTE_BlendFuncEnumT      equation;
        uint32_t                  enableBlend;
        uint32_t                  useGlobalalpha;
        uint32_t                  globalAlpha;
        uint32_t                  constColor; /* for blend with const color */
    } FAPEX_BLTE_BlendFuncParamsT;
    
    /*!
    ***************************************************************************
    **
    ** \brief blte alpha blend operation parameter
    **
    ***************************************************************************
    */
    typedef struct{
        /* mode: normal, blending with function, or with logical operation */
        FAPEX_BLTE_BlendModeEnumT mode;
        FAPEX_BLTE_BlendFuncParamsT    blendFuncParams;
        FAPEX_BLTE_LogOperationEnumT  logicOp;
    } FAPEX_BLTE_BlendingParamsT;
    
    /*!
    ***************************************************************************
    **
    ** \brief blte draw rect parameter
    **
    ***************************************************************************
    */
    typedef struct{
        uint32_t width;
        uint32_t height;
        uint32_t frameWidth;
        uint32_t frameColor;
        uint32_t fillColor;
    } FAPEX_BLTE_RectParamsT;
   
   
   /*!
    ***************************************************************************
    **
    ** \brief blte draw font parameter
    **
    ***************************************************************************
    */
   typedef struct{
        uint32_t width;
        uint32_t height;
        uint32_t fontColor;
        /* flag to set background as transparent */
        uint32_t BgTrans;
        /* set font background to some color if BgTrans = 0 */
        uint32_t BgColor;
        /* select form: normal, double size, half size */
        FAPEX_BLTE_BitBltScaleEnumT xScale;
        /* select form: normal, double size, half size */
        FAPEX_BLTE_BitBltScaleEnumT yScale;
        FAPEX_BLTE_BitBltScaleModeEnumT scaleMode;
    } FAPEX_BLTE_FontParamsT;
    
    /*!                                                                                  
     *******************************************************************************     
     **                                                                                  
     ** \brief Line mode.                                                     
     **                                                                                  
    ********************************************************************************
    */      
    typedef enum
    {
        FAPEX_BLTE_ULINE_DASH  = (uint8_t)0,
        FAPEX_BLTE_ULINE_SOLID  = (uint8_t)1
    }FAPEX_BLTE_UlineModeEnumT;     
    
    
    /*!
    ***************************************************************************
    **
    ** \brief blte draw underline parameter
    **
    ***************************************************************************
    */
    typedef struct{
        uint32_t length;
        uint32_t width;
        /* select from solid or dashed line */
        FAPEX_BLTE_UlineModeEnumT type;
        uint32_t dashColor;
        uint32_t spaceColor;
        uint32_t dashPattern;
        uint32_t BgTrans;     
    } FAPEX_BLTE_ULineParamsT;   
    
    /*!
    ***************************************************************************
    **
    ** \brief blte draw underline parameter
    **
    ***************************************************************************
    */
    typedef struct{
        /* specify a source address to enable bltCopy from a different frame */
        uint32_t*       srcAddr;
        /* srcStride is considered if srcAddr != NULL*/
        uint32_t        srcStride;
        FAPEX_BLTE_RectT srcRect;
        /* horizontal flipping flag */
        uint32_t        horFlip;
        /* vertical flipping flag */
        uint32_t        verFlip;
    
    } FAPEX_BLTE_CopyParamsT;
    
    /*!
    ***************************************************************************
    **
    ** \brief blte Postarizationmode parameter
    **
    ***************************************************************************
    */
    typedef struct{
        uint32_t rBits;
        uint32_t gBits;
        uint32_t bBits;
    
    } FAPEX_BLTE_PostParamsT;
    
    /*!
    ***************************************************************************
    **
    ** \brief blte binarization parameter
    **
    ***************************************************************************
    */
    typedef struct{
        uint8_t  thresholdValue;
        uint32_t rgba_high;
        uint32_t rgba_low;
    
    } FAPEX_BLTE_BinParamsT;

    /*!
    ***************************************************************************
    **
    ** \brief blte bitmap parameter
    **
    ***************************************************************************
    */
    typedef struct{
        /* specify a source address of the picture */
        uint32_t *srcAddr;
        /* srcStride */
        uint32_t srcStride;
        FAPEX_BLTE_RectT srcRect;
        /* mode can be normal, posterization, or binarization*/
        FAPEX_BLTE_ThumbnailModeEnumT mode;
        /* posterization parameters */
        FAPEX_BLTE_PostParamsT postarParam;
        /* binarization parameters */
        FAPEX_BLTE_BinParamsT binaryParam;
    
    } FAPEX_BLTE_BitmapParamsT;
    
    /*!
    ***************************************************************************
    **
    ** \brief blte MTXI parameter
    **
    ***************************************************************************
    */
    typedef struct{
        
        int32_t R_y; //!< y coefficient for the R part in the conversion formular(coefficient * 1024). 
        int32_t R_u; //!< u coefficient for the R part in the conversion formular(coefficient * 1024).
        int32_t R_v; //!< v coefficient for the R part in the conversion formular(coefficient * 1024).
        int32_t R_offset; //!< offset value for R 
        
        int32_t G_y;  //!< y coefficient for the G part in the conversion formular(coefficient * 1024). 
        int32_t G_u;  //!< u coefficient for the G part in the conversion formular(coefficient * 1024). 
        int32_t G_v;  //!< v coefficient for the G part in the conversion formular(coefficient * 1024).
        int32_t G_offset; //!< offset value for G
        
        int32_t B_y;   //!< y coefficient for the B part in the conversion formular(coefficient * 1024).
        int32_t B_u;   //!< u coefficient for the B part in the conversion formular(coefficient * 1024).
        int32_t B_v;    //!< v coefficient for the B part in the conversion formular(coefficient * 1024).
        int32_t B_offset;//!< offset value for B
       
    }FAPEX_BLTE_MTXIParamsT;
    
    /*!
    ***************************************************************************
    **
    ** \brief blte MTXO parameter
    **
    ***************************************************************************
    */
    typedef struct{
        
        int32_t Y_green; //!< green coefficient for the Y part in the conversion formular(coefficient * 1024). 
        int32_t Y_blue; //!< blue coefficient for the Y part in the conversion formular(coefficient * 1024).
        int32_t Y_red; //!< red coefficient for the Y part in the conversion formular(coefficient * 1024).
        int32_t Y_offset; //!< offset value for Y
        
        int32_t U_green;  //!< green coefficient for the U part in the conversion formular(coefficient * 1024). 
        int32_t U_blue;  //!< blue coefficient for the U part in the conversion formular(coefficient * 1024). 
        int32_t U_red;  //!< red coefficient for the U part in the conversion formular(coefficient * 1024).
        int32_t U_offset; //!< offset value for U
        
        int32_t V_green;   //!< y coefficient for the B part in the conversion formular(coefficient * 1024).
        int32_t V_blue;   //!< u coefficient for the B part in the conversion formular(coefficient * 1024).
        int32_t V_red;    //!< v coefficient for the B part in the conversion formular(coefficient * 1024).
        int32_t V_offset;//!< offset value for B
       
    }FAPEX_BLTE_MTXOParamsT;

    /*!
    ***************************************************************************
    **
    ** \brief set blte CLUT parameter
    **
    ***************************************************************************
    */
    typedef struct{
        uint8_t index;
	      int32_t value[256];
    }FAPEX_BLTE_ClutParamT;

    /*!
    ***************************************************************************
    **
    ** \brief set blte MTXI and MTXO parameter
    **
    ***************************************************************************
    */
    typedef struct{
        uint8_t index;
	      int32_t value[12];
    }FAPEX_BLTE_MtxParamT;

   /*!
    ***************************************************************************
    **
    ** \brief set blte clip parameter
    **
    ***************************************************************************
    */
    typedef struct{
        uint8_t enable;
	      FAPEX_BLTE_RectT  clipRect;
    }FAPEX_BLTE_ClipParamT;

   /*!
    ***************************************************************************
    **
    ** \brief set blte source layout  parameter
    **
    ***************************************************************************
    */
    typedef struct{
        uint8_t index;
        FAPEX_BLTE_LayoutT format;
   }FAPEX_BLTE_FormatParamT;

    /*!
    ***************************************************************************
    **
    ** \brief draw rect param
    **
    ***************************************************************************
    */
    typedef struct{
        uint32_t x;
        uint32_t y;
        FAPEX_BLTE_RectParamsT format;
   }FAPEX_BLTE_DrawRectParamT;

    /*!
    ***************************************************************************
    **
    ** \brief draw font param
    **
    ***************************************************************************
    */
    typedef struct{
        uint32_t x;
        uint32_t y;
        FAPEX_BLTE_FontParamsT format;
        uint32_t  fontData;
   }FAPEX_BLTE_DrawFontParamT;

    /*!
    ***************************************************************************
    **
    ** \brief draw underline param
    **
    ***************************************************************************
    */
    typedef struct{
        uint32_t x;
        uint32_t y;
        FAPEX_BLTE_ULineParamsT format;
   }FAPEX_BLTE_DrawUlineParamT;   

    /*!
    ***************************************************************************
    **
    ** \brief COPY param
    **
    ***************************************************************************
    */
    typedef struct{
        uint32_t x;
	      uint32_t y;
        FAPEX_BLTE_CopyParamsT format;
   }FAPEX_BLTE_CopyParamT;   

   /*!
    ***************************************************************************
    **
    ** \brief draw bitmap param
    **
    ***************************************************************************
    */
    typedef struct{
        FAPEX_BLTE_RectT   pos;  
        FAPEX_BLTE_BitmapParamsT format;
   }FAPEX_BLTE_DrawBitmapParamT;   

   /*!
    ****************************************************************************
    ** \brief Open parameters for the BLTE driver.
    **
    ** This data structure covers all parameters which need to be specified
    ** when an instance of the driver is opened.
    **
    ****************************************************************************
    */
    typedef struct {
       
        uint32_t  baseAddr;
        
        /*!
        ** the color format of the frame buffer
        **
        */
        FAPEX_BLTE_ColorFormatEnumT  colormode;
        
        /*!
        **
        **the color order of the frame buffer
        **
        */
        FAPEX_BLTE_ColorOrderEnumT  colororder;
        
        /*!
        ** the x size of the osd in pixel.
        */
        uint32_t  xsize;        
        uint32_t  ysize;        
    } FAPEX_BLTE_OpenParamsT;

    /*!
    ****************************************************************************
    ** \brief Change the framebase parameters for the BLTE driver.
    **
    ** This data structure covers all parameters which need to be specified
    ** when an instance of the driver is opened.
    **
    ****************************************************************************
    */
    typedef struct {
       
        uint32_t  framebase;
        
        /*!
        ** the color format of the frame buffer
        **
        */
        FAPEX_BLTE_ColorFormatEnumT  colormode;
        
        /*!
        **
        **the color order of the frame buffer
        **
        */
        FAPEX_BLTE_ColorOrderEnumT  colororder;
        
        /*!
        ** the x size of the osd in pixel.
        */
        uint32_t  xsize;        
        uint32_t  ysize;        
    } FAPEX_BLTE_SetFrameBaseParamsT;


/*!
*************************************************************************
** \brief ioctl specific command for the BLTE driver
*************************************************************************/
#define FAPEX_IOCCOM_BLTE_OPEN            _IOW(FAPEX_DEV1_TYPE, FAPEX_IOCCOM_NB_BLTE + 0,  FAPEX_BLTE_OpenParamsT)
#define FAPEX_IOCCOM_BLTE_COLOR_KEY 	  _IOW(FAPEX_DEV1_TYPE, FAPEX_IOCCOM_NB_BLTE + 1, FAPEX_BLTE_ColorKeyParamT)
#define FAPEX_IOCCOM_BLTE_SHADING_PARAM   _IOW(FAPEX_DEV1_TYPE, FAPEX_IOCCOM_NB_BLTE + 2, FAPEX_BLTE_ShadingParamsT)
#define FAPEX_IOCCOM_BLTE_BLENDING_PARAM  _IOW(FAPEX_DEV1_TYPE, FAPEX_IOCCOM_NB_BLTE + 3, FAPEX_BLTE_BlendingParamsT)
#define FAPEX_IOCCOM_BLTE_MTX_PARAM     _IOW(FAPEX_DEV1_TYPE, FAPEX_IOCCOM_NB_BLTE + 4, FAPEX_BLTE_MtxParamT)
#define FAPEX_IOCCOM_BLTE_CLUT_PARAM    _IOW(FAPEX_DEV1_TYPE, FAPEX_IOCCOM_NB_BLTE + 5, FAPEX_BLTE_ClutParamT)
#define FAPEX_IOCCOM_BLTE_CLIP_PARAM    _IOW(FAPEX_DEV1_TYPE,FAPEX_IOCCOM_NB_BLTE+6, FAPEX_BLTE_ClipParamT)
#define FAPEX_IOCCOM_BLTE_SOURCE_FORMAT _IOW(FAPEX_DEV1_TYPE,FAPEX_IOCCOM_NB_BLTE+7, FAPEX_BLTE_FormatParamT)
#define FAPEX_IOCCOM_BLTE_DRAW_RECT     _IOW(FAPEX_DEV1_TYPE,FAPEX_IOCCOM_NB_BLTE+8,FAPEX_BLTE_DrawRectParamT)
#define FAPEX_IOCCOM_BLTE_DRAW_FONT     _IOW(FAPEX_DEV1_TYPE,FAPEX_IOCCOM_NB_BLTE+9,FAPEX_BLTE_DrawFontParamT)
#define FAPEX_IOCCOM_BLTE_DRAW_ULINE   _IOW(FAPEX_DEV1_TYPE,FAPEX_IOCCOM_NB_BLTE+10,FAPEX_BLTE_DrawUlineParamT)
#define FAPEX_IOCCOM_BLTE_DRAW_COPY    _IOW(FAPEX_DEV1_TYPE,FAPEX_IOCCOM_NB_BLTE+11,FAPEX_BLTE_CopyParamT)
#define FAPEX_IOCCOM_BLTE_DRAW_BITMAP  _IOW(FAPEX_DEV1_TYPE,FAPEX_IOCCOM_NB_BLTE+12,FAPEX_BLTE_DrawBitmapParamT)
#define FAPEX_IOCCOM_BLTE_SET_FRAME      _IOW(FAPEX_DEV1_TYPE, FAPEX_IOCCOM_NB_BLTE+13, FAPEX_BLTE_SetFrameBaseParamsT)

/*
*************************************************************************
*************************************************************************
** prototype for application
*************************************************************************
*************************************************************************/
#ifndef __LINUX__
#include "fapex/abst_types.h"

#ifdef __cplusplus
}
#endif

FAPEXOPENRET_T FAPEX_BLTE_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags, FAPEXOPENARG3_T argptr);
FAPEXRELRET_T  FAPEX_BLTE_Close(FAPEXRELARG1_T id);
FAPEXIOCRET_T  FAPEX_BLTE_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg);

#ifdef __cplusplus
}
#endif

#endif /* !__LINUX__ */

#endif /* !_FAPEX_ABST_LIBBLTE_H */

