/*!
*************************************************************************
**
** \file    fapex/abst_libicc.h
**
** \brief   abstraction library header file (ICC driver)
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#ifndef _FAPEX_ABST_LIBICC_H
#define _FAPEX_ABST_LIBICC_H

#include "fapex/abst_lib.h"
#include "fapex/abst_error.h"

/*
*************************************************************************
*************************************************************************
** Defines and Macros
*************************************************************************
*************************************************************************/

/*
*************************************************************************
** Standard error codes for the driver
*************************************************************************/
/*! FAPex ICC error code base value */
#define FAPEX_ICC_ERR_BASE             (FAPEX_ICC_MODULE_BASE + FAPEX_RET_ERR_BASE)

/*! Bad parameter passed. */
#define FAPEX_ICC_ERR_BAD_PARAMETER    (FAPEX_ICC_ERR_BASE + FAPEX_ERR_BAD_PARAMETER)

/*! Memory allocation failed. */
#define FAPEX_ICC_ERR_OUT_OF_MEMORY    (FAPEX_ICC_ERR_BASE + FAPEX_ERR_OUT_OF_MEMORY)

/*! Timeout occured. */
#define FAPEX_ICC_ERR_TIMEOUT          (FAPEX_ICC_ERR_BASE + FAPEX_ERR_TIMEOUT)

/*! Invalid status. */
#define FAPEX_ICC_ERR_INVALID_STATUS   (FAPEX_ICC_ERR_BASE + FAPEX_ERR_INVALID_HANDLE)

/*! Semaphore could not be created. */
#define FAPEX_ICC_ERR_SEMAPHORE_CREATE (FAPEX_ICC_ERR_BASE + FAPEX_ERR_SEMAPHORE_CREATE)

/*
*************************************************************************
** ICC status flag
*************************************************************************/
#define FAPEX_ICC_STATUS_NORMAL                 0x0000  //!< normal status
#define FAPEX_ICC_STATUS_CARD_IN                0x0001  //<! ICC card inserted
#define FAPEX_ICC_STATUS_CARD_OUT               0x0002  //<! ICC card removed
#define FAPEX_ICC_STATUS_FINISHED_RESET_COLD    0x0004  //<!
#define FAPEX_ICC_STATUS_FINISHED_RESET_WARM    0x0008  //<!
#define FAPEX_ICC_STATUS_CLOSED                 0x0010  //<! closed status
#define FAPEX_ICC_STATUS_ERR_SLOT_EMPTY         0x0100  //<! card slot is empty
#define FAPEX_ICC_STATUS_ERR_RUNNING            0x0200  //<! an ICC function is currently running
#define FAPEX_ICC_STATUS_ERR_OVERFLOW           0x0400  //<! ICC read buffer overflow detected while reading data
#define FAPEX_ICC_STATUS_ERR_NO_ANSWER          0x0800  //<! no answer from smartcard
#define FAPEX_ICC_STATUS_ERR_TIMEOUT            0x1000  //<! Timeout occured
#define FAPEX_ICC_STATUS_ERR_UNKNOWN            0x2000  //<! unknown error

#define FAPEX_ICC_STATUS_CARD_IN_OUT            (FAPEX_ICC_STATUS_CARD_IN | FAPEX_ICC_STATUS_CARD_OUT)
#define FAPEX_ICC_STATUS_FINISHED_RESET         (FAPEX_ICC_STATUS_FINISHED_RESET_COLD | FAPEX_ICC_STATUS_FINISHED_RESET_WARM)
#define FAPEX_ICC_STATUS_NORMAL_CAHNGE          (FAPEX_ICC_STATUS_CARD_IN_OUT | FAPEX_ICC_STATUS_FINISHED_RESET)
#define FAPEX_ICC_STATUS_WARNING                (FAPEX_ICC_STATUS_CLOSED)
#define FAPEX_ICC_STATUS_ERROR                  \
 (FAPEX_ICC_STATUS_ERR_SLOT_EMPTY | FAPEX_ICC_STATUS_ERR_RUNNING | \
  FAPEX_ICC_STATUS_ERR_OVERFLOW | FAPEX_ICC_STATUS_ERR_NO_ANSWER | \
  FAPEX_ICC_STATUS_ERR_TIMEOUT | FAPEX_ICC_STATUS_ERR_UNKNOWN)

/*
*************************************************************************
** ICC reset mode
*************************************************************************/
#define FAPEX_ICC_RESET_MODE_WARM           (0)     //!< Requests a warm reset of the ICC
#define FAPEX_ICC_RESET_MODE_COLD           (1)     //!< Requests a cold reset of the ICC

/*
*************************************************************************
** ATR parameter
*************************************************************************/
#define FAPEX_ICC_ATR_MAX_SIZE              33      //!< Maximum size of ATR byte array
#define FAPEX_ICC_ATR_MAX_HIST_SIZE         15      //!< Maximum number of historical bytes


#define FAPEX_ICC_CARD_THREAD_STACKSIZE     1024
#define FAPEX_ICC_CARD_THREAD_PRIORITY      FAPEX_SYS_PRIORITY_MEDIUM
#define FAPEX_ICC_CARD_THREAD_NAME          "abst:ICCcardpoll"

/*
*************************************************************************
*************************************************************************
** Data types
*************************************************************************
*************************************************************************/
/*!
*************************************************************************
** \brief open parameters for the ICC driver
**
** \sa FAPEX_ICC_Open
*************************************************************************/
typedef void(*FAPEX_ICC_CardNotifierCALLBACK_T)(void);      //!< the card in/out notifier function signature type

typedef struct {
    /*!
        The number of the protocol which is inserted T=0:0, T=1:1
        \note Be sure that the number is correct with the protocol
    */
    uint32_t    protocolMode;

} FAPEX_ICC_OpenParamStrT;

/*!
*************************************************************************
** \brief open parameters for the ICC driver
**
** The related ioctl command type :
** \ref icc_cmd_open_subid "FAPEX_IOCCOM_ICC_OPEN_SUBID"
**
** \sa FAPEX_ICC_Ioctl
*************************************************************************/
typedef struct {
    /*!
        sub id<br>
        \param[in]  sub_id  Device minor_no (#FAPEXDEV_ICC_SUB_MINOR_FIRST .. #FAPEXDEV_ICC_SUB_MINOR_LAST).
        \param[out] sub_id  Device identifier for ICC channel.
    */
    uint32_t sub_id;
    /*!
        The index of the SmartCard to open, either 0 or 1
    */
    uint32_t    cardIndex;
    /*!
        Size of ATR data buffer
    */
    uint32_t    atrSize;
    /*!
        Size of historical buffer
    */
    uint32_t    histSize;
    /*!
         card-in user callback function
    */
    FAPEX_ICC_CardNotifierCALLBACK_T  cardInCallback;
   /*!
         card-out user callback function
    */
    FAPEX_ICC_CardNotifierCALLBACK_T  cardOutCallback;

} FAPEX_ICC_OpenSubIdParamStrT;

/*!
*************************************************************************
** \brief ioport parameters for the ICC driver
**
** The related ioctl command type :
** \ref icc_cmd_open_subid "FAPEX_IOCCOM_ICC_SET_IOPORT",
** \ref icc_cmd_get_ioport "FAPEX_IOCCOM_ICC_GET_IOPORT"
**
** \sa FAPEX_ICC_Ioctl
*************************************************************************/
typedef struct {
    /*!
        card clock in Hz
    */
    uint32_t    cardClock;
    /*!
        default baud rate (cardClock/372)
    */
    uint32_t    baudRate;
    /*!
        signal convention to be used, either direct or inverse<br>
        \param #FAPEX_ICC_CONVENTION_DIRECT   signal convention direct mode
        \param #FAPEX_ICC_CONVENTION_INVERSE  signal convention inverse mode
    */
    uint32_t    convention;
    /*!
        parity bit to be used, either odd, even or none<br>
        \param #FAPEX_ICC_PARITY_ODD   odd parity
        \param #FAPEX_ICC_PARITY_EVEN  even parity
        \param #FAPEX_ICC_PARITY_NON   parity none
    */
    uint32_t    parity;
    /*!
        data resending type in case of bit errors<br>
        enable:1 or disable:0
    */
    uint32_t    parityErrResend;
    /*!
        data bit with, either 8 or 9 bits
    */
    uint32_t    dataBits;
   /*!
        Enable flow control for NDS cards
        pssible values are: <br>
        #FAPEX_ICC_FLOWCONTROL_OFF <br>
        #FAPEX_ICC_FLOWCONTROL_ON
     */
    uint32_t    flowControl;

} FAPEX_ICC_IOportParamStrT;

// signal convention
#define FAPEX_ICC_CONVENTION_INVERSE            0
#define FAPEX_ICC_CONVENTION_DIRECT             1

// parity type
#define FAPEX_ICC_PARITY_ODD                    0
#define FAPEX_ICC_PARITY_EVEN                   1
#define FAPEX_ICC_PARITY_NON                    2

// flow control enable
#define FAPEX_ICC_FLOWCONTROL_OFF               0
#define FAPEX_ICC_FLOWCONTROL_ON                1


/*!
*************************************************************************
** \brief ATR parameters for the ICC driver
**
** The related ioctl command type :
** \ref icc_cmd_get_atr "FAPEX_IOCCOM_ICC_GET_ATR"
**
** \sa FAPEX_ICC_Ioctl
*************************************************************************/
typedef struct {
    /*!
        specifies the level coding, connecting level Z/A to digits 1 or 0
    */
    uint32_t    convention;
    /*!
        specifies the smart card protocol. (T=0 or T=1)
    */
    uint32_t    protocol;
    /*!
        specifies the clock rate conversion factor
    */
    uint32_t    FI;
    /*!
        specifies the bit rate adjustment factor
    */
    uint32_t    DI;
    /*!
        specifies the programming voltage factor
    */
    uint8_t     PI1;
    /*!
        specifies the programming voltage factor
    */
    uint8_t     PI2;
    /*!
        specifies the maximum programming current factor
    */
    uint8_t     II;
    /*!
        specifies extra guard time
    */
    uint8_t     N;
    /*!
        specifies the clock stop indicatior
    */
    uint32_t    XI;
    /*!
        specifies the class indicatior
    */
    uint32_t    UI;
    /*!
        is a pointer to historical data
    */
    uint8_t     *histBuffer;
    /*!
        specifies the length of the historical field
    */
    uint32_t    histBytes;
    /*!
        all of transmitted bytes at response of ATR
    */
    uint8_t     *atrBuffer;
    /*!
        number of recieved byte at ATR
    */
    uint32_t    atrBytes;

} FAPEX_ICC_AtrParamStrT;

/*!
*************************************************************************
** \brief data parameters sent and received for the ICC driver
**
** The related ioctl command type :
** \ref icc_cmd_get_data "FAPEX_IOCCOM_ICC_GET_DATA"
**
** \sa FAPEX_ICC_Ioctl
*************************************************************************/
typedef struct {
    /*!
        a pointer to a buffer containg the data to be send
    */
    uint8_t     *sendBuffer;
    /*!
        the number of bytes in the send buffer to be transfered
    */
    uint32_t    sendSize;
    /*!
        a pointer to the receive buffer where to store the response
    */
    uint8_t     *receiveBuffer;
    /*!
        size of the receive buffer<br>
        [in]  max size<br>
        [out] received size
    */
    uint32_t    receiveSize;

} FAPEX_ICC_DataParamStrT;


/*!
*************************************************************************
** \brief ioctl specific command for the ICC driver
*************************************************************************/
#define FAPEX_IOCCOM_ICC_OPEN           _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_ICC + 0,   FAPEX_ICC_OpenParamStrT)
#define FAPEX_IOCCOM_ICC_OPEN_SUBID     _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_SUBID_ICC, FAPEX_ICC_OpenSubIdParamStrT)
#define FAPEX_IOCCOM_ICC_SET_IOPORT     _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_ICC + 1,   FAPEX_ICC_IOportParamStrT)
#define FAPEX_IOCCOM_ICC_GET_IOPORT     _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_ICC + 1,   FAPEX_ICC_IOportParamStrT)
#define FAPEX_IOCCOM_ICC_GET_ATR        _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_ICC + 2,   FAPEX_ICC_AtrParamStrT)
#define FAPEX_IOCCOM_ICC_GET_DATA       _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_ICC + 3,   FAPEX_ICC_DataParamStrT)
#define FAPEX_IOCCOM_ICC_GET_STATUS     _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_ICC + 4,   uint32_t)
#define FAPEX_IOCCOM_ICC_GET_CARDNOTIFY _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_ICC + 5,   uint32_t)

/*
*************************************************************************
*************************************************************************
** prototype for application
*************************************************************************
*************************************************************************/
#ifndef __LINUX__
#include "fapex/abst_types.h"

#ifdef __cplusplus
extern "C" {
#endif

FAPEXOPENRET_T  FAPEX_ICC_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags, FAPEXOPENARG3_T argptr);
FAPEXRELRET_T   FAPEX_ICC_Close(FAPEXRELARG1_T id);
FAPEXREADRET_T  FAPEX_ICC_Read(FAPEXREADARG1_T id, FAPEXREADARG2_T buf, FAPEXREADARG3_T count);
FAPEXWRTRET_T   FAPEX_ICC_Write(FAPEXWRTARG1_T id, FAPEXWRTARG2_T buf, FAPEXWRTARG3_T count);
FAPEXIOCRET_T   FAPEX_ICC_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg);

#ifdef __cplusplus
}
#endif
#endif /* !__LINUX__ */

#endif /* !_FAPEX_ABST_LIBICC_H */

