/*!
*************************************************************************
**
** \file    fapex/abst_libir.h
**
** \brief   abstraction library header file (IR driver)
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#ifndef _FAPEX_ABST_LIBIR_H
#define _FAPEX_ABST_LIBIR_H

#include "fapex/abst_lib.h"
#include "fapex/abst_error.h"

/*
*************************************************************************
*************************************************************************
** Defines and Macros
*************************************************************************
*************************************************************************/

/*!
*************************************************************************
** Standard error codes for the driver
*************************************************************************/
/*! FAPex IR error code base value */
#define FAPEX_IR_ERR_BASE               (FAPEX_IR_MODULE_BASE + FAPEX_RET_ERR_BASE)

/*! Bad parameter passed. */
#define FAPEX_IR_ERR_BAD_PARAMETER      (FAPEX_IR_ERR_BASE + FAPEX_ERR_BAD_PARAMETER)

/*! Memory allocation failed. */
#define FAPEX_IR_ERR_OUT_OF_MEMORY      (FAPEX_IR_ERR_BASE + FAPEX_ERR_OUT_OF_MEMORY)

/*! Timeout occured. */
#define FAPEX_IR_ERR_TIMEOUT            (FAPEX_IR_ERR_BASE + FAPEX_ERR_TIMEOUT)

/*! Invalid status. */
#define FAPEX_IR_ERR_INVALID_STATUS     (FAPEX_IR_ERR_BASE + FAPEX_ERR_INVALID_HANDLE)

/*
****************************************************************************
** RCM KEY SPACE.
** The RCM keyspace is set from 0x0080 to 0x009F which is in ASCII a reserved
** character space. This is to avoid character overlapping with existing.
****************************************************************************
*/

/* terminal codes and character fonts */
#define FAPEX_IR_KEY_UNDEFINED     0x0000   // In case no key available

/* for ASCII compatibility: selection key set 0x08-0x0D */
#define FAPEX_IR_KEY_BACK          0x0008   /* backspace */
#define FAPEX_IR_KEY_TAB           0x0009
#define FAPEX_IR_KEY_OK            0x000D   /* enter */

/* for ASCII compatibility: numeric key set 0 to 9 [0x30 to 0x39] */
#define FAPEX_IR_KEY_0             0x0030
#define FAPEX_IR_KEY_1             0x0031
#define FAPEX_IR_KEY_2             0x0032
#define FAPEX_IR_KEY_3             0x0033
#define FAPEX_IR_KEY_4             0x0034
#define FAPEX_IR_KEY_5             0x0035
#define FAPEX_IR_KEY_6             0x0036
#define FAPEX_IR_KEY_7             0x0037
#define FAPEX_IR_KEY_8             0x0038
#define FAPEX_IR_KEY_9             0x0039

/* system control key */
#define FAPEX_IR_KEY_STANDBY       0x0080
#define FAPEX_IR_KEY_EXIT          0x0081
#define FAPEX_IR_KEY_TEXT          0x0082
#define FAPEX_IR_KEY_APPL          0x0083   /* OPT */
#define FAPEX_IR_KEY_MENU          0x0084
#define FAPEX_IR_KEY_HIDE          0x0085
#define FAPEX_IR_KEY_NAVI          0x0086   /* EPG */
#define FAPEX_IR_KEY_TV            0x0087
#define FAPEX_IR_KEY_CHANNEL_UP    0x0088
#define FAPEX_IR_KEY_CHANNEL_DOWN  0x0089
#define FAPEX_IR_KEY_INFO          0x008A
#define FAPEX_IR_KEY_VCR           0x008B

/* sound key set */
#define FAPEX_IR_KEY_MUTE          0x008C
#define FAPEX_IR_KEY_VOL_UP        0x008D
#define FAPEX_IR_KEY_VOL_DOWN      0x008E

/* cursor key set */
#define FAPEX_IR_KEY_RIGHT         0x0090
#define FAPEX_IR_KEY_LEFT          0x0091
#define FAPEX_IR_KEY_UP            0x0092
#define FAPEX_IR_KEY_DOWN          0x0093

/* VCR/PVR key set */
#define FAPEX_IR_KEY_PLAY          0x0094
#define FAPEX_IR_KEY_REC           0x0095
#define FAPEX_IR_KEY_STOP          0x0096
#define FAPEX_IR_KEY_PAUSE         0x0097
#define FAPEX_IR_KEY_REWIND        0x0098
#define FAPEX_IR_KEY_FORWARD       0x0099
#define FAPEX_IR_KEY_FORWSLOW      0x009E
#define FAPEX_IR_KEY_REWSLOW       0x009F

/* colour key set */
#define FAPEX_IR_KEY_RED           0x009A
#define FAPEX_IR_KEY_GREEN         0x009B
#define FAPEX_IR_KEY_YELLOW        0x009C
#define FAPEX_IR_KEY_BLUE          0x009D

/* extendet function key set */
#define FAPEX_IR_KEY_PIPSELECT     0x00A0
#define FAPEX_IR_KEY_F1            0x00A1
#define FAPEX_IR_KEY_F2            0x00A2
#define FAPEX_IR_KEY_F3            0x00A3
#define FAPEX_IR_KEY_F4            0x00A4
#define FAPEX_IR_KEY_PIPSWAP       0x00A5

/*
*************************************************************************
*************************************************************************
** Data types
*************************************************************************
*************************************************************************/

/*!
*************************************************************************
** \brief open parameters for the IR driver
**
** \sa FAPEX_IR_Open
*************************************************************************/
typedef struct {
    /*!
    **    IR protocol mode<br>
    **    \param #FAPEX_IR_MODE_NONE no protocol mode
    **    \param #FAPEX_IR_MODE_RC5  RC5 protocol mode
    **    \param #FAPEX_IR_MODE_NEC  NEC protocol mode
    */
    uint32_t  mode;

} FAPEX_IR_OpenParamStrT;

#define FAPEX_IR_MODE_NONE  0    //!< no protocol mode
#define FAPEX_IR_MODE_RC5   1    //!< RC5 protocol mode
#define FAPEX_IR_MODE_NEC   2    //!< NEC protocol mode

/*!
*************************************************************************
** \brief key parameters for the IR driver
**
** The related ioctl command type :
** \ref ir_cmd_get_key "FAPEX_IOCCOM_IR_GET_KEY"
**
** \sa FAPEX_IR_Ioctl
*************************************************************************/
typedef struct {
    /*!
        device ID to be received from IR device
    */
    int32_t    deviceId;
    /*!
        key value to be received from IR device
    */
    uint32_t   keyValue;
    /*!
        the mapped key value
    */
    uint32_t   rcmCode;
    /*!
        repeat flag to be received from IR device
    */
    uint32_t   repeatFlag;

} FAPEX_IR_KeyParamStrT;

/*!
*************************************************************************
** \brief Struct of the mapping table.
*************************************************************************
*/
typedef struct
{
    /*!
        Key value which is received by the protocol (NEC/RC5/...)
    */
    uint32_t             keyValue;
    /*!
       Remote key mapping code
       The mapped keyspace is set from 0x0080 to 0x009F which is in ASCII a reserved
       character space. This is to avoid character overlapping with existing.
    */
    uint32_t             rcmCode;

} FAPEX_IR_KeycodeMapStrT;

/*!
*************************************************************************
** \brief map table parameters for the IR driver
**
** The related ioctl command type :
** \ref ir_cmd_set_maptable "FAPEX_IOCCOM_IR_SET_MAPTABLE"
**
** \sa FAPEX_IR_Ioctl
*************************************************************************/
typedef struct {
    /*!
        device ID to be received from IR device
    */
    int32_t                 deviceId;
    /*!
        Keycode Map Table data
    */
    FAPEX_IR_KeycodeMapStrT *pKeycodeTable;
    /*!
        Number of Keycode Map Table (include last end identification data)
    */
    uint32_t                keycodeNum;

} FAPEX_IR_MapTableParamStrT;

/*!
*************************************************************************
** \brief status for the IR driver
**
** The related ioctl command type :
** \ref ir_cmd_get_status "FAPEX_IOCCOM_IR_GET_STATUS"
**
** \sa FAPEX_IR_Ioctl
*************************************************************************/
typedef struct {
    /*!
        FAPI_IR_GetStatus() value
    */
    int32_t   status;

} FAPEX_IR_StatusStrT;


/*!
*************************************************************************
** \brief ioctl specific command for the IR driver
*************************************************************************/
#define FAPEX_IOCCOM_IR_OPEN            _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_IR + 0, FAPEX_IR_OpenParamStrT)
#define FAPEX_IOCCOM_IR_GET_STATUS      _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_IR + 1, FAPEX_IR_StatusStrT)
#define FAPEX_IOCCOM_IR_GET_KEY         _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_IR + 2, FAPEX_IR_KeyParamStrT)
#define FAPEX_IOCCOM_IR_SET_MAPTABLE    _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_IR + 3, FAPEX_IR_MapTableParamStrT)
#define FAPEX_IOCCOM_IR_REMOVE_MAPTABLE _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_IR + 4, int32_t)


/*
*************************************************************************
*************************************************************************
** prototype for application
*************************************************************************
*************************************************************************/
#ifndef __LINUX__
#include "fapex/abst_types.h"

#ifdef __cplusplus
extern "C" {
#endif

FAPEXOPENRET_T FAPEX_IR_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags, FAPEXOPENARG3_T argptr);
FAPEXRELRET_T  FAPEX_IR_Close(FAPEXRELARG1_T id);
FAPEXREADRET_T FAPEX_IR_Read(FAPEXREADARG1_T id, FAPEXREADARG2_T buf, FAPEXREADARG3_T count);
FAPEXIOCRET_T  FAPEX_IR_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg);

#ifdef __cplusplus
}
#endif
#endif /* !__LINUX__ */

#endif /* !_FAPEX_ABST_LIBIR_H */

