/*!
*************************************************************************
**
** \file    fapex/abst_libpv.h
**
** \brief   abstraction library header file (PV driver)
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/


#ifndef _FAPEX_ABST_LIBPV_H
#define _FAPEX_ABST_LIBPV_H

#include "fapex/abst_lib.h"
#include "fapex/abst_error.h"

/*
*************************************************************************
*************************************************************************
** Defines and Macros
*************************************************************************
*************************************************************************/

#define FAPEX_PV_SIPROM_BURN_RETRY_NUM        16

/*
*************************************************************************
** Standard error codes for the driver
*************************************************************************/

/*! FAPex PV error code base value */

/*! FAPex PV error code base value */
#define FAPEX_PV_ERR_BASE            (FAPEX_PV_MODULE_BASE + FAPEX_RET_ERR_BASE)

/*!Bad parameter passed. */
#define FAPEX_PV_ERR_BAD_PARAMETER (FAPEX_PV_ERR_BASE+FAPEX_ERR_BAD_PARAMETER)

/*!Memory allocation failed. */
#define FAPEX_PV_ERR_OUT_OF_MEMORY (FAPEX_PV_ERR_BASE+FAPEX_ERR_OUT_OF_MEMORY)

/*!Device already initialised. */
#define FAPEX_PV_ERR_ALREADY_INITIALIZED (FAPEX_PV_ERR_BASE+FAPEX_ERR_ALREADY_INITIALIZED)

/*! Device not initialised. */
#define FAPEX_PV_ERR_NOT_INITIALIZED (FAPEX_PV_ERR_BASE+FAPEX_ERR_NOT_INITIALIZED)

/*!Feature or function is not available. */
#define FAPEX_PV_ERR_FEATURE_NOT_SUPPORTED (FAPEX_PV_ERR_BASE+FAPEX_ERR_FEATURE_NOT_SUPPORTED)

/*! Timeout occured. */
#define FAPEX_PV_ERR_TIMEOUT (FAPEX_PV_ERR_BASE+FAPEX_ERR_TIMEOUT)

/*! The device is busy, try again later. */
#define FAPEX_PV_ERR_DEVICE_BUSY (FAPEX_PV_ERR_BASE+FAPEX_ERR_DEVICE_BUSY)

/*! Invalid handle was passed. */
#define FAPEX_PV_ERR_INVALID_HANDLE (FAPEX_PV_ERR_BASE+FAPEX_ERR_INVALID_HANDLE)

/*! Semaphore could not be created. */
#define FAPEX_PV_ERR_SEMAPHORE_CREATE (FAPEX_PV_ERR_BASE+FAPEX_ERR_SEMAPHORE_CREATE)

/*! The driver's used version is not supported. */
#define FAPEX_PV_ERR_UNSUPPORTED_VERSION (FAPEX_PV_ERR_BASE+FAPEX_ERR_UNSUPPORTED_VERSION)

/*! The mailqueue could not be created. */
#define FAPEX_PV_ERR_MAILQUEUE_CREATE (FAPEX_PV_ERR_BASE + FAPEX_ERR_MAILQUEUE_CREATE)

/*! The pv have not been opend. */
#define FAPEX_PV_ERR_NOT_OPEN (FAPEX_PV_ERR_BASE + FAPEX_ERR_NOT_OPEN)

/*! The pv have been opened. */
#define FAPEX_PV_ERR_ALREADY_OPEN (FAPEX_PV_ERR_BASE + FAPEX_ERR_ALREADY_OPEN)

/*! The passed SiPROM word index is out of range. */
#define FAPEX_PV_ERR_OUT_OF_RANGE (FAPEX_PV_ERR_BASE - 101)

/*! The passed PV word index points to a hidden address.  */
#define FAPEX_PV_ERR_HIDDEN_ADDRESS (FAPEX_PV_ERR_BASE - 102)

/*! Burning SiPROM data failed.  */
#define FAPEX_PV_ERR_BURN_DATA (FAPEX_PV_ERR_BASE - 103)

/*
*************************************************************************
*************************************************************************
** Data types
*************************************************************************
*************************************************************************/

/*!
*************************************************************************
** \brief open parameters for the PV driver
**
** \sa FAPEX_PV_Ioctl 
*************************************************************************/
typedef struct {
    uint32_t wordIndex; //!< The index of the PV data word
    uint32_t bitMask;   //!< A bit mask
    uint32_t data;  //!< for read return data only, when write, no meaning
} FAPEX_PV_RWParamStrT;

typedef struct {
    uint32_t sectorIndex;   //!< The sector of the SiPROM PV boot to write
    FAPEX_PV_RWParamStrT baseParam;  //!< basic read wirte parameter
} FAPEX_PV_RWBootParamStrT;

typedef struct {
    uint32_t wordIndex : 16; //!< The word index within OTP space 0..78
    uint32_t startBit  :  8; //!< The index of 1st bit in bit field
    uint32_t bitCount  :  8; //!< The number of bits in bit field
    uint32_t bitMask   : 32; //!< The bit mask for data access
} FAPEX_PV_BitFieldStrT;

typedef struct {
    FAPEX_PV_BitFieldStrT bitField;  //!< The associated bitfield
    uint32_t data;  //!< for both read and write bit field
} FAPEX_PV_RWBitFieldParamStrT;

typedef struct {
   char aBitfieldName[256]; //!< The bitfield name
   FAPEX_PV_BitFieldStrT bitField;  //!< The associated bitfield
} FAPEX_PV_BitFieldXrefStrT;

typedef struct {
    uint32_t burnCount;
    uint32_t burnRetryCountArray[FAPEX_PV_SIPROM_BURN_RETRY_NUM];
} FAPEX_PV_BurnCountStrT;

/*!
*************************************************************************
** \brief ioctl specific command for the PV driver
*************************************************************************/
// IO
#define FAPEX_IOCCOM_PV_OPEN _IO(FAPEX_DEV1_TYPE, FAPEX_IOCCOM_NB_PV + 0)
#define FAPEX_IOCCOM_PV_RESETBURNCOUNTS   _IO(FAPEX_DEV1_TYPE, FAPEX_IOCCOM_NB_PV + 1)
// IOR
#define FAPEX_IOCCOM_PV_GETBURNCOUNT _IOR(FAPEX_DEV1_TYPE, FAPEX_IOCCOM_NB_PV + 0, FAPEX_PV_BurnCountStrT)
// IOWR
#define FAPEX_IOCCOM_PV_READDATA      _IOWR(FAPEX_DEV1_TYPE, FAPEX_IOCCOM_NB_PV + 0, FAPEX_PV_RWParamStrT)
#define FAPEX_IOCCOM_PV_READSIPROMDATA _IOWR(FAPEX_DEV1_TYPE, FAPEX_IOCCOM_NB_PV + 1, FAPEX_PV_RWParamStrT)
#define FAPEX_IOCCOM_PV_READSIPROMBOOT _IOWR(FAPEX_DEV1_TYPE, FAPEX_IOCCOM_NB_PV + 2, FAPEX_PV_RWBootParamStrT)
#define FAPEX_IOCCOM_PV_READBITFIELD _IOWR(FAPEX_DEV1_TYPE, FAPEX_IOCCOM_NB_PV + 3, FAPEX_PV_RWBitFieldParamStrT)
#define FAPEX_IOCCOM_PV_GETBITFIELDBYNAME _IOWR(FAPEX_DEV1_TYPE, FAPEX_IOCCOM_NB_PV + 4, FAPEX_PV_BitFieldXrefStrT)
// IOW
#define FAPEX_IOCCOM_PV_WRITEDATA      _IOW(FAPEX_DEV1_TYPE, FAPEX_IOCCOM_NB_PV + 0, FAPEX_PV_RWParamStrT)
#define FAPEX_IOCCOM_PV_WRITESIPROMDATA  _IOW(FAPEX_DEV1_TYPE, FAPEX_IOCCOM_NB_PV + 1, FAPEX_PV_RWParamStrT)
#define FAPEX_IOCCOM_PV_WRITESIPROMBOOT _IOW(FAPEX_DEV1_TYPE, FAPEX_IOCCOM_NB_PV + 2, FAPEX_PV_RWBootParamStrT)
#define FAPEX_IOCCOM_PV_WRITEBITFIELD _IOW(FAPEX_DEV1_TYPE, FAPEX_IOCCOM_NB_PV + 3, FAPEX_PV_RWBitFieldParamStrT)

/*
*************************************************************************
*************************************************************************
** prototype for application
*************************************************************************
*************************************************************************/
#ifndef __LINUX__
#include "fapex/abst_types.h"

#ifdef __cplusplus
extern "C" {
#endif

FAPEXOPENRET_T FAPEX_PV_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags);
FAPEXRELRET_T  FAPEX_PV_Close(FAPEXRELARG1_T id);
FAPEXIOCRET_T  FAPEX_PV_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg);

#ifdef __cplusplus
}
#endif
#endif /* !__LINUX__ */

#endif /* !_FAPEX_ABST_LIBPV_H */

