/*!
*************************************************************************
**
** \file    fapex/abst_libssp.h
**
** \brief   abstraction library header file (SSP driver)
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#ifndef _FAPEX_ABST_LIBI2C_H
#define _FAPEX_ABST_LIBI2C_H

#include "fapex/abst_lib.h"
#include "fapex/abst_error.h"

/*
*************************************************************************
*************************************************************************
** Defines and Macros
*************************************************************************
*************************************************************************/

#define FAPEX_SSP0             0    //!< SSP0 hardware block
#define FAPEX_SSP1             1    //!< SSP1 hardware block

#define FAPEX_SSP_USE_POLLING  0    //!< SSP use polling mode for transfer.
#define FAPEX_SSP_USE_ISR      1    //!< SSP use ISR for transfer.

#define FAPEX_SSP_CLK_POLARITY_LOW        0    //!< Clock is stady state low value if no data is transmitted
#define FAPEX_SSP_CLK_POLARITY_HIGH       1    //!< Clock is stady state high value if no data is transmitted
#define FAPEX_SSP_CLK_PHASE_LOW           0    //!< Data is captured on the first clock edge of transition
#define FAPEX_SSP_CLK_PHASE_HIGH          1    //!< Data is captured on the second clock edge of transition

/*
*************************************************************************
** Standard error codes for the driver
*************************************************************************/
/*! FAPex I2C error code base value */
#define FAPEX_SSP_ERR_BASE            (int32_t)(FAPEX_SSP_MODULE_BASE + FAPEX_RET_ERR_BASE)

/*! Bad parameter passed. */
#define FAPEX_SSP_ERR_BAD_PARAMETER   (int32_t)(FAPEX_SSP_ERR_BASE + FAPEX_ERR_BAD_PARAMETER)

/*!  Memory allocation failed. */
#define FAPEX_SSP_ERR_OUT_OF_MEMORY   (int32_t)(FAPEX_SSP_ERR_BASE + FAPEX_ERR_OUT_OF_MEMORY)
/*! Device already initialised. */
#define FAPEX_SSP_ERR_ALREADY_INITIALIZED   (FAPEX_SSP_MODULE_BASE +\
                                            FAPEX_ERR_ALREADY_INITIALIZED)

/*! Device not initialised. */
#define FAPEX_SSP_ERR_NOT_INITIALIZED       (FAPEX_SSP_MODULE_BASE +\
                                            FAPEX_ERR_NOT_INITIALIZED)

/*! Feature or function is not available. */
#define FAPEX_SSP_ERR_FEATURE_NOT_SUPPORTED (FAPEX_SSP_MODULE_BASE +\
                                            FAPEX_ERR_FEATURE_NOT_SUPPORTED)

/*! Timeout occured. */
#define FAPEX_SSP_ERR_TIMEOUT               (FAPEX_SSP_MODULE_BASE +\
                                            FAPEX_ERR_TIMEOUT)

/*! The device is busy, try again later. */
#define FAPEX_SSP_ERR_DEVICE_BUSY           (FAPEX_SSP_MODULE_BASE +\
                                            FAPEX_ERR_DEVICE_BUSY)

/*! Invalid handle was passed. */
#define FAPEX_SSP_ERR_INVALID_HANDLE        (FAPEX_SSP_MODULE_BASE +\
                                            FAPEX_ERR_INVALID_HANDLE)

/*! Semaphore could not be created. */
#define FAPEX_SSP_ERR_SEMAPHORE_CREATE      (FAPEX_SSP_MODULE_BASE +\
                                            FAPEX_ERR_SEMAPHORE_CREATE)

/*! The driver's used version is not supported. */
#define FAPEX_SSP_ERR_UNSUPPORTED_VERSION   (FAPEX_SSP_MODULE_BASE +\
                                            FAPEX_ERR_UNSUPPORTED_VERSION)

/*************************************************************************
*************************************************************************
** Data types
*************************************************************************
*************************************************************************/
/*!
*************************************************************************
**
** \brief SSP frame format.
**
** This enumeration covers all possible frame formates of operation.
**
** \sa    FAPEX_SSP_SetFrameFormat()
**
*************************************************************************
*/
#define FAPEX_SSP_MOTOROLA_SPI_FORMAT 0    //!< Motorola SPI frame format.
#define FAPEX_SSP_TEXAS_FORMAT        1    //!< Texas Instrumenst frame format.
#define FAPEX_SSP_MICROWIRE_FORMAT    2    //!< National Smeiconductor Microwire format.

/*!
*************************************************************************
** Callback for Transfer.
**
** This
**
** \param status Transfer status >0 transfered bytes <0 some error code.
** \param optArgPtr Optional data pointer.
**
** \sa FAPEX_SSP_OpenParamsT
*************************************************************************
*/
typedef void (*FAPEX_SSP_FinischCalbackT)(int32_t status,
                                          void* optArgPtr );

/*!
*************************************************************************
** \brief open parameters for the SSP driver
**
** \sa FAPEX_SSP_Open
*************************************************************************/
typedef struct {
    /*!
        The SSP HW block to be used. #FAPEX_SSP0, #FAPEX_SSP1
    */
    uint32_t                    blockIndex;
    /*!
        Select the transfer mode #FAPEX_SSP_USE_ISR or #FAPEX_SSP_USE_POLLING.
    */
    uint32_t                    transferMode;
    /*!
        Transfer finish callback. This callback is only used in ISR tranfer mode.
        This callback is called via FAPI_SSP_Bsr().
    */
    FAPEX_SSP_FinischCalbackT    finishCallback;
    /*!
        Optional data pointer for finish callback.
    */
    void*                       optArgPtr;

} FAPEX_SSP_OpenParamStrT;
/*!
*************************************************************************
** \brief read parameters for the SSP driver
**
** The related ioctl command type :
** \ref ssp_cmd_read  "FAPEX_IOCCOM_SSP_READ"
**
** \sa FAPEX_SSP_Ioctl
*************************************************************************/
typedef struct {
        void*     rxDataPtr;       // Pointer to data to be received from SSP device.
        uint32_t  rxDataLength;//Length of data to be received from an SSP device.
} FAPEX_SSP_RParamStrT;
/*!
*************************************************************************
** \brief write parameters for the SSP driver
**
** The related ioctl command type :
** \ref ssp_cmd_write  "FAPEX_IOCCOM_SSP_WRITE"
**
** \sa FAPEX_SSP_Ioctl
*************************************************************************/
typedef struct {
        void*     txDataPtr;       // Pointer to data to be transmitted to SSP device.
        uint32_t  txDataLength;//Length of data to be transmitted.
} FAPEX_SSP_WParamStrT;
/*!
*************************************************************************
** \brief getdata parameters for the SSP driver
**
** The related ioctl command type :
** \ref ssp_cmd_getdata  "FAPEX_IOCCOM_SSP_GETDATA"
**
** \sa FAPEX_SSP_Ioctl
*************************************************************************/
typedef struct {
        void*     txDataPtr;       // Pointer to data to be transmitted to SSP device.
        uint32_t  txDataLength;//Length of data to be transmitted.
        void*     rxDataPtr;       // Pointer to data to be received from SSP device.
        uint32_t  rxDataLength;//Length of data to be received from an SSP device.
} FAPEX_SSP_GetDataParamStrT;

/*!
*************************************************************************
** \brief speed parameters for the SSP driver
**
** The related ioctl command type :
** \ref ssp_cmd_set_speed  "FAPEX_IOCCOM_SSP_SETSPEED"
**
** \sa FAPEX_SSP_Ioctl
*************************************************************************/
typedef struct {
        uint32_t  serialClock;   // The serial clock this value can set to 0
        uint32_t  prescale;//The prescaler possible value >1 <255
} FAPEX_SSP_SpeedParamStrT;

/*
*************************************************************************
  **\brief Set the clock speed of the SSP driver
  **
  **  It is necessary to set the frame format for the used slave device.
  **  The value prescaler and serialClock was used to define the bitrate
  **  of the SSP comunication it was calculate by
  **  <b> bitrate=SystemClock/(prescale*(1+serialClock)) </b>
  **
  ** The related ioctl command type :
  ** \ref ssp_cmd_set_frameformat  "FAPEX_IOCCOM_SSP_SETFRAMEFORMAT"
  **\sa FAPEX_SSP_Ioctl
  **
  *************************************************************************
  */
  typedef struct
  {
      /*
          Defines the Frame Format of the SSP driver.
          Posible frame formats are:
                        - #FAPEX_SSP_MOTOROLA_SPI_FORMAT (default)
                        - #FAPEX_SSP_TEXAS_FORMAT
                        - #FAPEX_SSP_MICROWIRE_FORMAT
      */
      uint32_t               format;
      /*
          Specifies the data size. Possible values are 4 to 16 Bit
      */
      uint32_t               dataSize;
      /*!
          Specifies the Polarity of SPI Frame.
          Possible values are:
                   - #FAPEX_SSP_CLK_POLARITY_LOW (default)
                   - #FAPEX_SSP_CLK_POLARITY_HIGH
          This value only used in SPI Frame.
      */
      uint32_t               polarity;
      /*!
          Specifies the Phase of SPI Frame.
          Possible values are:
                   - #FAPEX_SSP_CLK_PHASE_LOW (default)
                   - #FAPEX_SSP_CLK_PHASE_HIGH
          This value only used in SPI Frame.
      */
      uint32_t               phase;
      /*!
          Clock prescale divisor is necessery for calculating the bitrate value.
          Must be an even number from 2 to 254
      */
      uint32_t               prescale;
      /*!
          The value serialClock is used to generate
          the transmit and receive bit rate.
          Must be an even number from 0 to 255.
          The bit rate was be callculated by <b> bitrate=99MHz/(prescaler*(1+serialClock)) </b>
      */
      uint32_t               serialClock;
  }
  FAPEX_SSP_FrameFormatT;
  /*!
  *************************************************************************
  ** \brief ioctl specific command for the SSP driver
  *************************************************************************/
#define FAPEX_IOCCOM_SSP_OPEN               _IOW(FAPEX_DEV1_TYPE, FAPEX_IOCCOM_NB_SSP + 0, FAPEX_SSP_OpenParamStrT)
#define FAPEX_IOCCOM_SSP_READ               _IOR(FAPEX_DEV1_TYPE, FAPEX_IOCCOM_NB_SSP + 1, FAPEX_SSP_RParamStrT)
#define FAPEX_IOCCOM_SSP_WRITE              _IOW(FAPEX_DEV1_TYPE, FAPEX_IOCCOM_NB_SSP + 1, FAPEX_SSP_WParamStrT)
#define FAPEX_IOCCOM_SSP_SET_FRAMEFORMAT    _IOW(FAPEX_DEV1_TYPE, FAPEX_IOCCOM_NB_SSP + 2, int32_t)
#define FAPEX_IOCCOM_SSP_SET_SPEED          _IOW(FAPEX_DEV1_TYPE, FAPEX_IOCCOM_NB_SSP + 3, int32_t)
#define FAPEX_IOCCOM_SSP_GETDATA            _IOR(FAPEX_DEV1_TYPE, FAPEX_IOCCOM_NB_SSP + 4, FAPEX_SSP_GetDataParamStrT)
/*
*************************************************************************
*************************************************************************
** prototype for application
*************************************************************************
*************************************************************************/
#ifndef __LINUX__
#include "fapex/abst_types.h"

#ifdef __cplusplus
extern "C" {
#endif

FAPEXOPENRET_T FAPEX_SSP_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags, FAPEXOPENARG3_T argptr);
FAPEXRELRET_T  FAPEX_SSP_Close(FAPEXRELARG1_T id);
FAPEXIOCRET_T  FAPEX_SSP_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg);

#ifdef __cplusplus
}
#endif
#endif /* !__LINUX__ */

#endif /* !_FAPEX_ABST_LIBSSP_H */

